﻿using System;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using System.Web.Security;

public partial class AdminEditUsersRoles : BasePage
{
    #region Initialize Variables
    private int _adminRoleId;
    private int _facRoleId;
    private int _natRptRoleId;
    private int _visnRptRoleId;
    private int _facRptRoleId;
    private string _userId;

    private string _currentRegistry = Helpers.GetConfigEntry("Registry");
    private UserAccountManager _currentUser = (UserAccountManager)HttpContext.Current.User;
    AccessControl ac = new AccessControl();
    USER userEntry;
    UsersManager usersManager = new UsersManager();

    #endregion

    #region Public Properties
    public AdminEditUsersRoles()
    {
        //Manager Save Notification Event Handler
        usersManager.BOSaveSuccess += new BaseBO.BOSaveSuccessEvent(manager_BOSaveSuccess);
    }
    #endregion

    #region Page Events
    protected void Page_Load(object sender, EventArgs e)
    {
        //set the user ID variable
        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;
        _userId = _user.UserId.ToString();

        if (Helpers.checkAdminAccess())
        {
            HiddenFieldRegistryName.Value = _currentRegistry;
            SetRoleIds();

            if (!IsPostBack)
            {
                try
                {
                    LabelBreadCrumb.Text = "Breast Care Registry Administration > Edit User Roles";
                    HiddenFieldBreadCrumbBaseText.Value = LabelBreadCrumb.Text;

                    //if page is loading for first time get ID from querystring and set to hidden field to use on postbacks
                    int? userID = Convert.ToInt32(Request.QueryString["id"]);
                    HiddenFieldUserID.Value = userID.GetValueOrDefault().ToString();

                    //if userID is not supplied or zero then display a list of all user roles
                    if (userID == null || userID == 0)
                    {
                        userID = 0;

                        //set page display for list of all user roles
                        SetFormDisplay(false);

                    }
                    else
                    {
                        //set page to display for single user role management
                        SetFormDisplay(true);

                    }
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_userId, ex);
                }
            }

            try
            {
                if (HiddenFieldUserID.Value != "0")
                {
                    //Create User Entity object and Set properties to state of form
                    getEntry(Convert.ToInt32(HiddenFieldUserID.Value));
                }


                if (IsPostBack && userEntry != null)
                {
                    //saves changes after postback
                    LoadObjectFromControls();
                }
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            Response.Redirect("ErrorPage.aspx?id=100");
        }
    }
    void Page_PreRender(object sender, EventArgs e)
    {
        try
        {
            if (userEntry != null)
            {
                //This is the last event in the page life cycle where changes can be made to the page object
                //so enforce business rules on the object (which could possibly change the propertie of the object
                //then load the form controls from the object

                //Enforce business rules on the referral object
                //need to call EnforceBusinessRules method in the ReferralManager because some rules require a database call
                //pass in the local instance of referral to the method.  The method will return the object with modifications due to business rules.
                if (IsPostBack)
                {
                    int ret;
                    ret = usersManager.EnforceBusinessRules(ref userEntry);
                }

                LoadControlsFromObject();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }
    #endregion

    #region Button Events

    /***No save is actually completed here.  The save is completed after postback***/
    protected void ButtonSave_Click(object sender, EventArgs e)
    {
        try
        {
            int returnSaveValue = 0;

           // AdhocReportingUserEditError saveAdhocReportsErrorCode = SaveUserToReportingServer();

            returnSaveValue = usersManager.Update(userEntry);
            this.IsSaveSuccess = (returnSaveValue > 0);

            //if (saveAdhocReportsErrorCode == AdhocReportingUserEditError.None)
            //{
            //    this.SuppressSaveMessage = false;
            //}
            //else
            //{
            //    this.SuppressSaveMessage = true;

            //    switch (saveAdhocReportsErrorCode)
            //    {
            //        case AdhocReportingUserEditError.AdminUserInfoIncorrect:
            //            Alert.Show("Failed to add the user to Report Builder.  Report Server Administrator credentials in the configuration file are incorrect.  Please notify administrator.");
            //            break;
            //        case AdhocReportingUserEditError.NoDomainInfo:
            //            Alert.Show("Domain name for this user has not been provided.  Please navigate to the User Account Information page to add it.");
            //            break;
            //        case AdhocReportingUserEditError.NoAdhocReportsFolderInfo:
            //            Alert.Show("Failed to add the user to Report Builder.  Ad Hoc Reports Folder in the configuration file is not specified.  Please notify administrator.");
            //            break;
            //        default:
            //            Alert.Show("Failed to add the user to Report Builder.  Please notify administrator.");  
            //            break;
            //    }
            //}

            //Set Broken Rule Error Display
            if (userEntry.BrokenRules.Count == 0)
            {
                ErrorList1.Visible = false;
            }
            else
            {
                ErrorList1.Visible = true;
            }

            ErrorList1.BrokenRules = userEntry.BrokenRules;

            //TO DO
            //Display Save Message

            //Refresh entity root data
            if (returnSaveValue != 0)
            {
                getEntry(Convert.ToInt32(returnSaveValue));
                var master = (RegistryMaster)Master;
                master.SetDirtyFlag(false);
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    protected void ButtonClose_Click(object sender, EventArgs e)
    {
        Response.Redirect("AdminListUsers.aspx");
    }

    protected void ButtonClear_Click(object sender, EventArgs e)
    {
        this.TextBoxSearchCriteria.Text = "";
        SetFormDisplay(true);
    }

    protected void ButtonSearch_Click(object sender, EventArgs e)
    {
        try
        {
            userEntry = usersManager.SelectByUsername(TextBoxSearchCriteria.Text);

            if (userEntry != null)
            {
                GridViewUserRoles.DataSourceID = "ObjectDataSourceUsersRoles";
                GridViewUserRoles.DataBind();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }

    #endregion

    #region Object/Form Mapping

    private void LoadControlsFromObject()
    {
        #region Profile Fields
        //Registry
        HiddenFieldRegistryName.Value = _currentRegistry;

        //Username:
        LabelUsername.Text = userEntry.USERNAME;

        //Domain:
        LabelDomain.Text = userEntry.DOMAIN;

        //Full Name:
        LabelFullName.Text = userEntry.FULL_NAME;

        //Employee Number:
        LabelEmployeeNumber.Text = userEntry.EMPLOYEE_NUMBER;

        //Job Title:
        LabelJobTitle.Text = userEntry.JOB_TITLE;

        //userid 
        HiddenFieldUserID.Value = userEntry.USER_ID.ToString();
        #endregion

        //User Roles
        SetPatientMetaDataRoles();
        SetReportingRoles();

    }

    private void LoadObjectFromControls()
    {
        try
        {
            //If Admin Role is not checked, remove the admin role
            if (!rbAdminRole.Checked)
            {
                userEntry.RemoveRoleFromUser(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_ADMIN")));
                UpdateUserRolesFromListBox(userEntry, ListBoxVISNsMetaData, Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_USER")));
            }
            else
            {
                userEntry.RemoveRoleFromUser(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_USER")));
                UpdateUserRolesFromListBox(userEntry, ListBoxVISNsMetaData, Convert.ToInt32(ac.GetRoleId("BCCCR_ADMIN")));
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

        /***Reporting Roles***/
        //If role is checked, add it and the associated VISN/facilities if applicable
        try
        {
            if (CheckBoxRoleReportingNational.Checked)
            {
                UpdateUserRoleNotVisnBased(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_NAT_RPT")));
            }
            else
            {
                userEntry.RemoveRoleFromUser(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_NAT_RPT")));
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

        try
        {
            if (CheckBoxRoleReportingVISN.Checked)
            {
                UpdateUserRolesFromListBox(userEntry, ListBoxReportingVISN, Convert.ToInt32(ac.GetRoleId("BCCCR_VISN_RPT")));
            }
            else
            {
                userEntry.RemoveRoleFromUser(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_VISN_RPT")));
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

        try
        {
            if (CheckBoxRoleReportingUserStation.Checked)
            {
                UpdateUserRolesFromListBox(userEntry, ListBoxReportingStation, Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_RPT")));
            }
            else
            {
                userEntry.RemoveRoleFromUser(userEntry, Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_RPT")));
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    private void UpdateUserRoleNotVisnBased(USER userEntity, int RoleID)
    {
        try
        {
            if (!userEntity.USER_ROLEs.Any(e => e.STD_ROLE_ID == RoleID && e.INACTIVE_FLAG == false))
            {
                USER_ROLE role = new USER_ROLE();
                role.STD_ROLE_ID = RoleID;
                role.STD_INSTITUTION_ID = STD_INSTITUTION.TopInstitutionId;
                userEntry.USER_ROLEs.Add(role);
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    private void UpdateUserRolesFromListBox(USER userEntity, ListBox ListBoxVISNsEditor, int RoleID)
    {
        try
        {
            foreach (ListItem checkboxItem in ListBoxVISNsEditor.Items)
            {
                //for each item in the check box list check if it already exists in the entityset
                USER_ROLE existingRole = (from e in userEntity.USER_ROLEs
                                          where e.STD_ROLE_ID == RoleID
                                          && e.STD_INSTITUTION_ID == Helpers.getIntFromString(checkboxItem.Value)
                                          && e.INACTIVE_FLAG == false
                                          select e).FirstOrDefault();

                //check if the checkboxlist item was selected,  
                if (checkboxItem.Selected)
                {
                    //if the item is selected then it needs to be in the set
                    //if the entity exists, that signifies the item is already in the set therefore nothing needs to be done. 
                    if (existingRole == null)
                    {
                        USER_ROLE newUserRole = new USER_ROLE();
                        newUserRole.STD_ROLE_ID = RoleID;
                        newUserRole.STD_INSTITUTION_ID = Helpers.getIntFromString(checkboxItem.Value);

                        userEntry.USER_ROLEs.Add(newUserRole);
                    }
                }
                else
                {
                    //if the item is not selected, then it needs to be marked as inactive
                    if (existingRole != null)
                    {
                        //existingRole.INACTIVE_FLAG = true;
                        existingRole.SetAsDeleteOnSubmit();
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #endregion

    #region Private Methods

    private void UnselectListBoxItems(ListBox listBox)
    {
        foreach (ListItem item in listBox.Items)
        {
            item.Selected = false;
        }
    }

    private void SetRoleIds()
    {
        try
        {
            _adminRoleId = Convert.ToInt32(ac.GetRoleId("BCCCR_ADMIN"));
            _facRoleId = Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_USER"));
            _natRptRoleId = Convert.ToInt32(ac.GetRoleId("BCCCR_NAT_RPT"));
            _visnRptRoleId = Convert.ToInt32(ac.GetRoleId("BCCCR_VISN_RPT"));
            _facRptRoleId = Convert.ToInt32(ac.GetRoleId("BCCCR_FAC_RPT"));
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    private void SetPatientMetaDataRoles()
    {
        try
        {
            ListBoxVISNsMetaData.DataBind();

            foreach (USER_ROLE userRole in userEntry.USER_ROLEs)
            {
                //patient metadata roles
                if (userRole.STD_ROLE_ID == _adminRoleId)
                {
                    rbAdminRole.Checked = true;
                }
                else if (userRole.STD_ROLE_ID == _facRoleId)
                {
                    rbFacilityRole.Checked = true;
                }

                ListItem foundListItem = ListBoxVISNsMetaData.Items.FindByValue(userRole.STD_INSTITUTION_ID.ToString());

                if (foundListItem != null)
                {
                    foundListItem.Selected = true;
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }

    private void SetReportingRoles()
    {
        try
        {
            ListBoxReportingVISN.DataBind();
            ListBoxReportingStation.DataBind();

            foreach (USER_ROLE userRole in userEntry.USER_ROLEs)
            {
                //dashboard default reporting roles
                if (userRole.STD_ROLE_ID == _natRptRoleId)
                {
                    CheckBoxRoleReportingNational.Checked = true;
                }
                else if (userRole.STD_ROLE_ID == _visnRptRoleId)
                {
                    CheckBoxRoleReportingVISN.Checked = true;

                    ListItem foundListItem = ListBoxReportingVISN.Items.FindByValue(userRole.STD_INSTITUTION_ID.ToString());

                    if (foundListItem != null)
                    {
                        foundListItem.Selected = true;
                    }
                }
                else if (userRole.STD_ROLE_ID == _facRptRoleId)
                {
                    CheckBoxRoleReportingUserStation.Checked = true;

                    ListItem foundListItem = ListBoxReportingStation.Items.FindByValue(userRole.STD_INSTITUTION_ID.ToString());

                    if (foundListItem != null)
                    {
                        foundListItem.Selected = true;
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }
    
    #endregion

    #region Protected Events
    protected void ButtonDownloadToExcel_Click(object sender, EventArgs e)
    {

        UsersManager usersManager = new UsersManager();

        try
        {
            int countAll = usersManager.SelectListUsersVISNsCount(HiddenFieldRegistryName.Value, "", "", 0, 0);

            GridViewUserRoles.PageSize = countAll;

            GridViewUserRoles.DataBind();

            PrepareGridViewForExport(GridViewUserRoles);

            ExportGridViewToExcel(GridViewUserRoles, "BCCCR User Roles.xls");
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }

    #endregion

    #region Protected Methods
    protected void getEntry(int id)
    {
        try
        {
            userEntry = usersManager.SelectByID(_currentRegistry, id);

            //initial display panels
            PanelNoAccess.Visible = false;
            PanelIdNotFound.Visible = false;
            PanelObjectContent.Visible = false;

            if (userEntry == null)
            {
                PanelObjectContent.Visible = false;

                if (usersManager.UserIDExists(id))
                {
                    PanelNoAccess.Visible = true;
                }
                else
                {
                    PanelIdNotFound.Visible = true;
                }
            }
            else
            {
                PanelObjectContent.Visible = true;
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }
    protected void SetFormDisplay(bool isSingleUserRoleManagement)
    {
        try
        {
            LabelBreadCrumb = (Label)Master.FindControl("LabelBreadCrumb");

            if (isSingleUserRoleManagement)
            {
                //set page to display for single user role management
                PanelListUserRoles.Visible = false;
                PanelRoleInformation.Visible = true;

                //Modify breadcrumb to display editing user id
                LabelBreadCrumb.Text = HiddenFieldBreadCrumbBaseText.Value + " > Edit Roles for User ID : " + HiddenFieldUserID.Value;

            }
            else
            {
                //set page display for list of all user roles
                PanelListUserRoles.Visible = true;
                PanelRoleInformation.Visible = false;

                //Modify breadcrumb to display adding new
                LabelBreadCrumb.Text = HiddenFieldBreadCrumbBaseText.Value + " > List all User / Roles ";
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }
    #endregion

    #region GUI Overrrides

    public override string MenuItemName()
    {
        return "BCCCR ADMIN USER ROLE";
    }

    public override string AppObjectCode()
    {
        return "BCCCR ADMIN USER ROLE";
    }

    public override string TopMenuName()
    {
        return "Administration";
    }

    #endregion

}

