﻿using System;
using System.Linq;
using Telerik.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI;


#region Custom Filtering Column Class
/// <summary>
/// Class for creating a custom filter for the GridBound Item.
/// </summary>
/// 
namespace CustomFilter
{
    public class CustomFilteringColumn : GridBoundColumn
    {
        private object listDataSource = null;
        //RadGrid calls this method when it initializes the controls inside the filtering item cells
        protected override void SetupFilterControls(TableCell cell)
        {
            base.SetupFilterControls(cell);
            cell.Controls.RemoveAt(0);
            Telerik.Web.UI.RadComboBox list = new RadComboBox();
            //DropDownList list = new DropDownList();
            list.ID = "list" + this.DataField;

            list.AutoPostBack = true;
            list.ToolTip = "Filter for " + this.DataField;
            list.SelectedIndexChanged += new RadComboBoxSelectedIndexChangedEventHandler(list_SelectedIndexChanged);
            cell.Controls.AddAt(0, list);
            cell.Controls.RemoveAt(1);
            list.Width = this.FilterControlWidth;
            list.DropDownWidth = Unit.Pixel(250);
            list.AutoCompleteSeparator = ",";
            list.EnableScreenBoundaryDetection = true;
            list.MarkFirstMatch = true;
            list.Skin = "Sunset";
            list.OffsetY = 1;
            list.DataTextField = this.DataField;
            list.DataValueField = this.DataField;
            list.DataSource = this.ListDataSource;

            list.Items.Clear();
            list.Items.Insert(0, new RadComboBoxItem("All"));
            list.AppendDataBoundItems = true;

            //list.DataBind();
            //list.Items.Insert(0, blankItem1);

        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the list control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void list_SelectedIndexChanged(object sender, EventArgs e)
        {
            GridFilteringItem filterItem = (sender as Telerik.Web.UI.RadComboBox).NamingContainer as GridFilteringItem;
            //if (this.DataType == System.Type.GetType("System.Int32") ||
            //    this.DataType == System.Type.GetType("System.Int16") ||
            //    this.DataType == System.Type.GetType("System.Int64"))
            //{
            filterItem.FireCommandEvent("Filter", new Pair("EqualTo", this.UniqueName));
            //}
            //else // treat everything else like a string
            //    filterItem.FireCommandEvent("Filter", new Pair("Contains", this.UniqueName));
        }
        public object ListDataSource
        {
            get { return this.listDataSource; }
            set { listDataSource = value; }
        }
        //RadGrid calls this method when the value should be set to the filtering input control(s)
        protected override void SetCurrentFilterValueToControl(TableCell cell)
        {
            base.SetCurrentFilterValueToControl(cell);
            Telerik.Web.UI.RadComboBox list = (Telerik.Web.UI.RadComboBox)cell.Controls[0];
            if (this.CurrentFilterValue != string.Empty)
            {
                list.SelectedValue = this.CurrentFilterValue;
            }
        }
        //RadGrid calls this method to extract the filtering value from the filtering input control(s)
        protected override string GetCurrentFilterValueFromControl(TableCell cell)
        {
            Telerik.Web.UI.RadComboBox list = (Telerik.Web.UI.RadComboBox)cell.Controls[0];
            return list.SelectedValue;
        }
        protected override string GetFilterDataField()
        {
            return this.DataField;
        }

    }

}

#endregion