﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.BusinessManager;

/// <summary>
/// Summary description for MenuBuilder
/// </summary>
/// 

public static class MenuBuilder
{

    /// <summary>
    /// Builds left menu based on top menu location and pages accessible by user
    /// </summary>
    /// <param name="horizontalMenu"></param>
    /// <param name="pageTopMenuItem"></param>
    /// <param name="leftMenu"></param>
    /// <param name="pageLeftMenuItem"></param>
    /// <param name="breadCrumb"></param>
    /// <param name="accessiblePageList"></param>
    public static void BuildDynamicNavigationItems(string currentURL, Menu horizontalMenu, string pageTopMenuItem, string pageLeftMenuItem, Label breadCrumb, string[] accessiblePageList)
    {
        //menu items that need name manipulated after breadcrumb is built
        MenuItem menuItemNewBCCCR = new MenuItem();
        int numNewItems = 0;
        MenuItem menuItemInProcessBCCCR = new MenuItem();
        int numInProcess = 0;

        switch (pageTopMenuItem)
        {

            case "Reporting":
                var parentStandardReports = new MenuItem("Standard Reports", "BCR_STANDARD_REPORTS");
                //*leftMenu.Items.Add(parentStandardReports);
                //Patient Data Reports
                parentStandardReports.ChildItems.Add(new MenuItem("Ordered Pending", "BCR_STANDARD_REPORTS", "", "~/Reports/OrderedPending_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms Due", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammogramsDue_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms w/BI-RADS", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammogramsWithBiRads_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Biopsies", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastBiopsies_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Cancer", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastCancer_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Surgeries", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastSurgeries_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Abnormal Mammograms", "BCR_STANDARD_REPORTS", "", "~/Reports/AllAbnormalMammograms_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms w/Follow Up", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammogramsWithFollowUp_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms w/Diagnosis", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammograms_ICD_PD.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms w/Results", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammogramsWithResults_PD.aspx"));

                //Agg Reports
                parentStandardReports.ChildItems.Add(new MenuItem("All Mammograms Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/AllMammograms_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Abnormal Mammograms Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/AbnormalMammograms_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Biopsies Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastBiopsies_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Cancer Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastCancer_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Breast Sugeries Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/BreastSurgeries_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Mammograms w/Follow-Up Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/MammogramsWithFollowUp_Agg.aspx"));
                parentStandardReports.ChildItems.Add(new MenuItem("Mammograms By Diagnosis Aggregate", "BCR_STANDARD_REPORTS", "", "~/Reports/MammogramsByDiagnosis_Agg.aspx"));

                //*leftMenu.Items.Add(new MenuItem("Ad Hoc Reports22", "BCR_ADHOC_REPORTING", "", "ReportAdhoc.aspx"));
                break;
            case "Dashboard":

                //*leftMenu.Items.Add(new MenuItem("", "BCCCR DASHBOARD", "", "Dashboard.aspx"));

                break;
            case "Administration":

                //leftMenu.Items.Add(new MenuItem("Users", "BCCCR ADMIN LIST USERS", "", "AdminListUsers.aspx"));
                //leftMenu.Items.Add(new MenuItem("Role Matrix", "BCCCR ADMIN USER ROLE", "", "AdminEditUsersRoles.aspx"));

                break;
            case "Demo":

                //*leftMenu.Items.Add(new MenuItem("", "Demo", "", "Demo.aspx"));
                //leftMenu.Items.Add(new MenuItem("Role Matrix", "BCCCR ADMIN USER ROLE", "", "AdminEditUsersRoles.aspx"));

                break;
            default:

                //leftMenu.Items.Add(new MenuItem("Top Menu Not Registered"));

                break;

        }

        //Disable all menu items
        //*foreach (MenuItem item in leftMenu.Items)
        //*    ChangeMenuState(item, false);

        //Enable selection for the accessible items only
        //*for (int i = 0; i < accessiblePageList.GetLength(0); i++)
        //*    ChangeMenuState(leftMenu.FindItem(accessiblePageList[i]), true);

        //SetMenuItemHighlight(leftMenu.Items, pageLeftMenuItem);

        BuildBreadCrumbDynamic(currentURL, pageTopMenuItem, pageLeftMenuItem, breadCrumb, accessiblePageList);

        menuItemNewBCCCR.Text += (numNewItems > 0 ? " (" + numNewItems.ToString() + ")" : "");
        menuItemInProcessBCCCR.Text += (numInProcess > 0 ? " (" + numInProcess.ToString() + ")" : "");

    }

    /// <summary>
    /// Builds bread crumb based item in selected menu
    /// </summary>
    /// <param name="horizontalMenu"></param>
    /// <param name="pageTopMenuItem"></param>
    /// <param name="leftMenu"></param>
    /// <param name="pageLeftMenuItem"></param>
    /// <param name="breadCrumb"></param>
    /// <param name="accessiblePageList"></param>
    public static void BuildBreadCrumbDynamic(string currentURL, string pageTopMenuItem, string pageLeftMenuItem, Label breadCrumb, string[] accessiblePageList)
    {

        MenuItem currentMenuItem = null;

        //if (leftMenu.Items.Count > 0)
        //{
        //    currentMenuItem = leftMenu.Items[0];
        //}
        //*foreach (MenuItem item in leftMenu.Items)
        //{
        //    currentMenuItem = SeachItemAndChildrenByURL(item, currentURL);
        //    if (currentMenuItem != null) break;
        //}

        //*if (currentMenuItem == null)
        //{
        //    currentMenuItem = leftMenu.FindItem(pageLeftMenuItem);
        //}


        if (currentMenuItem != null)
        {
            if (currentMenuItem.Selectable)
            {
                currentMenuItem.Selected = true;
            }

            //initialize breadcrumb with currently selected menu item
            string breadCrumbPath = currentMenuItem.Text;

            //add to path all parents of selected menu item
            while (currentMenuItem.Parent != null)
            {
                //set current menutree location to parent
                currentMenuItem = currentMenuItem.Parent;

                breadCrumbPath = currentMenuItem.Text + " > " + breadCrumbPath;
            }

            //add top menu to beginning of breadcrumb
            breadCrumbPath = pageTopMenuItem + " > " + breadCrumbPath;

            breadCrumb.Text = breadCrumbPath;
        }
    }

    public static MenuItem SeachItemAndChildrenByURL(MenuItem item, string URL)
    {
        MenuItem foundItem = null;

        string[] pathElems = HttpContext.Current.Request.Url.PathAndQuery.Split(new char[] { '/' });
        string pageName =
        pathElems[pathElems.Length - 1];

        //foreach (MenuItem elem in menu)
        //{
        if (item.NavigateUrl == pageName)
            return item;

        if (item.ChildItems.Count > 0)
        {
            foreach (MenuItem subitem in item.ChildItems)
            {
                foundItem = SeachItemAndChildrenByURL(subitem, pageName);
                if (foundItem != null)
                    return foundItem;
            }
        }

        return null;

    }

    /// <summary>
    /// Changes menu state recursively
    /// </summary>
    /// <param name="item"></param>
    /// <param name="flag"></param>
    private static void ChangeMenuState(MenuItem item, bool flag)
    {
        if (item != null)
        {
            if (flag == false) item.Selectable = flag;
            if (flag == true && item.ChildItems.Count == 0) item.Selectable = flag;
            if (item.ChildItems.Count > 0)
                foreach (MenuItem subitem in item.ChildItems)
                    ChangeMenuState(subitem, flag);
        }
    }

    /// <summary>
    /// Search and highlight appropriate menu item
    /// </summary>
    /// <param name="menu">Collection of Menu Items</param>
    /// <param name="value">Item to Search For</param>

    public static string GetDefaultPageForUserRole()
    {
        //AccessControl ac = new AccessControl();
        //UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        //int adminRole = Convert.ToInt32(ac.GetRoleId("BCCCR_ADMIN"));
        //int userId = _user.UserId;
        //if (ac.GetRoleByUserID(userId, adminRole))
        //{
            return ("~/Reports/Dashboard.aspx");
        //}
        //else
        //{
        //    return "demo.aspx";
        //}

    }

    /// <summary>
    /// This function sets the properties for my tasks menu item
    /// based on the user role.
    /// </summary>
    //public static void SetMyTasksLink(Menu horizontalMenu)
    //{
    //    string defaultPage = MenuBuilder.GetDefaultPageForUserRole();

    //    if (defaultPage == "") defaultPage = "Default.aspx";

    //    MenuItem myTaskMenuItem = horizontalMenu.FindItem("My Tasks");

    //    if (defaultPage == "DefaultReporting.aspx")
    //    {
    //        myTaskMenuItem.NavigateUrl = "";
    //        myTaskMenuItem.Selectable = false;
    //    }
    //    else
    //    {
    //        myTaskMenuItem.NavigateUrl = defaultPage;
    //    }
    //}

    /// <summary>
    /// This function sets the properties for patient menu item
    /// based on the user role.
    /// </summary>
    //public static void SetPatientLink(Menu horizontalMenu)
    //{
    //    UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    //    string patientMenuPage = "";

    //    //if (_user.GetRoles().Contains((int)RoleCode.TBI_ADMINISTRATOR)
    //    //        ||
    //    //    _user.GetRoles().Contains((int)RoleCode.TBI_EDITOR)
    //    //        ||
    //    //    _user.GetRoles().Contains((int)RoleCode.TBI_READ_ONLY))
    //    //{
    //        patientMenuPage = "Patients.aspx";
    //    //}
    //    //else if (_user.GetRoles().Contains((int)RoleCode.TBI_POLYTRAUMA_EDITOR)
    //    //            ||
    //    //        _user.GetRoles().Contains((int)RoleCode.TBI_POLYTRAUMA_READ_ONLY))
    //    //{
    //    //    patientMenuPage = "PolyTraumaPatients.aspx";
    //    //}

    //    MenuItem myTaskMenuItem = horizontalMenu.FindItem("Patients");

    //    myTaskMenuItem.NavigateUrl = patientMenuPage;

    //    //if (patientMenuPage == "")
    //    //{
    //    //    myTaskMenuItem.Selectable = false;
    //    //}
    //}


}
