﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml.Linq;
using System.Xml.Serialization;
using Telerik.Web.UI;

/// <summary>
/// Summary description for AdHocReports
/// </summary>
public class AdHocReports
{
    private const string CONNECTION_NAME = "RegistryConnectionString";
    private const string ADHOC_SEARCH = "bcccr.usp_Adhoc_Search";

    private const string ADHOC_INSERT_UPDATE_REPORT = "bcccr.usp_Adhoc_Insert";
    private const string ADHOC_DELETE_REPORT = "bcccr.usp_Adhoc_Delete";
    private const string ADHOC_LOAD_ALL_USER_REPORT_TEMPLATES = "bcccr.usp_Adhoc_Load_All_User_Reports";
    private const string ADHOC_LOAD_SINGLE_USER_REPORT_TEMPLATE = "bcccr.usp_Adhoc_Load_User_Report";
    private const string ADHOC_UPDATE_LAST_EXEC_DATE = "bcccr.usp_Adhoc_Last_Exec_Update";

    public List<AdHocTemplateDisplay> ReportColumnList { get; set; }

	public AdHocReports()
    {
        #region Create ReportColumnList

        ReportColumnList = new List<AdHocTemplateDisplay>();

        /*
         * Rules of processing
         * 
         * All columns from BCCCR.BCR_ALL will be prefaced with bcrall. in the value
         * 
         * All columns from BCCCR.RadiologyMammos will be prefaced with radmam. in the value
         * 
         * All columns from BCCCR.Oncology will be prefaced with onc. in the value
         */

        #region BCR_ALL Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MonthSID as [bcr.MonthSID]", DisplayValue = "MonthSID", InstructionText = "" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientSID as [bcr.PatientSID]", DisplayValue = "PatientSID" }); // always present
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientICN as [bcr.PatientICN]", DisplayValue = "PatientICN", InstructionText = "Enter whole or partial ICN" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientIEN as [bcr.PatientIEN]", DisplayValue = "PatientIEN", InstructionText = "Enter whole or partial IEN" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientSSN as [bcr.PatientSSN]", DisplayValue = "PatientSSN", InstructionText = "Enter whole or partial SSN" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Sta3n as [bcr.Sta3n]", DisplayValue = "Sta3n" }); // always present
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.VAID as [bcr.VAID]", DisplayValue = "VAID", InstructionText = "Enter First initial of last name and last 4 of SSN (i.e. V1111)" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientLastName as [bcr.PatientLastName]", DisplayValue = "Patient Last Name", InstructionText = "Enter whole or partial last name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PatientFirstName as [bcr.PatientFirstName]", DisplayValue = "Patient First Name", InstructionText = "Enter whole or partial first name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfBirth as [bcr.DateOfBirth]", DisplayValue = "Date Of Birth", InstructionText = "Select a date from date picker" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Age as [bcr.Age]", DisplayValue = "Age", InstructionText = "Enter a valid age as a number" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Gender as [bcr.Gender]", DisplayValue = "Gender", InstructionText = "Enter M for male and F for female" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Race as [bcr.Race]", DisplayValue = "Race", InstructionText = "Available race values: AMERICAN INDIAN OR ALASKAN NATIVE, ASIAN OR PACIFIC ISLANDER, BLACK, CAUCASIAN, DECLINED TO ANSWER, HISPANIC WHITE, HISPANIC BLACK, MEXICAN AMERICAN, NATIVE HAWAIIAN OR OTHER PACIFIC ISLANDER, UNKNOWN, WHITE" });

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.VeteranFlag as [bcr.VeteranFlag]", DisplayValue = "Veteran Flag", InstructionText = "Enter Y for Veteran or N for Not Veteran" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.CombatFlag as [bcr.CombatFlag]", DisplayValue = "Combat Flag", InstructionText = "Enter Y for Combat Veteran or N for No Combat" });

        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PhoneResidence as [bcr.PhoneResidence]", DisplayValue = "PhoneResidence" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PhoneWork as [bcr.PhoneWork]", DisplayValue = "PhoneWork" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PhoneCellular as [bcr.PhoneCellular]", DisplayValue = "PhoneCellular" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.StreetAddress1 as [bcr.StreetAddress1]", DisplayValue = "StreetAddress1" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.StreetAddress2 as [bcr.StreetAddress2]", DisplayValue = "StreetAddress2" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.StreetAddress3 as [bcr.StreetAddress3]", DisplayValue = "StreetAddress3" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.City as [bcr.City]", DisplayValue = "City" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.State as [bcr.State]", DisplayValue = "State" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Zip as [bcr.Zip]", DisplayValue = "Zip" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Deceased as [bcr.Deceased]", DisplayValue = "Deceased", InstructionText = "Enter Y for Deceased or N for Living" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.Region as [bcr.Region]", DisplayValue = "Region", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.VISN as [bcr.VISN]", DisplayValue = "VISN", InstructionText = "Enter the VISN number (i.e. 1-23)" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.ISOEFOIF as [bcr.ISOEFOIF]", DisplayValue = "IS OEF/OIF", InstructionText = "Enter Y for OEF/OIF or N for Other" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CASE bcr.ISOEFOIF WHEN 0 THEN 'No' WHEN 1 THEN 'Yes' END as [bcr.ISOEFOIF]", DisplayValue = "IS OEF/OIF", InstructionText = "Enter Yes for OEF/OIF or No for Other" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.HomeSta3N as [bcr.HomeSta3N]", DisplayValue = "HomeSta3N" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PCMMStaffSID as [bcr.PCMMStaffSID]", DisplayValue = "PCMMStaffSID" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PCMMTeamSID as [bcr.PCMMTeamSID]", DisplayValue = "PCMMTeamSID" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PCMMTeam as [bcr.PCMMTeam]", DisplayValue = "PCMM Team", InstructionText = "Enter whole or partial team name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.PCMMStaffName as [bcr.PCMMStaffName]", DisplayValue = "PCMM Staff Name", InstructionText = "Enter whole or partial staff name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfPrimaryCareEncounters as [bcr.NumberOfPrimaryCareEncounters]", DisplayValue = "Primary Care Visits (Last 12 mo)", InstructionText = "Enter number of Primary Care visits this year (last 12 mo)" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentPrimaryCareEncounter as [bcr.DateOfMostRecentPrimaryCareEncounter]", DisplayValue = "Most Recent Primary Care Visit Date", InstructionText = "Filter by the date of most recent Primary Care visit" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfNexusClinicEncounters as [bcr.NumberOfNexusClinicEncounters]", DisplayValue = "Nexus Clinic Encounters (Last 12 mo)", InstructionText = "Enter number of Nexus Clinic Encounters this year (last 12 mo)" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentNexusClinicEncounter as [bcr.DateOfMostRecentNexusClinicEncounter]", DisplayValue = "Most Recent Nexus Clinic Visit Date", InstructionText = "Filter by the date of most recent Nexus Clinic visit" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfNexusClinicPriorEncounters as [bcr.NumberOfNexusClinicPriorEncounters]", DisplayValue = "NumberOfNexusClinicPriorEncounters" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRRecentNexusClinicPriorEncounter as [bcr.DateOfMostRRecentNexusClinicPriorEncounter]", DisplayValue = "DateOfMostRRecentNexusClinicPriorEncounter" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfNexusClinicFormerEncounters as [bcr.NumberOfNexusClinicFormerEncounters]", DisplayValue = "NumberOfNexusClinicFormerEncounters" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentNexusClinicFormerEncounter as [bcr.DateOfMostRecentNexusClinicFormerEncounter]", DisplayValue = "DateOfMostRecentNexusClinicFormerEncounter" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.WomensHealthVisitStatus as [bcr.WomensHealthVisitStatus]", DisplayValue = "Womens Health Visit Status", InstructionText = "Enter 'Established Patient', 'Last Seen 2 - 3 Years Ago', 'New Patient'" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfWomensHealthEncounters as [bcr.NumberOfWomensHealthEncounters]", DisplayValue = "Womens Health Visit Count", InstructionText = "Enter the number of women's health visits" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentWomensHealthEncounter as [bcr.DateOfMostRecentWomensHealthEncounter]", DisplayValue = "Most Recent Womens Health Visit Date", InstructionText = "Filter by the date of most recent Women's Health visit" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfCancerDiagnosis as [bcr.NumberOfCancerDiagnosis]", DisplayValue = "Number Of Cancer Dx", InstructionText = "Enter the number of cancer diagnosis entries" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.NumberOfTerminalCADDiagnosis as [bcr.NumberOfTerminalCADDiagnosis]", DisplayValue = "Number Of Terminal CAD Dx", InstructionText = "Enter the number of Terminal CAD Diagnosis entries" });

        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.HadBilateralMastectomy as [bcr.HadBilateralMastectomy]", DisplayValue = "Had Bilateral Mastectomy" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CASE bcr.HadBilateralMastectomy WHEN 0 THEN 'No' WHEN 1 THEN 'Yes' END as [bcr.HadBilateralMastectomy]", DisplayValue = "Had Bilateral Mastectomy", InstructionText = "Enter Yes or No" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CASE bcr.HasATerminalHealthFactor WHEN 0 THEN 'No' WHEN 1 THEN 'Yes' END as [bcr.HasATerminalHealthFactor]", DisplayValue = "Has A Terminal Health Factor", InstructionText = "Enter Yes or No" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CASE bcr.HadHysterectomy WHEN 0 THEN 'No' WHEN 1 THEN 'Yes' END as [bcr.HadHysterectomy]", DisplayValue = "Had Hysterectomy", InstructionText = "Enter Yes or No" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CASE bcr.HasMammogramExclusionHealthFactor WHEN 0 THEN 'No' WHEN 1 THEN 'Yes' END as [bcr.HasMammogramExclusionHealthFactor]", DisplayValue = "Has MMG Exclusion Health Factor", InstructionText = "Enter Yes or No" });

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentMammogramExclusionHealthFactor as [bcr.DateOfMostRecentMammogramExclusionHealthFactor]", DisplayValue = "Most Recent MMG Exclusion HF Date", InstructionText = "Date for most recent mammogram exclusion health factor" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MammogramMostRecentScreenFreq as [bcr.MammogramMostRecentScreenFreq]", DisplayValue = "Most Recent Screen Freq (MMG)", InstructionText = "Enter 1Mo, 3Mo, 4Mo, 6Mo, 1Yr, 2Yr, 3Yr, 4Yr, 5Yr, 10Yr, OR NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentMammogramScreenFreq as [bcr.DateOfMostRecentMammogramScreenFreq]", DisplayValue = "Most Recent MMG Screen Freq Date", InstructionText = "Date for most recent screening frequency entry" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.BIRADScore as [bcr.BIRADScore]", DisplayValue = "BIRAD Score", InstructionText = "Filter by BIRAD Score" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MostRecentMammogramProcedure as [bcr.MostRecentMammogramProcedure]", DisplayValue = "Most Recent MMG Procedure", InstructionText = "Enter whole or partial mammogram procedure name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOFMostRecentMammogram as [bcr.DateOFMostRecentMammogram]", DisplayValue = "Most Recent MMG Date", InstructionText = "Date of most recent mammogram" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MostRecentMammogramSource as [bcr.MostRecentMammogramSource]", DisplayValue = "Most Recent MMG Source", InstructionText = "Enter Fee, HF, Radiology, VDiagnosis, VProcedure, or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateNextMammogramDue as [bcr.DateNextMammogramDue]", DisplayValue = "Date Next MMG Due", InstructionText = "Date next mammogram due" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MammogramComplianceText as [bcr.MammogramComplianceText]", DisplayValue = "Mammogram Compliance", InstructionText = "Yes, No, or N/A" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfNextAppointment as [bcr.DateOfNextAppointment]", DisplayValue = "Next Appointment Date", InstructionText = "Date of next appointment" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.LocationNameOfNextAppointment as [bcr.LocationNameOfNextAppointment]", DisplayValue = "Next Appointment Location", InstructionText = "Enter whole or partial appointment location" });
        
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.DateOfMostRecentMammogramResult as [bcr.DateOfMostRecentMammogramResult]", DisplayValue = "DateOfMostRecentMammogramResult" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "bcr.MostRecentMammogramDiagnosticCode as [bcr.MostRecentMammogramDiagnosticCode]", DisplayValue = "MostRecentMammogramDiagnosticCode" });

        #endregion

        #region RadiologyMammos Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.PatientSID as [Rad.PatientSID]", DisplayValue = "(Rad) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.PatientIEN as [Rad.PatientIEN]", DisplayValue = "(Rad) PatientIEN", InstructionText = "Enter whole or partial IEN" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.PatientICN as [Rad.PatientICN]", DisplayValue = "(Rad) PatientICN", InstructionText = "Enter whole or partial ICN" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.Sta3n as [Rad.Sta3n]", DisplayValue = "(Rad) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.sub_registered_exams as [Rad.sub_registered_exams]", DisplayValue = "Rad.sub_registered_exams" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.ChildSub2 as [Rad.ChildSub2]", DisplayValue = "Rad.ChildSub2" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.case_number as [Rad.case_number]", DisplayValue = "(Rad) Case Number", InstructionText = "Full or partial case number" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.examstatusx as [Rad.examstatusx]", DisplayValue = "(Rad) Exam Status", InstructionText = "Full or partial exam status" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.exam_date as [Rad.exam_date]", DisplayValue = "(Rad) Exam Date", InstructionText = "Date of exam" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.category_of_exam as [Rad.category_of_exam]", DisplayValue = "(Rad) Exam Category", InstructionText = "Enter CONTRACT, EMPLOYEE, INPATIENT, OUTPATIENT, SHARING, or NULL" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.primary_diagnostic_code as [Rad.primary_diagnostic_code]", DisplayValue = "(Rad) Primary Diagnostic Code", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.OrderingProvider as [Rad.OrderingProvider]", DisplayValue = "(Rad) Ordering Provider", InstructionText = "Full or partial ordering provider name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.diagnostic_code as [Rad.diagnostic_code]", DisplayValue = "(Rad) Diagnostic Code", InstructionText = "Full or partial diagnostic description" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.PatientName as [Rad.PatientName]", DisplayValue = "(Rad) Patient Name", InstructionText = "Full or partial patient name" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.procedure2 as [Rad.procedure2]", DisplayValue = "(Rad) Procedure2" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.ProcedureName as [Rad.ProcedureName]", DisplayValue = "(Rad) Procedure Name", InstructionText = "Full or partial procedure name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.cpt_code as [Rad.cpt_code]", DisplayValue = "(Rad) CPT Code", InstructionText = "Full or partial CPT code for procedure" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.type_of_imaging as [Rad.type_of_imaging]", DisplayValue = "(Rad) Imaging Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.typeofimagingx as [Rad.typeofimagingx]", DisplayValue = "(Rad) Type of Imaging", InstructionText = "Enter ANGIO/NEURO/INTERVENTIONAL, CT SCAN, GENERAL RADIOLOGY, MAGNETIC RESONANCE IMAGING, MAMMOGRAPHY, NUCLEAR MEDICINE, OUTSIDE FACILITIES, or ULTRASOUND" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.HospitalDivision as [Rad.HospitalDivision]", DisplayValue = "(Rad) Hospital Division", InstructionText = "Full or partial hospital division" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.ImagingLocation as [Rad.ImagingLocation]", DisplayValue = "(Rad) Imaging Location", InstructionText = "Full or partial imaging location" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.BIRADScore as [Rad.BIRADScore]", DisplayValue = "(Rad) BIRAD Score", InstructionText = "Filter by BIRAD Score" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.SecondaryDiagnosticCode as [Rad.SecondaryDiagnosticCode]", DisplayValue = "(Rad) Secondary Diagnostic Code", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.reporttextwp as [Rad.reporttextwp]", DisplayValue = "(Rad) Report Text", InstructionText = "Search for keywords in report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.impressiontextwp as [Rad.impressiontextwp]", DisplayValue = "(Rad) Impression Text", InstructionText = "Search for keywords in impression text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Rad.RadiologyType as [Rad.RadiologyType]", DisplayValue = "(Rad) Radiology Type", InstructionText = "Enter Mammogram, MRI, Ultrasound, Biopsy, or Fee" });

        #endregion

        #region Oncology Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.PatientSID as [Onc.PatientSID]", DisplayValue = "(Onc) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.PatientiEN as [Onc.PatientiEN]", DisplayValue = "(Onc) PatientiEN", InstructionText = "Enter whole or partial IEN" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.Sta3n as [Onc.Sta3n]", DisplayValue = "(Onc) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.date_dx as [Onc.date_dx]", DisplayValue = "(Onc) Dx Date", InstructionText = "Date of Diagnosis" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.primarysitex as [Onc.primarysitex]", DisplayValue = "(Onc) Primary Site", InstructionText = "Full or partial search of primary site" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.histologyicdo3x as [Onc.histologyicdo3x]", DisplayValue = "(Onc) Histology", InstructionText = "Search for keywords in Histology" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.diagnostic_confirmation as [Onc.diagnostic_confirmation]", DisplayValue = "(Onc) Dx Confirmation", InstructionText = "Full or partial diagnostic confirmation" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.laterality as [Onc.laterality]", DisplayValue = "(Onc) Laterality", InstructionText = "Enter 'Left', 'Right', 'One side involved, right/left not specified', 'Paired site, no laterality information', 'Paired site, midline tumor', 'Bilateral involvement, side of origin unknown', 'Not a paired site', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.stage_groupingajcc as [Onc.stage_groupingajcc]", DisplayValue = "(Onc) Breast Cancer Stage", InstructionText = "Enter '0', 'I', 'II', 'III', 'IV', 'NA', 'Unk/Uns', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.radiation as [Onc.radiation]", DisplayValue = "(Onc) Radiation", InstructionText = "Enter 'None', 'Beam + implants or radioisotopes', 'Unknown if administered', 'Radiation, NOS', 'Beam radiation', 'Refused radiation', 'Radioactive implants', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.radiationsurgery_sequence as [Onc.radiationsurgery_sequence]", DisplayValue = "(Onc) Rad/Surg Sequence", InstructionText = "Enter 'Sequence unknown', 'Rad after surgery', 'No rad and/or surgery', 'Intraoperative rad', 'Rad both before/after surgery', 'Rad before surgery', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.chemotherapy_date as [Onc.chemotherapy_date]", DisplayValue = "(Onc) Chemo Date", InstructionText = "Date of chemo treatment or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.chemotherapy as [Onc.chemotherapy]", DisplayValue = "(Onc) Chemo", InstructionText = "Enter 'None', 'Unknown', 'Chemotherapy, NOS', 'Refusal', 'Single-agent', 'Recommended, unknown if admin', 'Not administered/contraindicated', 'Recommended, not admin, no reason given', 'Multiagent', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.hormone_therapy_date as [Onc.hormone_therapy_date]", DisplayValue = "(Onc) Hormone Therapy Date", InstructionText = "Date of hormonal therapy treatment or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.hormone_therapy as [Onc.hormone_therapy]", DisplayValue = "(Onc) Hormone Therapy", InstructionText = "Enter 'None', 'Unknown', 'Refusal', 'Pt died prior to tx', 'Recommended, unknown if admin', 'Not administered/contraindicated', 'Hormone therapy', 'Recommended, not admin, no reason given', NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.reasonnosurgeryofprimary as [Onc.reasonnosurgeryofprimary]", DisplayValue = "(Onc) No Surg Reason", InstructionText = "Enter 'Unknown', 'Pt died prior to surgery', 'Recommended, not performed, no reason given', 'Refusal', 'Surgery performed', 'Not part of 1st course', 'Recommended, unknown if performed', 'Contraindicated', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.reason_for_no_radiation as [Onc.reason_for_no_radiation]", DisplayValue = "(Onc) No Rad Reason", InstructionText = "Enter 'Unknown', 'Refusal', 'Radiation administered', 'Not part of 1st course', 'Pt died prior to tx', 'Contraindicated', 'Recommended, unknown if admin', 'Recommended, not admin, no reason given', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.reason_for_no_chemotherapy as [Onc.reason_for_no_chemotherapy]", DisplayValue = "(Onc) No Chemo Reason", InstructionText = "Enter 'Pt refused chemo', 'Reason unk', 'Chemo not recommended', 'Chemo administered', 'Chemo recommended, unk if administered', 'Unk if administered, death cert-only cases', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.reason_for_no_hormone_therapy as [Onc.reason_for_no_hormone_therapy]", DisplayValue = "(Onc) No Hormone Reason", InstructionText = "Enter 'HT recommended, unk if administered', 'Reason unk', 'Pt refused HT', 'HT not recommended', 'HT administered', 'Unk if administered, death cert-only cases', or NULL" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.rxtextradiationwp as [Onc.rxtextradiationwp]", DisplayValue = "(Onc) Radiation Rx", InstructionText = "Search keywords in radiation report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.rxtextchemowp as [Onc.rxtextchemowp]", DisplayValue = "(Onc) Chemo Rx", InstructionText = "Search keywords in chemotherapy report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.rxtexthormonewp as [Onc.rxtexthormonewp]", DisplayValue = "(Onc) Hormone Rx", InstructionText = "Search keywords in hormone therapy report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.textdxproclabtestswp as [Onc.textdxproclabtestswp]", DisplayValue = "(Onc) Lab Report", InstructionText = "Search keywords in lab test report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Onc.textdxprocpewp as [Onc.textdxprocpewp]", DisplayValue = "(Onc) Onc Report", InstructionText = "Search keywords in procedure report text" });

        #endregion

        #region Surgery / Biopsy Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.PatientSID as [SurgBiop.PatientSID]", DisplayValue = "(SurgBiop) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.Sta3n as [SurgBiop.Sta3n]", DisplayValue = "(SurgBiop) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.Event_Date as [SurgBiop.Event_Date]", DisplayValue = "(SurgBiop) Event Date", InstructionText = "Date of Procedure" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.[Procedure] as [SurgBiop.Procedure]", DisplayValue = "(SurgBiop) Procedure", InstructionText = "CPT Code or ICD Code of Procedure" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.Report as [SurgBiop.Report]", DisplayValue = "(SurgBiop) Report", InstructionText = "Enter keyword search for report text" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.Type as [SurgBiop.Type]", DisplayValue = "(SurgBiop) Type", InstructionText = "Enter type of procedure: 'Surgery' or 'Biopsy'" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.CPTCode as [SurgBiop.CPTCode]", DisplayValue = "(SurgBiop) CPT", InstructionText = "" });
        //ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.ICDCode as [SurgBiop.ICDCode]", DisplayValue = "(SurgBiop) ICD", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SurgBiop.HospitalLocation as [SurgBiop.HospitalLocation]", DisplayValue = "(SurgBiop) Location", InstructionText = "Enter whole or partial hospital location name" });

        #endregion

        #region Breast Cancer Entries Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CancerDx.PatientSID as [CancerDx.PatientSID]", DisplayValue = "(CancerDx) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CancerDx.VisitDateTime as [CancerDx.VisitDateTime]", DisplayValue = "(CancerDx) Visit Date", InstructionText = "Date of Visit" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CancerDx.[Diagnosis Date] as [CancerDx.Diagnosis Date]", DisplayValue = "(CancerDx) Dx Date", InstructionText = "Diagnosis Date" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CancerDx.ICD9Code as [CancerDx.ICD9Code]", DisplayValue = "(CancerDx) ICD", InstructionText = "Enter whole or partial ICD Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "CancerDx.ICD9Description as [CancerDx.ICD9Description]", DisplayValue = "(CancerDx) ICD Desc", InstructionText = "Enter whole or partial ICD Description" });

        #endregion

        #region Health Factor Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.PatientSID as [HealthFactor.PatientSID]", DisplayValue = "(HF) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.Sta3n as [HealthFactor.Sta3n]", DisplayValue = "(HF) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.HealthFactorType as [HealthFactor.HealthFactorType]", DisplayValue = "(HF) Health Factor Type", InstructionText = "Enter whole or partial Health Factor Type" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.Comments as [HealthFactor.Comments]", DisplayValue = "(HF) Comments", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.VisitDateTime as [HealthFactor.VisitDateTime]", DisplayValue = "(HF) Visit Date", InstructionText = "Date of Visit" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "HealthFactor.HealthFactorDateTime as [HealthFactor.HealthFactorDateTime]", DisplayValue = "(HF) Health Factor Date", InstructionText = "Health Factor Date" });

        #endregion

        #region Orders Columns

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.PatientSID as [Orders.PatientSID]", DisplayValue = "(Orders) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.Sta3n as [Orders.Sta3n]", DisplayValue = "(Orders) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.Requesting_Provider as [Orders.Requesting_Provider]", DisplayValue = "(Orders) Requesting Provider", InstructionText = "Enter whole or partial requesting Provider Name" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.ImagingType as [Orders.ImagingType]", DisplayValue = "(Orders) Imaging Type", InstructionText = "Imaging type options include: 'ANGIO/NEURO/INTERVENTIONAL', 'CT SCAN', 'MAGNETIC RESONANCE IMAGING', 'MAMMOGRAPHY', 'NUCLEAR MEDICINE', 'RADIOLOGY', 'ULTRASOUND', or 'NULL'" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.OrderStartDateTime as [Orders.OrderStartDateTime]", DisplayValue = "(Orders) Start Date (Desired)", InstructionText = "Order Start Date/Time" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.OrderStopDateTime as [Orders.OrderStopDateTime]", DisplayValue = "(Orders) Stop Date", InstructionText = "Order Stop Date/Time" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.OrderableItemCode as [Orders.OrderableItemCode]", DisplayValue = "(Orders) Orderable Item Code", InstructionText = "CPT Code for Order" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.CPTName as [Orders.CPTName]", DisplayValue = "(Orders) CPT Name", InstructionText = "Short Name for CPT Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.CPTDescription as [Orders.CPTDescription]", DisplayValue = "(Orders) CPT Desc", InstructionText = "Description for CPT Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.DiscontinuedDateTime as [Orders.DiscontinuedDateTime]", DisplayValue = "(Orders) Discontinued Date", InstructionText = "If an order has a discontinued date/time." });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.DiscontinuedOrderReasonText as [Orders.DiscontinuedOrderReasonText]", DisplayValue = "(Orders) Disc. Order Reason", InstructionText = "Enter whole or partial reason for discontinued order" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "Orders.OrderStatus as [Orders.OrderStatus]", DisplayValue = "(Orders) Order Status", InstructionText = "Order status options include: 'ACTIVE', 'CANCELLED', 'COMPLETE', 'DISCONTINUED', 'EXPIRED', 'HOLD', 'LAPSED', 'PENDING', or 'SCHEDULED'" });

        #endregion

        #region SMART HF

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.PatientSID as [SMARTHF.PatientSID]", DisplayValue = "(SMART) PatientSID", InstructionText = "Enter PatientSID Identifier" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.Sta3n as [SMARTHF.Sta3n]", DisplayValue = "(SMART) Sta3n", InstructionText = "Enter whole or partial Sta3n - Facility Code" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.HealthFactorType as [SMARTHF.HealthFactorType]", DisplayValue = "(SMART) Health Factor Type", InstructionText = "Enter whole or partial health factor type" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.Comments as [SMARTHF.Comments]", DisplayValue = "(SMART) Comments", InstructionText = "" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.VisitDateTime as [SMARTHF.VisitDateTime]", DisplayValue = "(SMART) Visit Date", InstructionText = "Date of Visit" });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "SMARTHF.HealthFactorDateTime as [SMARTHF.HealthFactorDateTime]", DisplayValue = "(SMART) Health Factor Date", InstructionText = "Health Factor Date/Time" });

        #endregion

        #region Appointments

        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "APPT.ApptDteTm as [APPT.ApptDteTm]", DisplayValue = "(Appointment) Date", InstructionText = "Date for scheduled appointment time." });
        ReportColumnList.Add(new AdHocTemplateDisplay() { Value = "APPT.ApptLocation as [APPT.ApptLocation]", DisplayValue = "(Appointment) Location", InstructionText = "Enter whole or partial appointment location name" });
        
        #endregion

        #endregion
    }

    /// <summary>
    /// Finds the Display Name for a given field name from the Report Column List
    /// </summary>
    /// <param name="fieldName"></param>
    /// <returns></returns>
    public string GetDisplayNameFromFieldName(string fieldName)
    {
        foreach (AdHocTemplateDisplay atd in ReportColumnList) {
            if (atd.Value.Contains(fieldName))
            {
                return atd.DisplayValue;
            }
        }

        return null;
    }

    /// <summary>
    /// Returns a formatted Display Name - InstructionText for a given field name from the Report Column List
    /// </summary>
    /// <param name="fieldName"></param>
    /// <returns></returns>
    public string GetDisplayNameAndInstructionsFromFieldName(string fieldName)
    {
        foreach (AdHocTemplateDisplay atd in ReportColumnList)
        {
            if (atd.Value.Contains(fieldName))
            {
                if (!String.IsNullOrEmpty(atd.InstructionText))
                {
                    return "<strong>" + atd.DisplayValue + "</strong>:<br />" + atd.InstructionText;
                }
                else
                {
                    return null;
                }
            }
        }

        return null;
    }

    /// <summary>
    /// Finds the Field Name for a given display name from the Report Column List
    /// </summary>
    /// <param name="displayName"></param>
    /// <returns></returns>
    public string GetFieldNameFromDisplayName(string displayName)
    {
        foreach (AdHocTemplateDisplay atd in ReportColumnList)
        {
            if (atd.DisplayValue.Contains(displayName))
            {
                return atd.Value;
            }
        }

        return null;
    }

    /// <summary>
    /// GenerateAdhocSchema: returns the "schema" of the report template.  Pass 1 as third parameter for "schema-only"
    /// bcccr.usp_Adhoc_Search params - (requestedColumnStr, whereClauseStr, 0/1 IsSchemaOnly)
    /// </summary>
    /// <param name="requestedColumns"></param>
    /// <returns></returns>
    public static DataTable GenerateAdhocSchema(string requestedColumns, string joinType)
    {
        return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, ADHOC_SEARCH, new object[] { requestedColumns, null, 1, null, joinType });
    }

    /// <summary>
    /// GenerateAdhocResults: Like GenerateAdhocSchema, this returns the "schema" of the report template, however, passing 0 as second parameter returns the data.
    /// </summary>
    /// <param name="requestedColumns"></param>
    /// <returns></returns>
    public static DataTable GenerateAdhocResults(string requestedColumns, string whereClause, string accessibleFacilities, string joinType)
    {
        return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, ADHOC_SEARCH, new object[] { requestedColumns, whereClause, 0, accessibleFacilities, joinType });
    }

    /// <summary>
    /// InsertUpdateAdhocReportTemplate: Used for saving new or updated report template.
    /// </summary>
    /// <param name="id"></param>
    /// <param name="userId"></param>
    /// <param name="reportName"></param>
    /// <param name="xmlData"></param>
    /// <param name="whereData"></param>
    /// <param name="sortExpression"></param>
    /// <returns></returns>
    public static int InsertUpdateAdhocReportTemplate(int id, int userId, string reportName, string xmlData, string whereData, string whereSQL, string sortExpression, string joinType, bool isSystemTemplate)
    {
        return SqlProvider.PerformInsert(CONNECTION_NAME, ADHOC_INSERT_UPDATE_REPORT, new object[] { id, userId, reportName, xmlData, whereData, whereSQL, sortExpression, joinType, isSystemTemplate });
    }

    /// <summary>
    /// Deletes the report with the corresponding ID.
    /// </summary>
    /// <param name="id"></param>
    public static void DeleteUserReportingTemplate(int userId, int reportId)
    {
        SqlProvider.PerformDelete(CONNECTION_NAME, ADHOC_DELETE_REPORT, new object[] { userId, reportId });
    }

    /// <summary>
    /// UpdateLastExecutionDate: Updates the execution date for the user template report.
    /// </summary>
    /// <param name="id"></param>
    public static void UpdateLastExecutionDate(int id, DateTime updatedDate)
    {
        SqlProvider.PerformUpdate(CONNECTION_NAME, ADHOC_UPDATE_LAST_EXEC_DATE, new object[] { id, updatedDate });
    }

    /// <summary>
    /// LoadAllUserReportTemplates: Loads all user reports for the grid.
    /// </summary>
    /// <param name="userId"></param>
    /// <returns></returns>
    public static DataTable LoadAllUserReportTemplates(int userId) 
    {
        return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, ADHOC_LOAD_ALL_USER_REPORT_TEMPLATES, new object[] { userId });
    }

    /// <summary>
    /// LoadSingleUserReportTemplate: Returns full object for a passed id and user.
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="reportId"></param>
    /// <returns></returns>
    public static AdHocReportTemplate LoadSingleUserReportTemplate(int userId, int reportId)
    {
        DataTable dt = SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, ADHOC_LOAD_SINGLE_USER_REPORT_TEMPLATE, new object[] { userId, reportId });

        if (dt != null)
        {
            if (dt.Rows.Count > 0)
            {
                // get data from row 0 (only record)
                //[ID],[UserID],[ReportName],[LastExecutionDate],[TemplateData],[WhereData],[SortExpression],[CreatedDate],[UpdatedDate]
                try
                {
                    AdHocReportTemplate rptTemplate = new AdHocReportTemplate();
                    rptTemplate.ID = Int32.Parse(dt.Rows[0]["ID"].ToString());
                    rptTemplate.UserId = Int32.Parse(dt.Rows[0]["UserID"].ToString());
                    rptTemplate.ReportName = dt.Rows[0]["ReportName"].ToString();
                    rptTemplate.ReportColumnsXmlData = (dt.Rows[0]["TemplateData"] != null) ? dt.Rows[0]["TemplateData"].ToString() : "";
                    rptTemplate.WhereDataExpressions = (dt.Rows[0]["WhereData"] != null) ? dt.Rows[0]["WhereData"].ToString() : "";
                    rptTemplate.WhereSQLString = (dt.Rows[0]["WhereSQL"] != null) ? dt.Rows[0]["WhereSQL"].ToString() : "";
                    rptTemplate.SortExpressionString = (dt.Rows[0]["SortExpression"] != null) ? dt.Rows[0]["SortExpression"].ToString() : "";

                    if (dt.Rows[0]["LastExecutionDate"] != null)
                    {
                        if (!String.IsNullOrEmpty(dt.Rows[0]["LastExecutionDate"].ToString()))
                        {
                            rptTemplate.LastExecutionDate = DateTime.Parse(dt.Rows[0]["LastExecutionDate"].ToString());
                        }
                    }

                    rptTemplate.CreatedDate = DateTime.Parse(dt.Rows[0]["CreatedDate"].ToString());
                    rptTemplate.UpdatedDate = DateTime.Parse(dt.Rows[0]["UpdatedDate"].ToString());

                    rptTemplate.JoinType = (dt.Rows[0]["JoinType"] != null) ? dt.Rows[0]["JoinType"].ToString() : "INNER JOIN";
                    rptTemplate.IsSystemTemplate = (bool)dt.Rows[0]["IsSystemTemplate"];

                    return rptTemplate;

                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile("Error Loading Single User Template (Id: " + reportId + " - UserId: " + userId + ") - " + ex.Message);
                    return null;
                }
            }
            else
            {
                return null;
            }
        }
        return null;
    }

    #region Utility Methods

    public static Object Deserialize(string val)
    {
        return LosSerializer.Deserialize(val);
    }

    public static string Serialize(Object val)
    {
        return LosSerializer.Serialize(val);
    }

    /// <summary>
    /// Deserialize Passed Object
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="toDeserialize"></param>
    /// <returns></returns>
    public static object DeserializeObject<T>(string toDeserialize)
    {
        XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
        StringReader textReader = new StringReader(toDeserialize);
        object temp = xmlSerializer.Deserialize(textReader);
        return temp;
    }

    /// <summary>
    /// Serialize passed object to string.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="toSerialize"></param>
    /// <returns></returns>
    public static string SerializeObject<T>(T toSerialize)
    {
        XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
        StringWriter textWriter = new StringWriter();
        xmlSerializer.Serialize(textWriter, toSerialize);
        return textWriter.ToString();
    }

    /// <summary>
    /// GenerateColumnStringForStoredProc: Generate report column string for stored procedure
    /// </summary>
    /// <param name="xDoc"></param>
    /// <returns></returns>
    public static string GenerateColumnStringForStoredProc(XDocument xDoc)
    {
        // get all columns that were selected.
        StringBuilder selectedColumnStr = new StringBuilder();

        try
        {
            XElement rootElem = xDoc.Element("ReportTemplate");
            XElement reportColElem = rootElem.Element("ReportColumns");

            foreach (XElement subElem in reportColElem.Elements("ReportColumn"))
            {
                selectedColumnStr.Append(subElem.Attribute("Value").Value + ",");
            }

            // remove last ,
            string selectString = selectedColumnStr.ToString();

            if (selectString.Length > 0)
            {
                if (selectString.EndsWith(","))
                {
                    selectString = selectString.Substring(0, selectString.Length - 1);
                }
            }

            return selectString;
        }
        catch (Exception ex)
        {
            // TODO: display error message ... 
            Logging.WriteLogToFile("Error Generating Report Template Schema: " + ex.Message);
            return null;
        }
    }

    #endregion
}