﻿using System.Data;
using Telerik.Web.UI;
using System;

public class Reports
{
    #region Member Variables 

    //stored proc const
    private const string GET_USER_FAC = "bcccr.usp_Get_Fac_by_UserId";
    private const string GET_VISNS = "bcccr.usp_Get_VISNs";
    private const string GET_FACS = "bcccr.usp_Get_Facs";
    private const string GET_FACILITY_DETAILS_BY_ID = "bcccr.usp_Get_FacilityDetails_By_Id";
    private const string GET_PACS = "bcccr.usp_Get_PACs";
    private const string GET_STA3N_BY_INSTITUTION_ID = "bcccr.usp_GetSta3nByInstitutionId";
    private const string GET_FAC_BY_VISN = "bcccr.usp_Get_Fac_by_VISNId";
    private const string GET_PAC_BY_FAC = "bcccr_sp_Get_PAC_by_FacId";
    private const string GET_PATIENT_RACES = "bcccr.usp_Get_Race";
    private const string GET_DISTINCT_PATIENT_RACES = "bcccr.usp_Get_Distinct_Patient_Races";
    private const string GET_PATIENT_GENDER = "bcccr.usp_Get_Gender";
    private const string GET_DIVS_BY_DIV_ID = "bcccr.usp_Get_Children_by_DivId";
    private const string GET_SETTING = "CRS_SP_RETRIEVE_SETTING";
    private const string GET_ALL_SETTINGS = "CRS_SP_RETRIEVE_ALL_SETTINGS";
    private const string GET_SETTING_FOR_UPDATE = "CRS_SP_RETRIEVE_SETTING_FOR_UPDATE";
    private const string SET_SETTING_FOR_UPDATE = "CRS_SP_UPDATE_SETTING";

    private const string GET_ETL_REFRESH_DATE = "bcccr.usp_Get_ETL_Refresh_Date";

    private const string GET_PACT_TEAMS_BY_STA3N = "bcccr.usp_Get_PACT_Teams_By_Sta3n";

    private string _userId;

    private DataTable _dtFac = null;
    private DataTable _dtRaces = null;
    private DataTable _dtGenders = null;

    public DataTable dtFac { get { return _dtFac; } }
    public DataTable dtRace { get { return _dtRaces; } }
    public DataTable dtGender { get { return _dtGenders; } }

    #endregion

    public Reports(string userId)
    {
        _userId = userId;
    }

    //***Populate Listboxes***//
    //Aggregate
    public DataTable GetVISNs()
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_VISNS, new object[] { });
    }

    public DataTable GetDivsByDivId(string divId)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_DIVS_BY_DIV_ID, new object[] { divId });
    }

    public DataTable GetFacs()
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_FACS, new object[] { });
    }

    public DataTable GetPACs()
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PACS, new object[] { });
    }

    public string GetSta3nFromInstitutionId(string institutionId)
    {
        return SqlProvider.ExecuteScalar(Globals.GetConnectionName, GET_STA3N_BY_INSTITUTION_ID, new object[] { institutionId }).ToString();
    }

    public DataTable GetFacsByVISNList(string visnList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_FAC_BY_VISN, new object[] { visnList });
    }

    public DataTable GetPACByFacList(string facList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PAC_BY_FAC, new object[] { facList });
    }

    public Object GetSetting(string settingName, int registryId)
    {
        //return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_SETTING, new object[] { settingName, registryId });
        return SqlProvider.ExecuteScalar(Globals.GetConnectionName, GET_SETTING, new object[] { settingName, registryId });
    }

    public DataTable GetAllSettings(int registryId)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_SETTINGS, new object[] { registryId });
    }

    public DataTable GetSettingForUpdate(int registryId, int settingId)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_SETTING_FOR_UPDATE, new object[] { registryId, settingId });
    }

    public Object SetSettingForUpdate(int registryId, int settingId, string value, string description, string comment)
    {
        return SqlProvider.ExecuteScalar(Globals.GetConnectionName, SET_SETTING_FOR_UPDATE, new object[] { registryId, settingId, value, description, comment });
    }


    //Patient Data
    public bool GetFacilitiesByUserId()
    {
        _dtFac = SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_USER_FAC, new object[] { _userId });
        if ((_dtFac != null) && (_dtFac.IsInitialized) && (_userId.Length > 0))
        {
            return true;
        }
        else
            return false;
    }

    //Race Data
    public bool GetPatientRaces()
    {
        _dtRaces = SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PATIENT_RACES, new object[] { });
        if ((_dtRaces != null) && (_dtRaces.IsInitialized))
        {
            return true;
        }
        else
            return false;
    }

    /// <summary>
    /// Returns a list of distinct patient races from the BCR_ALL table.
    /// </summary>
    /// <returns></returns>
    public DataTable GetDistinctPatientRaces()
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_DISTINCT_PATIENT_RACES, new object[] { });
    }


    //Gender Data
    public bool GetPatientGenders()
    {
        _dtGenders = SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PATIENT_GENDER, new object[] { });
        if ((_dtGenders != null) && (_dtGenders.IsInitialized))
        {
            return true;
        }
        else
            return false;
    }

    //BiRad Data
    //public bool GetPatientBirads()
    //{
    //    _dtBiRad = SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PATIENT_BIRAD, new object[] { });
    //    if ((_dtBiRad != null) && (_dtBiRad.IsInitialized))
    //    {
    //        return true;
    //    }
    //    else
    //        return false;
    //}


    //Gets comma delimited string of VISNs, FACs or PACs
    public string GetCommaDelimitedList(RadListBox rlb)
    {
        string CheckedList = String.Empty;

        foreach (RadListBoxItem item in rlb.Items)
        {
            if (item.Checked)
            {
                CheckedList += item.Value + ",";
            }
            //make sure list is populated
            if (CheckedList.Length > 0)
            {
                CheckedList = CheckedList.Trim().Substring(0, CheckedList.Length - 1);
            }
        }

        return CheckedList;
    }
    //***

    /// <summary>
    /// Returns the DateTime of the latest ETL Completion Date.
    /// </summary>
    /// <returns></returns>
    public static DateTime GetETLRefreshDate()
    {
        return (DateTime)SqlProvider.ExecuteScalar(Globals.CONNECTION_NAME, GET_ETL_REFRESH_DATE, new object[] { });
    }

    /// <summary>
    /// Return all details for a facility.
    /// </summary>
    /// <param name="facilityId"></param>
    /// <returns></returns>
    public static DataTable GetFacilityDetailsById(string sta3n)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_FACILITY_DETAILS_BY_ID, new object[] { sta3n });
    }

    /// <summary>
    /// Returns a list of PACT Teams for a facility.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    public static DataTable GetPACTTeamsBySta3n(int sta3n)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_PACT_TEAMS_BY_STA3N, new object[] { sta3n });
    }
}

public class PatientData
{
    //template 
    private const string GET_REPORT_NAME = "bcccr.usp_RptPD_Report_Name";
    //***

    private const string GET_ORDERED_PENDING_PD = "bcccr.usp_RptPD_Ordered_Pending";
    private const string GET_ALL_MAM_WITH_BIRAD_PD = "bcccr.usp_RptPD_All_Mammo_By_Birad_By_Date";
    private const string GET_ALL_MAM_BY_DIAG_PD = "bcccr.usp_RptPD_All_Mammogram_By_Diagnosis";
    private const string GET_ALL_MAM_DUE_PD = "bcccr.usp_RptPD_All_Mammo_Due";
    private const string GET_BREAST_BIOPSIES_PD = "bcccr.usp_RptPD_Breast_Biopsies";
    private const string GET_BREAST_SURGERIES_PD = "bcccr.usp_RptPD_Breast_Surgery";
    private const string GET_BREAST_CANCER_PD = "bcccr.usp_RptPD_Breast_Cancer";
    private const string GET_ALL_MAM_FOLLOW_UP_PD = "bcccr.usp_RptPD_All_Mammo_followup";
    private const string GET_ALL_ABNORMAL_MAM_PD = "bcccr.usp_RptPD_All_Abnormal_Mammo_Surgeries_Bios_By_Birad";
    private const string GET_ALL_MAM_ICD_PD = "bcccr.usp_RptPD_All_Mammogram_icd";


    // New
    private const string GET_ALL_MAMMO_ABNORMAL_MAMMO_PD = "BCCCR.usp_Std_Rpt_GetMammogramsByDateAndFacId";
    private const string GET_MAMMO_DUE_ORDERED_PENDING_PD = "BCCCR.usp_Std_Rpt_GetMammoDueAndOrderedPendingByFacId";
    private const string GET_MAMMO_DUE_PD = "BCCCR.usp_Std_Rpt_GetMammoDueByDateRangeAndFacId";
    private const string GET_ORDERED_PENDING_RPT_PD = "BCCCR.usp_Std_Rpt_GetOrderedPendingByDateRangeAndFacId";
    private const string GET_BREAST_BIOPSIES_SURGERIES_PD = "BCCCR.usp_Std_Rpt_GetBiopAndSurgByDateAndFacId";
    private const string GET_STD_BREAST_CANCER_PD = "BCCCR.usp_Std_Rpt_GetBreastCancerByDateAndFacId";
    private const string GET_PATIENT_DETAILS = "BCCCR.usp_GetPatientDetails";
    private const string GET_PATIENT_SEARCH_RESULTS = "BCCCR.usp_PatientSearch";

    private DataTable _dtRpt = null;
    public DataTable dtRptPD { get { return _dtRpt; } }

    public PatientData()
    {
    }

    //template
    public DataTable GetReportName(string facList, string pacList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_REPORT_NAME, new object[] { facList });
    }
    //***
    //Patient Data Reports

    public DataTable GetOrderedPending(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ORDERED_PENDING_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllMamDue(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_DUE_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllMamWithBiRad(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_WITH_BIRAD_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetMamByDiag(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_BY_DIAG_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllMamWithResults(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_ICD_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllMamWithFollowUp(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_FOLLOW_UP_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllAbnormalMam(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_ABNORMAL_MAM_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetAllMamsICD(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_ICD_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetBreastBiopsies(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_BIOPSIES_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetBreastSurgeries(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_SURGERIES_PD, new object[] { facId, fromDate, toDate });
    }
    public DataTable GetBreastCancer(string facId, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_CANCER_PD, new object[] { facId, fromDate, toDate });
    }


    #region NEW METHODS - DME contract

    /// <summary>
    /// GetAllMammogramsAbnormalMammo: Calls the BCCCR.usp_Std_Rpt_GetMammogramsByDateAndFacId stored procedure to return a data table of mammogram-related data.
    /// </summary>
    /// <param name="facId"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <param name="showLatest"></param>
    /// <param name="biradList"></param>
    /// <param name="patName"></param>
    /// <param name="gender"></param>
    /// <param name="providerName"></param>
    /// <returns></returns>
    public DataTable GetAllMammogramsAbnormalMammo(string facId, string startDate, string endDate, int showLatest, string biradList, string ageRestriction, string patName, string gender, string providerName, string race, int showOnlyMammos, string followUpAction, string pactTeamString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAMMO_ABNORMAL_MAMMO_PD, new object[] { facId, startDate, endDate, showLatest, biradList, ageRestriction, patName, gender, providerName, race, showOnlyMammos, followUpAction, pactTeamString });
    }

    /// <summary>
    /// GetMammogramsDue: Calls the BCCCR.usp_Std_Rpt_GetMammoDueByDateRangeAndFacId stored procedure to return a data table of mammogram-related data.
    /// </summary>
    /// <param name="facId"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <param name="ageRestriction"></param>
    /// <param name="patName"></param>
    /// <param name="gender"></param>
    /// <param name="providerName"></param>
    /// <returns></returns>
    public DataTable GetMammogramsDue(string facId, string startDate, string endDate, string ageRestriction, string patName, string gender, string providerName, string mammotypeString, string race, string pactTeamString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_MAMMO_DUE_PD, new object[] { facId, startDate, endDate, ageRestriction, patName, gender, providerName, mammotypeString, race, pactTeamString });
    }

    /// <summary>
    /// GetOrderedPending: Calls the BCCCR.usp_Std_Rpt_GetOrderedPendingByDateRangeAndFacId stored procedure to return a data table of mammogram-related data.
    /// </summary>
    /// <param name="facId"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <param name="showLatest"></param>
    /// <param name="ageRestriction"></param>
    /// <param name="patName"></param>
    /// <param name="gender"></param>
    /// <param name="providerName"></param>
    /// <param name="mammotypeString"></param>
    /// <returns></returns>
    public DataTable GetOrderedPending(string facId, string startDate, string endDate, int showLatest, string ageRestriction, string patName, string gender, string providerName, string mammotypeString, string race, string pactTeamString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ORDERED_PENDING_RPT_PD, new object[] { facId, startDate, endDate, showLatest, ageRestriction, patName, gender, providerName, mammotypeString, race, pactTeamString });
    }

    /// <summary>
    /// GetAllMammogramsAbnormalMammo: Calls the BCCCR.usp_Std_Rpt_GetMammoDueAndOrderedPendingByFacId stored procedure to return a data table of mammograms due and ordered pending data.
    /// </summary>
    /// <param name="facId"></param>
    /// <returns></returns>
    public DataTable GetMammogramsDueOrderedPending(string facId)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_MAMMO_DUE_ORDERED_PENDING_PD, new object[] { facId });
    }

    /// <summary>
    /// GetBreastBiopsiesSurgeries: Calls the  stored procedure to return a data table of biopsy and surgery-related data.
    /// </summary>
    /// <param name="facId"></param>
    /// <returns></returns>
    public DataTable GetBreastBiopsiesSurgeries(string sta3n, string startDate, string endDate, string patName, string ageRestriction, string gender, string searchReportType, string providerName, string race, string followUpAction, string procedureCodeSearch, string pactTeamString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_BIOPSIES_SURGERIES_PD, new object[] { sta3n, startDate, endDate, patName, ageRestriction, gender, searchReportType, providerName, race, followUpAction, procedureCodeSearch, pactTeamString });
    }

    /// <summary>
    /// GetBreastBiopsiesSurgeries: Calls the  stored procedure to return a data table of biopsy and surgery-related data.
    /// </summary>
    /// <param name="facId"></param>
    /// <returns></returns>
    public DataTable GetBreastCancerStdRpt(string facId, string startDate, string endDate, string patName, string ageRestrictString, string gender, string providerName, string race, string followUpAction, string pactTeamString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_STD_BREAST_CANCER_PD, new object[] { facId, startDate, endDate, patName, ageRestrictString, gender, providerName, race, followUpAction, pactTeamString });
    }

    /// <summary>
    /// GetPatientDetails: Returns a Dataset for Patient Details-related data.
    /// </summary>
    /// <param name="patientId"></param>
    /// <returns></returns>
    public DataSet GetPatientDetails(string patientId)
    {
        return SqlProvider.ExecuteSPDataset(Globals.GetConnectionName, GET_PATIENT_DETAILS, new object[] { patientId });
    }

    /// <summary>
    /// GetPatientSearchResults: Returns a DataTable for the Patient Search Results for the passed Data.
    /// </summary>
    /// <param name="searchTerms"></param>
    /// <returns></returns>
    public DataTable GetPatientSearchResults(string searchTerms, string facStr, string sta3n)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_PATIENT_SEARCH_RESULTS, new object[] { searchTerms, facStr, sta3n });
    }

    #endregion
    //***
}

public class AdHocR
{
    //template 
    private const string GET_REPORT_NAME = "bcccr.usp_RptAdHoc_Report_Name";
    //***

    private const string GET_AD_HOC = "bcccr.usp_RptAH_Main";
    private const string GET_AD_HOC_ALL_ABNORMAL = "bcccr.usp_RptAH_All_Abnormal_Mammo_Surgeries_Bios_By_Birad";
    private const string GET_AD_HOC_ALL_MAMMOGRAMS = "bcccr.usp_RptAH_All_Mammo_By_Birad_By_Date";
    private const string GET_AD_HOC_ALL_MAMMOGRAMS_DUE = "bcccr.usp_RptAH_All_Mammo_due";
    private const string GET_AD_HOC_BREAST_BIOPSY = "bcccr.usp_RptAH_Breast_Biopsies";
    private const string GET_AD_HOC_BREAST_CANCER = "bcccr.usp_RptAH_Breast_Cancer";
    private const string GET_AD_HOC_BREAST_SURGERY = "bcccr.usp_RptAH_Breast_Surgery";
    private const string GET_AD_HOC_ORDERED_PENDING = "bcccr.usp_RptAH_Ordered_Pending";
    private const string GET_AD_HOC_MAMMOGRAM_WITH_FOLLOWUP = "bcccr.usp_RptAH_Mammogram_With_Followup";

    private DataTable _dtRpt = null;
    public DataTable dtRptPD { get { return _dtRpt; } }

    public AdHocR()
    {
    }

    //template
    public DataTable GetReportName(string facList, string pacList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_REPORT_NAME, new object[] { facList });
    }
    //***
    //AdHoc Reports

    public DataTable GetAdHoc(int facId, string ageList, string raceList, string genderList, string biradList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_AD_HOC, new object[] { facId, ageList, raceList, genderList, biradList });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocAllAbnormalMammograms(string patientId)
    {
        //return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_AD_HOC_ALL_ABNORMAL, new object[] { patientId });
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_ALL_ABNORMAL, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocAllMammograms(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_ALL_MAMMOGRAMS, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocAllMammogramsDue(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_ALL_MAMMOGRAMS_DUE, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocBreastBiopsy(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_BREAST_BIOPSY, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocBreastCancer(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_BREAST_CANCER, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocBreastSurgery(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_BREAST_SURGERY, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocOrderedPending(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_ORDERED_PENDING, new object[] { patientId });
    }

    public System.Data.SqlClient.SqlDataReader GetAdHocMammogramWithFollowup(string patientId)
    {
        return SqlProvider.ExecuteSPReader(Globals.GetConnectionName, GET_AD_HOC_MAMMOGRAM_WITH_FOLLOWUP, new object[] { patientId });
    }

    //public DataTable GetAdHocAllAbnormalMammograms(string patientId)
    //{
    //    //return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_AD_HOC_ALL_ABNORMAL, new object[] { patientId });
    //    return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_AD_HOC_ALL_ABNORMAL, new object[] { patientId });
    //}
}

public class Aggregate
{
    #region Member Variables / Constants

    //template 
    private const string GET_REPORT_NAME = "bcccr.usp_RptAGG_Report_Name";
    //***
    private const string GET_ORDERED_PENDING_BIRAD_AGG = "bcccr.usp_RptAGG_Ordered_Pending_By_Birad";

    private const string GET_ALL_ABNORMAL_MAM_AGG = "[BCCCR].[usp_RptAgg_all_abnormal_mam_biop_surg]";
    private const string GET_ALL_MAM_AGG = "bcccr.usp_RptAgg_All_Mammo";
    private const string GET_ALL_MAM_DUE_AGG = "bcccr.usp_RptAgg_All_Mammo_due";
    private const string GET_ALL_MAM_FOLLOW_UP_AGG = "bcccr.usp_RptAgg_All_Mammo_followup";
    private const string GET_ALL_MAM_BY_DIAG_AGG = "bcccr.usp_RptAgg_All_Mammo_icd";
    private const string GET_BREAST_BIOPSIES_AGG = "bcccr.usp_RptAgg_all_biopsies";
    private const string GET_BREAST_SURGERIES_AGG = "bcccr.usp_RptAgg_All_Surgeries";
    private const string GET_BREAST_CANCER_AGG = "bcccr.usp_RptAgg_All_Breast_Cancer";

    // New
    private const string GET_ALL_MAMMO_BIRAD_DIST_AGG = "BCCCR.usp_Agg_Rpt_GetMammoCountBiRADDistribution";
    private const string GET_ALL_MAMMO_BIRAD_DIST_BY_FAC_AGG = "BCCCR.usp_Agg_Rpt_GetMammoCountBiRADDistributionByFac";
    private const string GET_MAMMOGRAM_COMPLIANCE_BY_VISN_AGG = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISN";
    private const string GET_BREAST_CANCER_BY_VISN_AGG = "BCCCR.usp_Agg_Rpt_GetBreastCancerByVISN";
    private const string GET_BREAST_CANCER_BY_STA3N_AGG = "BCCCR.usp_Agg_Rpt_GetBreastCancerBySta3n";

    private const string GET_MAMMO_COMPLIANCE_HISTORICAL_MONTHSIDs = "BCCCR.usp_GetMammoCompHistoryMonthSIDS";
    private const string GET_HISTORICAL_MAMMO_COMP_BY_VISNS = "BCCCR.usp_Get_Historical_Mammogram_Compliance";
    private const string GET_MAMMOGRAM_COMPLIANCE_BY_STA3N_AGG = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompBySta3n";

    private const string GET_MAMMO_COMPLIANCE_BY_AGE = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISNAge"; // charting
    private const string GET_MAMMO_COMPLIANCE_BY_RACE = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISNRace"; // charting

    private const string GET_MAMMO_COMPLIANCE_BY_VISN_AND_AGE = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISNAgeSta3n"; // table
    private const string GET_MAMMO_COMPLIANCE_BY_VISN_AND_RACE = "BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISNRaceSta3n"; // table

    private const string GET_BREAST_CANCER_BY_VISN_AGE = "BCCCR.usp_Agg_Rpt_GetBreastCancerByVISNAge";
    private const string GET_BREAST_CANCER_BY_VISN_RACE = "BCCCR.usp_Agg_Rpt_GetBreastCancerByVISNRace";

    private const string GET_MAMMOGRAM_FOLLOWUP_COMPLIANCE_BY_VISN_AGG = "BCCCR.usp_Agg_Rpt_GetFollowUpScreeningCompByVISN";
    private const string GET_MAMMOGRAM_FOLLOWUP_COMPLIANCE_BY_STA3N_AGG = "BCCCR.usp_Agg_Rpt_GetFollowUpScreeningCompBySta3n";

    private DataTable _dtRpt = null;
    public DataTable dtRptAGG { get { return _dtRpt; } }

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public Aggregate() {}

    #region Legacy Methods

    //template
    public DataTable GetReportName(string facList, string pacList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_REPORT_NAME, new object[] { facList });
    }
    //***

    public DataTable GetAllMam(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate, string ageList, string raceList, string genderList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate, ageList, raceList, genderList });
    }

    public DataTable GetAllMam u ( ool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate, string ageList, string raceList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_DUE_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate, ageList, raceList });
    }

    public DataTable GetAllMamFollowUp(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate, string ageList, string raceList, string genderList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_FOLLOW_UP_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate, ageList, raceList, genderList });
    }

    public DataTable GetAllMamByDiag(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAM_BY_DIAG_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate });
    }

    public DataTable GetBreastBiopsiesAgg(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_BIOPSIES_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate });
    }

    public DataTable GetBreastSurgeriesAgg(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_SURGERIES_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate });
    }

    public DataTable GetBreastCancerAgg(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate, string ageList, string genderList, string raceList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_CANCER_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate, ageList, raceList, genderList });
    }

    public DataTable GetAbnormalAgg(bool all, bool nat, string visnList, string facList, string pacList, string fromDate, string toDate, string ageList, string raceList, string genderList, string biradList)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_ABNORMAL_MAM_AGG, new object[] { all, nat, visnList, facList, pacList, fromDate, toDate, ageList, raceList, genderList, biradList });
    }

    #endregion

    #region New DME Methods

    /// <summary>
    /// GetMammogramsWithBiRadDistrAgg: Calls the BCCCR.usp_Agg_Rpt_GetMammoCountBiRADDistribution stored procedure to return the aggregate report for the passed parameters.
    /// </summary>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <returns></returns>
    public DataTable GetMammogramsWithBiRadDistrAgg(string visnFacilityId, string startDate, string endDate, string ageRestrictString, string searchGender, string raceRestrictString) {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAMMO_BIRAD_DIST_AGG, new object[] { visnFacilityId, startDate, endDate, ageRestrictString, searchGender, raceRestrictString });
    }

    /// <summary>
    /// GetMammogramsWithBiRadDistrByFacilityAgg: Returns the aggregate report for the passed facility.
    /// </summary>
    /// <param name="institutionId"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <returns></returns>
    public DataTable GetMammogramsWithBiRadDistrByFacilityAgg(string institutionId, string startDate, string endDate)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_ALL_MAMMO_BIRAD_DIST_BY_FAC_AGG, new object[] { institutionId, startDate, endDate });
    }

    /// <summary>
    /// GetMammogramComplianceByVISNAgg: Calls the BCCCR.usp_Agg_Rpt_GetMammoScreeningCompByVISN stored procedure to return the aggregate report for the passed parameters.
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceByVISNAgg(string visnIDs, string raceRestrictString, string ageRestrictString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_MAMMOGRAM_COMPLIANCE_BY_VISN_AGG, new object[] { visnIDs, raceRestrictString, ageRestrictString });
    }

    /// <summary>
    /// returns the breakdown of compliance for selected VISNs by Age
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceByVISNAndAgeAgg(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMO_COMPLIANCE_BY_VISN_AND_AGE, new object[] { visnIDs });
    }

    /// <summary>
    /// returns the breakdown of compliance for selected VISNs by Race
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceByVISNAndRaceAgg(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMO_COMPLIANCE_BY_VISN_AND_RACE, new object[] { visnIDs });
    }

    /// <summary>
    /// Returns the MMG Compliance breakdown by age for the selected VISNs
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceByVisnAge(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMO_COMPLIANCE_BY_AGE, new object[] { visnIDs });
    }

    /// <summary>
    /// Returns the MMG Compliance breakdown by race for the selected VISNs
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceByVisnRace(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMO_COMPLIANCE_BY_RACE, new object[] { visnIDs });
    }

    /// <summary>
    /// GetHistoricalMammogramComplianceByVISNAgg: Returns the historical data for the given month and VISN list.
    /// </summary>
    /// <param name="monthSID"></param>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetHistoricalMammogramComplianceByVISNAgg(int monthSID, string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_HISTORICAL_MAMMO_COMP_BY_VISNS, new object[] { monthSID, visnIDs });
    }

    /// <summary>
    /// Get mammogram compliance records per facility
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    public DataTable GetMammogramComplianceBySta3nAgg(string sta3n, string complianceStatus, string ageRestrictString, string raceString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMOGRAM_COMPLIANCE_BY_STA3N_AGG, new object[] { sta3n, complianceStatus, ageRestrictString, raceString });
    }

    /// <summary>
    /// GetBreastCancerByVISNAgg: Calls the BCCCR.usp_Agg_Rpt_GetBreastCancerByVISN stored procedure to return the aggregate report for the passed parameters.
    /// </summary>
    /// <param name="visnFacilityId"></param>
    /// <returns></returns>
    public DataTable GetBreastCancerByVISNAgg(string visnFacilityId, string ageRestrictString, string gender, string race)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_CANCER_BY_VISN_AGG, new object[] { visnFacilityId, ageRestrictString, gender, race });
    }

    /// <summary>
    /// Returns the Breast Cancer breakdown by age for the selected VISNs
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetBreastCancerByVISNAge(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_BREAST_CANCER_BY_VISN_AGE, new object[] { visnIDs });
    }

    /// <summary>
    /// Returns the Breast Cancer breakdown by race for the selected VISNs
    /// </summary>
    /// <param name="visnIDs"></param>
    /// <returns></returns>
    public DataTable GetBreastCancerByVISNRace(string visnIDs)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_BREAST_CANCER_BY_VISN_RACE, new object[] { visnIDs });
    }

    /// <summary>
    /// GetBreastCancerByFacilityIdAgg: Calls the BCCCR.usp_Agg_Rpt_GetBreastCancerByFacId stored procedure to return the aggregate report for the passed parameters.
    /// </summary>
    /// <param name="facilityId"></param>
    /// <returns></returns>
    public DataTable GetBreastCancerBySta3nAgg(string sta3n, string newDx, string ageRestrictString, string searchGender, string raceRestrictString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_BREAST_CANCER_BY_STA3N_AGG, new object[] { sta3n, newDx, ageRestrictString, searchGender, raceRestrictString });
    }

    /// <summary>
    /// GetMammoComplianceHistoricalMonthSIDs: returns a list of distinct MonthSID values for Mammogram Compliance History
    /// </summary>
    /// <returns></returns>
    public DataTable GetMammoComplianceHistoricalMonthSIDs()
    {
        return SqlProvider.ExecuteSPDataTable(Globals.GetConnectionName, GET_MAMMO_COMPLIANCE_HISTORICAL_MONTHSIDs, new object[] { });
    }

    /// <summary>
    /// Returns the data table for follow up compliance for a set of VISNs
    /// </summary>
    /// <param name="visnIds"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <returns></returns>
    public DataTable GetMammogramFollowupComplianceByVISNAgg(string visnIds, string startDate, string endDate, string raceRestrictString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMOGRAM_FOLLOWUP_COMPLIANCE_BY_VISN_AGG, new object[] { visnIds, startDate, endDate, raceRestrictString });
    }

    /// <summary>
    /// Returns the data table for follow up compliance for a facility
    /// </summary>
    /// <param name="sta3n"></param>
    /// <param name="startDate"></param>
    /// <param name="endDate"></param>
    /// <returns></returns>
    public DataTable GetMammogramFollowupComplianceBySta3nAgg(string sta3n, string startDate, string endDate, string raceRestrictString, string followUpStatusString)
    {
        return SqlProvider.ExecuteSPDataTable(Globals.CONNECTION_NAME, GET_MAMMOGRAM_FOLLOWUP_COMPLIANCE_BY_STA3N_AGG, new object[] { sta3n, startDate, endDate, raceRestrictString, followUpStatusString });
    }

    #endregion

}