﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using VeteransAffairs.Registries.BusinessManager;

public partial class RegistryMaster : System.Web.UI.MasterPage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

    /// <summary>
    /// This script sets a focus to the control with a name to which
    /// REQUEST_LASTFOCUS was replaced. Setting focus heppens after the page
    /// (or update panel) was rendered. To delay setting focus the function
    /// window.setTimeout() will be used.
    /// </summary>
    private const string SCRIPT_DOFOCUS =
        @"window.setTimeout('DoFocus()', 1);
        function DoFocus()
        {
            try {
                document.getElementById('REQUEST_LASTFOCUS').focus();
            } catch (ex) {}
        }";

    #endregion
 
    public delegate void MasterPageMenuClickHandler(object sender, MenuEventArgs e);

    public event MasterPageMenuClickHandler MenuClick;
    
    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        AccessControl ac = new AccessControl(); 
        UserAccountManager user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            if (user.UserId < 1)
            {
                Logging.WriteLogToFile(user.UserName + ": Attempting to log in without an account");
            }
            else if (Session["loggedIn"] != "true")
            {
                Logging.WriteLogToFile(user.UserName + ": Logging in with roles: " + user.RoleName);
                Session["loggedIn"] = "true";
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Failed logging user login attempt" + ex.Message);
        }

        if (Convert.ToBoolean(Helpers.GetConfigEntry("DisplayUser")) == true)
        {
            lblCurrentDateTime.Text = DateTime.Now.ToString();
            lblCurrentUser.Text = user.FullName;
            lblRole.Text = user.RoleName;
            Label1.Visible = true;
            Label2.Visible = true;
            lblCurrentDateTime.Visible = true;
            lblCurrentUser.Visible = true;
            lblRole.Visible = true;
            lblRefreshDate.Text = Reports.GetETLRefreshDate().ToString();
        }

        //Set onfocus event to all apropriate controls on the page.
        if (!IsPostBack)
        {
            HookOnFocus(this.Page as Control);

            MenuItem miReporting = new MenuItem("Reporting", "Reporting", null, "~/Reports/Dashboard.aspx");
            MenuHorizontalMenu.Items.Add(miReporting);

            // Only admin users should see the Admin menu option
            if (Helpers.checkAdminAccess())
            {
                MenuItem miAdmin = new MenuItem("Administration", "Administration", null, "~/AdminListUsers.aspx");
                MenuHorizontalMenu.Items.Add(miAdmin);
            }

            MenuItem miHelp = new MenuItem("Help","Help","~/images/help.gif","", "_blank");
            miHelp.NavigateUrl = GetContextSensitiveHelpURL();

            MenuHorizontalMenu.Items.Add(miHelp);
        }
    }

    /// <summary>
    /// Menu Item Click Handler
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void MenuItemClick(object sender, MenuEventArgs e)
    {
        if (MenuClick != null)
        {
            MenuClick(sender, e);
        }
    }

    #region Private Methods

    /// <summary>
    /// Generate context-sensitive help link
    /// </summary>
    /// <returns></returns>
    private string GetContextSensitiveHelpURL()
    {
        return "~/help/" + Page.Page.ToString().Substring(4).Replace("_aspx", ".html");
    }

    /// <summary>
    /// This function goes recursively all child controls and sets 
    /// onfocus attribute if the control has one of defined types.
    /// </summary>
    /// <param name="CurrentControl">the control to hook.</param>
    private void HookOnFocus(Control CurrentControl)
    {
        //checks if control is one of TextBox, DropDownList, ListBox or Button

        if ((CurrentControl is TextBox) ||
            (CurrentControl is DropDownList) ||
            (CurrentControl is ListBox) ||
            (CurrentControl is CheckBoxList) ||
            (CurrentControl is RadioButton) ||
            (CurrentControl is Button))

            (CurrentControl as WebControl).Attributes.Add(
               "onfocus",
               "try{document.getElementById('__LASTFOCUS').value=this.id} catch(e) {}"
            );

        //checks if the control has children
        if (CurrentControl.HasControls())
            //if yes do them all recursively

            foreach (Control CurrentChildControl in CurrentControl.Controls)
                HookOnFocus(CurrentChildControl);
    }

    #endregion

    /// <summary>
    /// Setting Dirty Flag - warn user.
    /// </summary>
    /// <param name="flag"></param>
    public void SetDirtyFlag(bool flag)
    {
        dirtyFlag.Value = flag == true ? "1" : "0";
    }
}