﻿using System;
using System.Text;
using System.Web;
using System.Xml.Linq;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_AdhocReportingAdministrationWizard : BasePage
{
    #region Member Variables

    private string _userId;
    private string _mode;
    private int _reportTemplateId;
    private AdHocReports adhocReports;
    private Reports r;
    private UserAccountManager _user;

    private bool filtersApplied;
    public bool FiltersApplied
    {
        get
        {
            if (ViewState["filtersApplied"] != null)
            {
                return (bool)ViewState["filtersApplied"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            filtersApplied = value;
            ViewState["filtersApplied"] = value;
        }
    }

    private bool isInitialEditLoad;
    public bool IsInitialEditLoad
    {
        get
        {
            if (ViewState["isInitialEditLoad"] != null)
            {
                return (bool)ViewState["isInitialEditLoad"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isInitialEditLoad = value;
            ViewState["isInitialEditLoad"] = value;
        }
    }

    protected AdHocReportTemplate loadedTemplate;

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"./Dashboard.aspx\" title=\"Return to main reporting home page.\" style=\"color:#FFFFFF;\">Reporting Home</a> > <a href=\"./AdhocReportingDashboard.aspx\" title=\"Return to the Ad hoc Reporting Dashboard\" style=\"color:#FFFFFF;\">Ad hoc Dashboard</a> > Ad hoc Reporting Administration (v2)";

        adhocReports = new AdHocReports();

        _user = (UserAccountManager)HttpContext.Current.User;
        _userId = _user.UserId.ToString();

        //set the user ID variable
        r = new Reports(_userId);

        // Added to correct the "missing" datepicker images.
        RadStyleSheetManager.GetCurrent(this.Page).RegisterSkinnableControl(rfConditionalBuilder);

        #region Get Query String Variables

        // get mode and reportTemplateId
        if (!String.IsNullOrEmpty(Request.QueryString.Get("mode")))
        {
            _mode = Request.QueryString.Get("mode");
        }
        else
        {
            throw new Exception("Required Parameter Exception - mode");
        }

        if (!String.IsNullOrEmpty(Request.QueryString.Get("id")))
        {
            if (!Int32.TryParse(Request.QueryString.Get("id"), out _reportTemplateId))
            {
                // failed ---
                throw new Exception("Incorrect Parameter Exception - report id of unknown type");
            }
        }

        #endregion

        #region Security Check for Edit

        // if mode is edit - check to ensure ID belongs to current user.
        if (!String.IsNullOrEmpty(_mode))
        {
            if (_mode.ToLower() == "edit")
            {
                if (_reportTemplateId > 0)
                {
                    // check
                    loadedTemplate = AdHocReports.LoadSingleUserReportTemplate(_user.UserId, _reportTemplateId);

                    if (loadedTemplate != null)
                    {
                        IsInitialEditLoad = true;

                        if (loadedTemplate.UserId != _user.UserId)
                        {
                            Logging.WriteLogToFile("Report Template Access Violation. Report ID: " + _reportTemplateId + " was attemped to be accessed by UserId: " + _user.UserId);

                            // display message to user - TODO
                            throw new Exception("Report Template Access Violation. User does not have access to report");
                        }
                    }
                    else
                    {
                        // throw error
                        Logging.WriteLogToFile("Report Template Loading Error (No Template Found) - ID: " + _reportTemplateId + " - UserId: " + _user.UserId);
                    }
                }
                else
                {
                    // should never get here, but in case, stop processing
                }
            }
            else if (_mode.ToLower() == "clone") 
            {
                if (_reportTemplateId > 0)
                {
                    // check
                    loadedTemplate = AdHocReports.LoadSingleUserReportTemplate(_user.UserId, _reportTemplateId);

                    if (loadedTemplate != null)
                    {
                        IsInitialEditLoad = true;
                    }
                    else
                    {
                        // throw error
                        Logging.WriteLogToFile("Report Template Loading Error (No Template Found) - ID: " + _reportTemplateId + " - UserId: " + _user.UserId);
                    }
                }
            }
        }

        #endregion

        if (!IsPostBack)
        {
            FiltersApplied = false;
            fillAvailableColumns();

            if (loadedTemplate != null)
            {
                IsInitialEditLoad = true;
                hfReportTemplateID.Value = _reportTemplateId.ToString();

                if (_mode.ToLower() == "clone")
                {
                    rtbTemplateName.Text = loadedTemplate.ReportName + " Copy";
                }
                else
                {
                    rtbTemplateName.Text = loadedTemplate.ReportName;
                }

                // Set Join Type
                if (loadedTemplate.JoinType == "LEFT OUTER JOIN")
                {
                    rbLeftOuterJoin.Checked = true;
                    rbInnerJoin.Checked = false;
                }
                else
                {
                    rbLeftOuterJoin.Checked = false;
                    rbInnerJoin.Checked = true;
                }

                populateColumnListSelections();
            }
            else
            {
                // no record found - TODO: write and display message
                Logging.WriteLogToFile("Report Template Loading Error (No Template Found) - ID: " + _reportTemplateId + " - UserId: " + _user.UserId);
            }
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "Ad hoc Reporting Administration";
    }

    public override string AppObjectCode()
    {
        return "Ad hoc Reporting Administration";
    }

    #endregion

    #region GUI Methods

    /// <summary>
    /// Custom code to handle drag / drop
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void RadListBox_Dropped(object sender, RadListBoxDroppedEventArgs e)
    {
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// fill columns from AdHocReports list
    /// </summary>
    private void fillAvailableColumns()
    {
        rlbAvailableColumns.DataSource = adhocReports.ReportColumnList;
        rlbAvailableColumns.DataTextField = "DisplayValue";
        rlbAvailableColumns.DataValueField = "Value";
        rlbAvailableColumns.DataBind();
    }

    /// <summary>
    /// determines which datasource to call ... schema only or with data.
    /// </summary>
    private void detemineDatasource()
    {
        XDocument xDoc = generateTemplateXml();
        string generatedData = AdHocReports.GenerateColumnStringForStoredProc(xDoc);

        string joinType = "";
        if (rbLeftOuterJoin.Checked)
        {
            joinType = "LEFT OUTER JOIN";
        }
        else
        {
            joinType = "INNER JOIN";
        }

        if (IsInitialEditLoad)
        {
            populateLoadedGridSortExpressions();
        }

        if (FiltersApplied)
        {
            string normalizedWhere = NormalizeWhereClause(hfWhereClause.Value);
            rgResultsPreview.DataSource = AdHocReports.GenerateAdhocResults(generatedData, normalizedWhere, generateAccessibleSta3nListForUser(), joinType);
        }
        else
        {
            rgResultsPreview.DataSource = AdHocReports.GenerateAdhocSchema(generatedData, joinType);
        }
    }

    /// <summary>
    /// Adds saved Sort Expressions to grid.
    /// </summary>
    private void populateLoadedGridSortExpressions()
    {
        #region Load and Create Sort Settings

        if (!String.IsNullOrEmpty(loadedTemplate.SortExpressionString))
        {
            string[] sortItems = loadedTemplate.SortExpressionString.Split(';');
            foreach (string item in sortItems)
            {
                string[] sortExpressions = item.Split('|');
                string fieldName = sortExpressions[0];
                string direction = (sortExpressions[1] != null) ? sortExpressions[1] : "ASC";

                rgResultsPreview.MasterTableView.SortExpressions.AddSortExpression(fieldName + " " + direction);
            }
        }

        #endregion
    }

    /// <summary>
    /// Populates the list controls from user selected values stored in the database
    /// </summary>
    private void populateColumnListSelections()
    {
        XDocument myReportDoc = XDocument.Parse(loadedTemplate.ReportColumnsXmlData);

        try
        {
            XElement rootElem = myReportDoc.Element("ReportTemplate");
            XElement reportColElem = rootElem.Element("ReportColumns");

            foreach (XElement subElem in reportColElem.Elements("ReportColumn"))
            {
                // get attributes
                XAttribute displayName = subElem.Attribute("DisplayName");
                XAttribute val = subElem.Attribute("Value");

                // check before adding
                if (rlbSelectedColumns.FindItemByText(displayName.Value) == null)
                {
                    rlbSelectedColumns.Items.Add(new RadListBoxItem() { Value = val.Value, Text = displayName.Value });
                }

                // delete selected from the available
                RadListBoxItem avail = rlbAvailableColumns.FindItemByText(displayName.Value);
                if (avail != null)
                {
                    rlbAvailableColumns.Delete(avail);
                }
            }
        }
        catch (Exception ex)
        {
            // TODO: display error message ... 
            Logging.WriteLogToFile("Error Generating Report Template Schema (populateColumnListSelections): " + ex.Message);
        }
    }

    /// <summary>
    /// Generates the XML for the passed columns.
    /// </summary>
    /// <returns></returns>
    private XDocument generateTemplateXml()
    {
        try
        {
            XDocument xDoc = new XDocument();
            xDoc.Add(new XElement("ReportTemplate"));

            XElement rootElem = xDoc.Element("ReportTemplate");
            rootElem.Add(new XElement("ReportColumns"));
            rootElem.Add(new XElement("GroupByColumns"));
            rootElem.Add(new XElement("WhereClauseColumns"));

            XElement reportColElem = rootElem.Element("ReportColumns");
            XElement groupByColElem = rootElem.Element("GroupByColumns");
            XElement whereClauseColElem = rootElem.Element("WhereClauseColumns");

            foreach (RadListBoxItem rlbi in rlbSelectedColumns.Items)
            {
                XElement newElem = new XElement("ReportColumn");
                newElem.Add(new XAttribute("DisplayName", rlbi.Text));
                newElem.Add(new XAttribute("Value", rlbi.Value));

                reportColElem.Add(newElem);
            }


            return xDoc;
        }
        catch (Exception ex)
        {
            // TODO: display error message ... 

            Logging.WriteLogToFile("Error Creating Report Template: " + ex.Message);

            return null;
        }
    }

    /// <summary>
    /// Converts a telerik where clause format into a sql format
    /// Handles Contains, Does Not Contain, Starts With, and Ends With telerik options
    /// </summary>
    /// <param name="whereClauseVal"></param>
    /// <returns></returns>
    private string NormalizeWhereClause(string whereClauseVal)
    {
        string whereClause = whereClauseVal;
        string fixedWhereClause = whereClauseVal;

        // correction for CancerDx.Diagnosis Date ... change to CancerDx.[Diagnosis Date]
        fixedWhereClause = fixedWhereClause.Replace("CancerDx.Diagnosis Date", "CancerDx.[Diagnosis Date]");
        fixedWhereClause = fixedWhereClause.Replace("SurgBiop.Procedure", "SurgBiop.[Procedure]");

        fixedWhereClause = fixedWhereClause.Replace("'no'", "'0'");
        fixedWhereClause = fixedWhereClause.Replace("'yes'", "'1'");
        fixedWhereClause = fixedWhereClause.Replace("'No'", "'0'");
        fixedWhereClause = fixedWhereClause.Replace("'Yes'", "'1'");

        System.Text.RegularExpressions.Regex containsRegex = new System.Text.RegularExpressions.Regex(@"Contains '(.*?)'");
        System.Text.RegularExpressions.Regex notContainRegex = new System.Text.RegularExpressions.Regex(@"Does Not Contain '(.*?)'");
        System.Text.RegularExpressions.Regex startWithRegex = new System.Text.RegularExpressions.Regex(@"Starts With '(.*?)'");
        System.Text.RegularExpressions.Regex endWithRegex = new System.Text.RegularExpressions.Regex(@"Ends With '(.*?)'");
        System.Text.RegularExpressions.Regex insideQuoteRegex = new System.Text.RegularExpressions.Regex(@"'(.*?)'");
        System.Text.RegularExpressions.Match itemMatch = null;

        //Process for "Contains" items
        foreach (System.Text.RegularExpressions.Match item in containsRegex.Matches(whereClause))
        {
            if (!string.IsNullOrEmpty(item.Value))
            {
                itemMatch = insideQuoteRegex.Match(item.Value);
                fixedWhereClause = fixedWhereClause.Replace(item.Value, " LIKE '%" + itemMatch.Value.Substring(1, itemMatch.Value.Length - 2) + "%' ");
            }
        }

        //Process for "Does Not Contain" items
        foreach (System.Text.RegularExpressions.Match item in notContainRegex.Matches(whereClause))
        {
            if (!string.IsNullOrEmpty(item.Value))
            {
                itemMatch = insideQuoteRegex.Match(item.Value);
                fixedWhereClause = fixedWhereClause.Replace(item.Value, " NOT LIKE '%" + itemMatch.Value.Substring(1, itemMatch.Value.Length - 2) + "%' ");
            }
        }

        //Process for "Starts With" items
        foreach (System.Text.RegularExpressions.Match item in startWithRegex.Matches(whereClause))
        {
            if (!string.IsNullOrEmpty(item.Value))
            {
                itemMatch = insideQuoteRegex.Match(item.Value);
                fixedWhereClause = fixedWhereClause.Replace(item.Value, " LIKE '" + itemMatch.Value.Substring(1, itemMatch.Value.Length - 2) + "%' ");
            }
        }

        //Process for "Ends With" items
        foreach (System.Text.RegularExpressions.Match item in endWithRegex.Matches(whereClause))
        {
            if (!string.IsNullOrEmpty(item.Value))
            {
                itemMatch = insideQuoteRegex.Match(item.Value);
                fixedWhereClause = fixedWhereClause.Replace(item.Value, " LIKE '%" + itemMatch.Value.Substring(1, itemMatch.Value.Length - 2) + "' ");
            }
        }

        return fixedWhereClause;
    }

    /// <summary>
    /// generates schema from current selections.
    /// </summary>
    private void generateSchema()
    {
        XDocument xDoc = generateTemplateXml();

        // generate sql ... call sproc that returns 0 rows for passed sql params ... just to get column listing, etc.
        string generatedData = AdHocReports.GenerateColumnStringForStoredProc(xDoc);

        string joinType = "";
        if (rbLeftOuterJoin.Checked)
        {
            joinType = "LEFT OUTER JOIN";
        }
        else
        {
            joinType = "INNER JOIN";
        }

        rgResultsPreview.DataSource = AdHocReports.GenerateAdhocSchema(generatedData, joinType);
        rgResultsPreview.DataBind();
    }

    /// <summary>
    /// save entry
    /// </summary>
    private void saveTemplateEntry()
    {
        try
        {
            // get all information ... including sorts ...
            XDocument savedDoc = generateTemplateXml();
            string xmlData = savedDoc.ToString();
            string whereSettings = rfConditionalBuilder.SaveSettings();
            string joinType = "";

            // get sort settings ... 
            GridSortExpressionCollection sortCollection = rgResultsPreview.MasterTableView.SortExpressions;

            StringBuilder sortColStr = new StringBuilder();

            foreach (GridSortExpression se in sortCollection)
            {
                sortColStr.Append(se.FieldName + "|" + se.SortOrderAsString() + ";");
            }

            string sortExpressionStr = sortColStr.ToString();

            if (!String.IsNullOrEmpty(sortExpressionStr))
            {
                if (sortExpressionStr.EndsWith(";"))
                    sortExpressionStr = sortExpressionStr.Substring(0, sortExpressionStr.Length - 1);
            }

            #region JOIN TYPE

            if (rbLeftOuterJoin.Checked)
            {
                joinType = "LEFT OUTER JOIN";
            }
            else
            {
                joinType = "INNER JOIN";
            }

            #endregion

            if (_mode == "create")
            {
                int returnVal = AdHocReports.InsertUpdateAdhocReportTemplate(0, _user.UserId, rtbTemplateName.Text, xmlData, whereSettings, NormalizeWhereClause(hfWhereClause.Value), sortExpressionStr, joinType, chkIsPublicTemplate.Checked);

                if (returnVal > 0)
                {
                    //hfReportTemplateID.Value = returnVal;
                    //Response.Redirect("AdhocReportingDashboard.aspx");
                }
                else
                {
                    // Error
                    Logging.WriteLogToFile("Error Saving Report Template");
                }
            }
            else if (_mode == "edit")
            {
                int returnVal = AdHocReports.InsertUpdateAdhocReportTemplate(_reportTemplateId, _user.UserId, rtbTemplateName.Text, xmlData, whereSettings, NormalizeWhereClause(hfWhereClause.Value), sortExpressionStr, joinType, chkIsPublicTemplate.Checked);

                if (returnVal > 0)
                {
                    //hfReportTemplateID.Value = returnVal;
                    //Response.Redirect("AdhocReportingDashboard.aspx");
                }
                else
                {
                    // Error - TODO - Display to user
                    Logging.WriteLogToFile("Error Saving Report Template (Update)");
                }
            }
            else if (_mode == "clone")
            {
                int returnVal = AdHocReports.InsertUpdateAdhocReportTemplate(0, _user.UserId, rtbTemplateName.Text, xmlData, whereSettings, NormalizeWhereClause(hfWhereClause.Value), sortExpressionStr, joinType, chkIsPublicTemplate.Checked);

                if (returnVal > 0)
                {
                    //hfReportTemplateID.Value = returnVal;
                    //Response.Redirect("AdhocReportingDashboard.aspx");
                }
                else
                {
                    // Error
                    Logging.WriteLogToFile("Error Saving Report Template");
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Saving Report Template: " + ex.Message);
            // show error panel.
        }
    }

    /// <summary>
    /// returns a string list of accessible facilities for user.
    /// </summary>
    /// <returns></returns>
    private string generateAccessibleSta3nListForUser()
    {
        string accessibleFacilities = "";

        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    accessibleFacilities += dr["sta3n"].ToString() + ",";
                }
            }
        }

        if (!String.IsNullOrEmpty(accessibleFacilities))
        {
            if (accessibleFacilities.EndsWith(","))
            {
                // remove last comma
                accessibleFacilities = accessibleFacilities.Substring(0, accessibleFacilities.Length - 1);
            }
        }

        return accessibleFacilities;
    }

    #endregion

    #region Result Grid Events

    /// <summary>
    /// Event fired when rgResultsPreview needs a datasource.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgResultsPreview_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
    {
        try
        {
            detemineDatasource(); // private method
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Item Data Bound
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgResultsPreview_ItemDataBound(object sender, GridItemEventArgs e)
    {
        //if (e.Item is GridDataItem)
        //{
        //    GridDataItem gdi = e.Item as GridDataItem;
        //    object test = gdi.DataItem;
        //    GridItemType git = gdi.ItemType;
        //}
    }

    /// <summary>
    /// Item Created
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgResultsPreview_ItemCreated(object sender, GridItemEventArgs e)
    {
    }

    /// <summary>
    /// On Grid Data Bound ... if initial edit load ... bind the load settings.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgResultsPreview_DataBound(object sender, EventArgs e)
    {
        //// check for any sta3n columns
        //foreach (DataColumn dc in ((DataTable)rgResultsPreview.DataSource).Columns)
        //{
        //    if (dc.ColumnName.ToLower().Contains("sta3n"))
        //    {
        //        // limit results for user to only include facilities they have access to
        //        foreach (DataRow dr in ((DataTable)rgResultsPreview.DataSource).Rows)
        //        {
        //            if (!UserPermittedToViewFacility(dr[dc.ColumnName].ToString()))
        //            {
        //                dr.Delete();
        //            }
        //        }
        //    }
        //}

        //rgResultsPreview.Rebind();
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Filter Events

    /// <summary>
    /// Fires when the Apply button clicked on Filter section
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rfConditionalBuilder_ApplyExpressions(object sender, RadFilterApplyExpressionsEventArgs e)
    {
        if (!String.IsNullOrEmpty(hfWhereClause.Value))
        {
            FiltersApplied = true;

            // Call stored procedure ... and bind to grid ... show.
            //pnlResultsView.Visible = true;

            // will only show columns. -- display label
            //pnlOnlyShowingSchema.Visible = false;
            //pnlEnterFilters.Visible = false;

            RadFilterSqlQueryProvider queryProvider = new RadFilterSqlQueryProvider();
            queryProvider.ProcessGroup(e.ExpressionRoot);

            //rgResultsPreview.MasterTableView.FilterExpression = queryProvider.Result;
            //rgResultsPreview.MasterTableView.Rebind();
            //rgResultsPreview.DataBind();

            // Enable Save Button
            //rbSaveReport.Enabled = true;
        }
        else
        {
            FiltersApplied = false;

            //pnlOnlyShowingSchema.Visible = true;
            //pnlEnterFilters.Visible = true;

            //rbSaveReport.Enabled = false;
        }
    }

    /// <summary>
    /// Fired when an expression is added.  Set FiltersUpdated to true.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rfConditionalBuilder_ExpressionItemCreated(object sender, RadFilterExpressionItemCreatedEventArgs e)
    {
        //rbExecuteQuery.Enabled = false;
        if (rwReportTemplateWizard.ActiveStepIndex == 3)
        {
            FiltersApplied = false;
            rwReportTemplateWizard.DisplayNavigationButtons = FiltersApplied;
        }
    }

    /// <summary>
    /// Apply filters forces postback to update the expression preview and enables the Execute Button.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbApplyFilters_Click(object sender, EventArgs e)
    {
        if (!String.IsNullOrEmpty(hfWhereClause.Value))
        {
            FiltersApplied = true;
        }
        else
        {
            FiltersApplied = false;
        }

        rwReportTemplateWizard.DisplayNavigationButtons = FiltersApplied;
    }

    #endregion

    #region Wizard Events

    /// <summary>
    /// Active step changing on the wizard control
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rwReportTemplateWizard_ActiveStepChanged(object sender, EventArgs e)
    {
        // perform check to see if fields required to get to the requested step have been fulfilled ... if not ... direct back to step needed.
        rbApplyFilters.Enabled = false;

        // if filter step ... ensure filters have been applied
        if (rwReportTemplateWizard.ActiveStepIndex == 3)
        {
            rbApplyFilters.Enabled = true;

            if (IsInitialEditLoad && loadedTemplate != null)
            {
                // load settings for filter
                rfConditionalBuilder.LoadSettings(loadedTemplate.WhereDataExpressions);
                IsInitialEditLoad = false;
            }

            //rwReportTemplateWizard.ActiveStep
            FiltersApplied = false;
            rwReportTemplateWizard.DisplayNavigationButtons = FiltersApplied;
        }
        else if (rwReportTemplateWizard.ActiveStepIndex == 4)
        {
            // if preview screen - if admin show pnlPublicTemplate
            if (Helpers.checkAdminAccess())
            {
                if (_mode.ToLower() == "clone")
                {
                    chkIsPublicTemplate.Checked = false;
                }
                else
                {
                    if (loadedTemplate != null)
                    {
                        chkIsPublicTemplate.Checked = loadedTemplate.IsSystemTemplate;
                    }
                }
                pnlPublicTemplate.Visible = true;
            }
            else
            {
                chkIsPublicTemplate.Checked = false;
                pnlPublicTemplate.Visible = false;
            }

            rwReportTemplateWizard.DisplayNavigationButtons = true;
        }
        else
        {
            rwReportTemplateWizard.DisplayNavigationButtons = true;
        }
    }

    /// <summary>
    /// Should not get to this step (client script cancels), but in case - will set step to current step
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rwReportTemplateWizard_NavigationBarButtonClick(object sender, WizardEventArgs e)
    {
        // do nothing ...
        rwReportTemplateWizard.ActiveStepIndex = e.CurrentStepIndex;
    }

    /// <summary>
    /// Redirect to Ad Hoc Dashboard.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rwReportTemplateWizard_CancelButtonClick(object sender, WizardEventArgs e)
    {
        // Redirect to dashboard.
        Response.Redirect("AdhocReportingDashboard.aspx");
    }

    /// <summary>
    /// Next button firing ... perform required actions before moving to next step.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rwReportTemplateWizard_NextButtonClick(object sender, WizardEventArgs e)
    {
        // capture what the current tab is ... and where we're going to determine what functions need to occur.
        switch (e.CurrentStepIndex) {
            case 0:
                break;
            case 1:
                break;
            case 2:
                // current step - select columns ... generate schema before moving forward.
                generateSchema();
                break;
            case 3:
                // ensure filters have been applied
                FiltersApplied = true;
                rfConditionalBuilder.FireApplyCommand();
                break;
            case 4:
                // perform save ...if fail - show error
                saveTemplateEntry();
                break;
            default:
                break;
        }
    }

    /// <summary>
    /// FieldEditor Binding
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rfConditionalBuilder_FieldEditorCreated(object sender, RadFilterFieldEditorCreatedEventArgs e)
    {
        lblSelectedColumnInstructions.Text = "";

        foreach (RadFilterDataFieldEditor ed in rfConditionalBuilder.FieldEditors)
        {
            string displayName = adhocReports.GetDisplayNameAndInstructionsFromFieldName(ed.FieldName);
            if (!String.IsNullOrEmpty(displayName))
            {
                lblSelectedColumnInstructions.Text += displayName + "<br /><br />";
            }
        }

        //string displayName = adhocReports.GetDisplayNameFromFieldName(e.Editor.FieldName);
        //e.Editor.DisplayName = (String.IsNullOrEmpty(displayName)) ? e.Editor.FieldName : displayName;
        //e.Editor.FieldName = e.Editor.FieldName;
    }

    #endregion
}