﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using System.Xml.Linq;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_AdhocReportingDashboard : BasePage
{

    #region Intialize Variables

    private string _userId;
    private UserAccountManager _user;
    private Reports r;

    protected string reportName = "Custom Ad Hoc Report";

    bool isExport = false;

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsTemplatePDFExport"] != null)
            {
                return (bool)ViewState["IsTemplatePDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsTemplatePDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            LabelBreadCrumb.Text = "<a href=\"./Dashboard.aspx\" title=\"Return to main reporting home page.\" style=\"color:#FFFFFF;\">Reporting Home</a> > Ad hoc Dashboard";

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            _user = (UserAccountManager)HttpContext.Current.User;
            _userId = _user.UserId.ToString();

            //set the user ID variable
            r = new Reports(_userId);

            if (!String.IsNullOrEmpty(hfCustomReportName.Value))
            {
                reportName = hfCustomReportName.Value;
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "Ad hoc Reporting Dashboard";
    }

    public override string AppObjectCode()
    {
        return "Ad hoc Reporting Dashboard";
    }

    #endregion

    #region MyTemplate Grid Events

    protected void rgMyTemplates_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        fillTemplateGrid();
    }

    protected void rgMyTemplates_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton cloneButton = (LinkButton)item["CloneButtonGUI"].Controls[0];
                LinkButton editButton = (LinkButton)item["EditButtonGUI"].Controls[0];
                LinkButton deleteButton = (LinkButton)item["DeleteButtonGUI"].Controls[0];

                bool isSystemTemplate = (bool)item.GetDataKeyValue("IsSystemTemplate");

                cloneButton.Visible = isSystemTemplate;
                editButton.Visible = !isSystemTemplate;
                deleteButton.Visible = !isSystemTemplate;

                if (_userId == item.GetDataKeyValue("UserID").ToString())
                {
                    editButton.Visible = true;
                    deleteButton.Visible = true;
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    protected void rgMyTemplates_ItemCreated(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
    }

    /// <summary>
    /// Grid Command
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMyTemplates_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        int reportId;

        if (e.CommandName == "ExecuteReport")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];
                hfReportId.Value = reportId.ToString();

                pnlPreviewArea.Visible = true;

                loadReportTemplate(reportId);
                // load data ... show grid, bind
            }
        }
        else if (e.CommandName == "DeleteReport")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];

                try
                {
                    AdHocReports.DeleteUserReportingTemplate(_user.UserId, reportId);
                    fillTemplateGrid();
                    rgMyTemplates.DataBind();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
        else if (e.CommandName == "EditReport")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];

                Response.Redirect("AdhocReportingAdministrationWizard.aspx?mode=edit&id=" + reportId);
            }
        }
        else if (e.CommandName == "CloneReport")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                reportId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];

                Response.Redirect("AdhocReportingAdministrationWizard.aspx?mode=clone&id=" + reportId);
            }
        }
    }

    #endregion

    #region Results Grid Events

    protected void rgResults_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        rgResults.DataSource = "";
    }

    protected void rgResults_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {

    }

    protected void rgResults_ItemCreated(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgResults_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        if (!String.IsNullOrEmpty(hfReportId.Value))
        {
            int loadedId = Convert.ToInt32(hfReportId.Value);
            executeSortExpression(loadedId, e.SortExpression, e.NewSortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    #endregion

    #region UI Events

    /// <summary>
    /// Create New Template Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCreateNewTemplate_Click(object sender, EventArgs e)
    {
        //Response.Redirect("AdhocReportingAdministration.aspx?mode=create");
        Response.Redirect("AdhocReportingAdministrationWizard.aspx?mode=create");
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        if (!String.IsNullOrEmpty(hfReportId.Value) && hfCustomSortApplied.Value == "0")
        {
            //loadReportTemplate(Int32.Parse(hfReportId.Value));
            GridSortExpression gse = rgResults.MasterTableView.SortExpressions[0];
            if (gse != null)
            {
                executeSortExpression(Int32.Parse(hfReportId.Value), gse.FieldName, gse.SortOrder);
            }
        }
        else if (!String.IsNullOrEmpty(hfReportId.Value) && hfCustomSortApplied.Value == "1")
        {
            GridSortExpression gse = rgResults.MasterTableView.SortExpressions[0];
            if (gse != null)
            {
                executeSortExpression(Int32.Parse(hfReportId.Value), gse.FieldName, gse.SortOrder);
            }
        }

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgResults.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                IsPDFExport = false;
                rgResults.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                IsPDFExport = true;
                rgResults.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                IsPDFExport = false;
                rgResults.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgResults.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgResults.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgResults.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgResults.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgResults.MasterTableView.AllowPaging = false;
        rgResults.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgResults.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgResults.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues().Replace("<", "&lt;").Replace(">", "&gt;");
        rgResults.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";

        rgResults.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value.Replace("<", "&lt;").Replace(">", "&gt;");
        rgResults.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgResults.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgResults.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgResults.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgResults.MasterTableView.Items.Count.ToString();

        rgResults.ExportSettings.Pdf.ForceTextWrap = true;

        if (IsPDFExport)
        {
            rgResults.MasterTableView.Caption = reportHeaderString.ToString().Replace("<", "&lt;").Replace(">", "&gt;");
        }
        else
        {
            rgResults.MasterTableView.Caption = reportHeaderString.ToString();
        }
        rgResults.Rebind();
        rgResults.ExportSettings.FileName = hfCustomReportName.Value;
        rgResults.ExportSettings.IgnorePaging = true;
        rgResults.ExportSettings.OpenInNewWindow = true;
        rgResults.ExportSettings.HideStructureColumns = true;

        rgResults.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgResults.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgResults.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgResults.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgResults.Rebind();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Fill the Template Grid
    /// </summary>
    private void fillTemplateGrid()
    {
        try
        {
            rgMyTemplates.DataSource = AdHocReports.LoadAllUserReportTemplates(_user.UserId);
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Loading Report Templates for User (" + _user.UserId + ") " + ex.Message);
        }
    }

    /// <summary>
    /// loads report template for passed id.
    /// </summary>
    /// <param name="id"></param>
    private void loadReportTemplate(int id)
    {
        try
        {
            rgResults.MasterTableView.SortExpressions.Clear();
            DateTime start = DateTime.Now;

            AdHocReportTemplate myTemplate = AdHocReports.LoadSingleUserReportTemplate(_user.UserId, id);

            if (myTemplate != null)
            {
                reportName = myTemplate.ReportName;

                XDocument myReportDoc = XDocument.Parse(myTemplate.ReportColumnsXmlData);

                string reportCols = AdHocReports.GenerateColumnStringForStoredProc(myReportDoc);

                //rgResults.DataSource = AdHocReports.GenerateAdhocResults(reportCols, myTemplate.WhereSQLString, generateAccessibleSta3nListForUser(), myTemplate.JoinType);

                #region Load and Create Sort Settings

                if (!String.IsNullOrEmpty(myTemplate.SortExpressionString))
                {
                    string[] sortItems = myTemplate.SortExpressionString.Split(';');
                    foreach (string item in sortItems)
                    {
                        string[] sortExpressions = item.Split('|');
                        string fieldName = sortExpressions[0];
                        string direction = (sortExpressions[1] != null) ? sortExpressions[1] : "ASC";
                        GridSortOrder gso;

                        if (direction == "ASC") {
                            gso = GridSortOrder.Ascending;
                        } else if (direction == "DESC") {
                            gso = GridSortOrder.Descending;
                        } else {
                            gso = GridSortOrder.None;
                        }

                        rgResults.MasterTableView.SortExpressions.AddSortExpression(fieldName + " " + direction);
                        executeSortExpression(id, fieldName, gso);
                        break;
                    }
                }

                #endregion

                hfExportWhere.Value = myTemplate.WhereSQLString;

                //rgResults.DataBind();

                DateTime end = DateTime.Now;
                lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";

                // Update Execution DateTime.
                AdHocReports.UpdateLastExecutionDate(id, DateTime.Now);
                fillTemplateGrid();
                rgMyTemplates.DataBind();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Loading Reporting Template (ID: " + id + " - User: " + _user.UserId + ") - " + ex.Message);
        }
    }

    /// <summary>
    /// returns a string list of accessible facilities for user.
    /// </summary>
    /// <returns></returns>
    private string generateAccessibleSta3nListForUser()
    {
        string accessibleFacilities = "";

        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    accessibleFacilities += dr["sta3n"].ToString() + ",";
                }
            }
        }

        if (!String.IsNullOrEmpty(accessibleFacilities))
        {
            if (accessibleFacilities.EndsWith(","))
            {
                // remove last comma
                accessibleFacilities = accessibleFacilities.Substring(0, accessibleFacilities.Length - 1);
            }
        }

        return accessibleFacilities;
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        if (!String.IsNullOrEmpty(hfExportWhere.Value))
        {
            return hfExportWhere.Value;
        }
        else
        {
            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(int loadedId, string sortExpression, GridSortOrder newSortOrder)
    {
        AdHocReportTemplate myTemplate = AdHocReports.LoadSingleUserReportTemplate(_user.UserId, loadedId);

        if (myTemplate != null)
        {
            reportName = myTemplate.ReportName;
            XDocument myReportDoc = XDocument.Parse(myTemplate.ReportColumnsXmlData);
            string reportCols = AdHocReports.GenerateColumnStringForStoredProc(myReportDoc);
            hfExportWhere.Value = myTemplate.WhereSQLString;

            DataTable myDT = AdHocReports.GenerateAdhocResults(reportCols, myTemplate.WhereSQLString, generateAccessibleSta3nListForUser(), myTemplate.JoinType);

            if (myDT != null)
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        myDT.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        myDT.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgResults.DataSource = myDT.DefaultView.ToTable();
            rgResults.DataBind();

            hfCustomSortApplied.Value = "1";
        }
    }

    #endregion
}