﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_BreastBiospies_Surgeries_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;

    protected string reportName = "Breast Biopsies and Surgeries Report";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsPDFExport"] != null)
            {
                return (bool)ViewState["IsPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > Breast Biopsies / Surgeries";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            if (!IsPostBack)
            {
                //set date
                rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
                rdpEndDate.SelectedDate = DateTime.Today;
                rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

                rgBreastBiopsiesSurgeries.DataSource = "";
                rgBreastBiopsiesSurgeries.DataBind();

                //Fill the facilities listbox
                FillFACs();

                // Fill the Race list
                FillRaces();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }
    
    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        rgBreastBiopsiesSurgeries.DataSource = getDataset();
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            //get the user's facilities and populate listboxs
            if (r.GetFacilitiesByUserId())
            {
                // Station drop down clear existing items

                rcbFac.Items.Clear();
                rcbFac.EmptyMessage = "Select a Station";

                // VISN drop down clear existing items

                rcbVisn.Items.Clear();
                rcbVisn.EmptyMessage = "Select a VISN";

                System.Data.DataTable tVisns = new DataTable();

                SetupVisnList(tVisns);

                DataView vw = new DataView(tVisns);
                vw.Sort = "[visnId] asc";

                rcbVisn.DataSource = vw;
                rcbVisn.DataTextField = "visnText";
                rcbVisn.DataValueField = "visnId";
                rcbVisn.DataBind();

                // If there's only one option, select it for them
                if (rcbVisn.Items.Count == 1)
                {
                    rcbVisn.SelectedIndex = 0;
                    rcbVisn.Enabled = false;
                    ProcessVisnSelection();
                }
                else if (rcbVisn.Items.Count == 0)
                {
                    rcbVisn.EmptyMessage = "You do not have access to any VISNs to select";
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Populates VISN list based on the facilities that a user has access to
    /// </summary>
    /// <param name="tVisns"></param>
    private void SetupVisnList(System.Data.DataTable tVisns)
    {
        try
        {
            DataColumn idColumn = new DataColumn();
            idColumn = new DataColumn();
            idColumn.DataType = System.Type.GetType("System.Int32");
            idColumn.ColumnName = "visnId";
            idColumn.ReadOnly = true;
            idColumn.Unique = true;

            DataColumn nameColumn = new DataColumn();
            nameColumn = new DataColumn();
            nameColumn.DataType = System.Type.GetType("System.String");
            nameColumn.ColumnName = "visnText";
            nameColumn.ReadOnly = true;
            nameColumn.Unique = true;

            tVisns.Columns.Add(idColumn);
            tVisns.Columns.Add(nameColumn);

            tVisns.PrimaryKey = new DataColumn[] { tVisns.Columns["visnId"] };

            DataRow drVisn;

            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (!tVisns.Rows.Contains(dr["visnId"]))
                {
                    drVisn = tVisns.NewRow();
                    drVisn["visnId"] = dr["visnId"];
                    drVisn["visnText"] = dr["visnText"];

                    tVisns.Rows.Add(drVisn);
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        PatientData pd = new PatientData();

        try
        {
            if (rcbFac.SelectedItem != null)
            {
                //get the report data
                string searchPatName = String.IsNullOrEmpty(rtbPatientName.Text) ? null : rtbPatientName.Text;
                string searchProviderName = String.IsNullOrEmpty(rtbProviderName.Text) ? null : rtbProviderName.Text;

                string ageRestrictString = "";
                ageRestrictString = (String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) ? null : rddlAgeRestriction.SelectedValue);

                string searchGender = null;
                if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
                {
                    searchGender = null;
                }
                else
                {
                    searchGender = rcbGender.SelectedValue;
                }

                string searchReportType = null;
                if (String.IsNullOrEmpty(rddlReportType.SelectedValue) || rddlReportType.SelectedValue == "All")
                {
                    searchReportType = null;
                }
                else
                {
                    searchReportType = rddlReportType.SelectedValue;
                }

                string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

                // follow-up action
                string followUpAction = null;
                if (String.IsNullOrEmpty(rcbFollowUpActionStatus.SelectedValue) || rcbFollowUpActionStatus.SelectedValue == "All")
                {
                    followUpAction = null;
                }
                else
                {
                    followUpAction = rcbFollowUpActionStatus.SelectedValue;
                }

                //PACT TEAM STRING CREATION
                var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
                string pactTeamString = "";

                if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
                {
                    pactTeamString = null; // all
                }
                else
                {
                    foreach (var entry in pactTeamList)
                    {
                        pactTeamString += entry.Value + ",";
                    }

                    if (pactTeamString.EndsWith(","))
                    {
                        pactTeamString = pactTeamString.Substring(0, pactTeamString.Length - 1);
                    }

                    if (String.IsNullOrEmpty(pactTeamString))
                        pactTeamString = null;
                }

                string procedureCodeSearch = (!String.IsNullOrEmpty(rtbProcedureCode.Text) ? rtbProcedureCode.Text : null);

                DateTime tempStartDate = (DateTime)rdpStartDate.SelectedDate;
                DateTime tempEndDate = (DateTime)rdpEndDate.SelectedDate;

                DataTable dt = pd.GetBreastBiopsiesSurgeries(rcbFac.SelectedValue, tempStartDate.ToShortDateString(), tempEndDate.ToShortDateString(), searchPatName, ageRestrictString, searchGender, searchReportType, searchProviderName, raceRestrictString, followUpAction, procedureCodeSearch, pactTeamString);

                //bind the data to the grid
                return dt;
            }
            //if no selected facility
            return null;
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            //if error
            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(string sortExpression, GridSortOrder newSortOrder)
    {
        DataTable dt = getDataset();
        if (dt != null)
        {
            if (sortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (sortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgBreastBiopsiesSurgeries.DataSource = dt.DefaultView.ToTable();
            rgBreastBiopsiesSurgeries.DataBind();
        }
    }

    /// <summary>
    /// reset filters (non-location)
    /// </summary>
    private void clearNonFacilityFilters()
    {
        rtbPatientName.Text = "";
        rcbGender.SelectedIndex = 0;
        rtbProviderName.Text = "";
        rcbRace.ClearSelection();
        rddlAgeRestriction.ClearSelection();
        rddlReportType.ClearSelection();
        rtbProcedureCode.Text = "";
        rcbFollowUpActionStatus.ClearSelection();

        rcbPACTTeamsForFacility.ClearCheckedItems();
        rcbPACTTeamsForFacility.ClearSelection();
        rcbPACTTeamsForFacility.DataSource = "";
        rcbPACTTeamsForFacility.DataBind();
        rcbPACTTeamsForFacility.Enabled = false;
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISN - rcbVisn
        if (rcbVisn.SelectedValue != null)
        {
            sb.Append(" VISN: " + rcbVisn.SelectedItem.Text + " |");
        }

        // Facility - rcbFac
        if (rcbFac.SelectedValue != null)
        {
            sb.Append(" Facility: " + rcbFac.SelectedItem.Text + " |");
        }

        // PACT Team List - rcbBiRadsList
        var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
        string tempPACTString = "";

        if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
        {
            tempPACTString = null; // all
        }
        else
        {
            foreach (var entry in pactTeamList)
            {
                tempPACTString += entry.Text + ",";
            }

            if (tempPACTString.EndsWith(","))
            {
                tempPACTString = tempPACTString.Substring(0, tempPACTString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempPACTString))
                tempPACTString = null;
        }

        if (String.IsNullOrEmpty(tempPACTString))
        {
            sb.Append(" PACT TEAM: All |");
        }
        else
        {
            sb.Append(" PACT TEAM: " + tempPACTString + " |");
        }

        // Start Date - rdpStartDate
        if (rdpStartDate.SelectedDate != null)
        {
            sb.Append(" Start Date: " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + " |");
        }

        // End Date - rdpEndDate
        if (rdpEndDate.SelectedDate != null)
        {
            sb.Append(" End Date: " + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        }

        // Type
        sb.Append(" Biopsy vs Surgery: " + ((String.IsNullOrEmpty(rddlReportType.SelectedValue) || rddlReportType.SelectedValue == "All") ? "All |" : rddlReportType.SelectedValue + " |"));

        // Age Restriction - rddlAgeRestriction
        sb.Append(" Ages: " + ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? "All |" : rddlAgeRestriction.SelectedValue + " |"));

        // Patient Name / SSN - rtbPatientName
        if (!String.IsNullOrEmpty(rtbPatientName.Text))
        {
            sb.Append(" Patient Name/SSN: " + rtbPatientName.Text + " |");
        }

        // Gender - rcbGender
        if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
        {
            sb.Append(" Gender: All |");
        }
        else
        {
            sb.Append(" Gender: " + rcbGender.SelectedValue + " |");
        }

        // Race - rcbRace
        sb.Append(" Race: " + ((rcbRace.SelectedValue == "All") ? "All |" : rcbRace.SelectedValue + " |"));

        // Provider - rtbProviderName
        if (!String.IsNullOrEmpty(rtbProviderName.Text))
        {
            sb.Append(" Provider: " + rtbProviderName.Text + " |");
        }

        // Follow-Up Action - rtbFollowUpAction - TODO

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Breast Biopsies / Surgeries Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgBreastBiopsiesSurgeries_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgBreastBiopsiesSurgeries_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        try
        {
            //if (e.CommandName == RadGrid.FilterCommandName)
            //{
            //    Pair filterPair = (Pair)e.CommandArgument;

            //    switch (filterPair.Second.ToString())
            //    {
            //        case "date_of_last_mammogram":
            //            this.startDate = ((e.Item as GridFilteringItem)[filterPair.Second.ToString()].FindControl("FromDatePicker") as RadDatePicker).SelectedDate;
            //            this.endDate = ((e.Item as GridFilteringItem)[filterPair.Second.ToString()].FindControl("ToDatePicker") as RadDatePicker).SelectedDate;
            //            break;
            //        default:
            //            break;
            //    }
            //}
            //if (e.CommandName == "Reset")
            //{
            //    foreach (GridColumn column in rgBreastBiopsiesSurgeries.MasterTableView.OwnerGrid.Columns)
            //    {
            //        column.CurrentFilterFunction = GridKnownFunction.NoFilter;
            //        column.CurrentFilterValue = string.Empty;
            //    }
            //    rgBreastBiopsiesSurgeries.MasterTableView.FilterExpression = "";
            //    rgBreastBiopsiesSurgeries.Rebind();
            //}
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }
    }

    protected void rgBreastBiopsiesSurgeries_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }

        if (e.Item is GridHeaderItem && !isExport)
        {

            GridHeaderItem header = (GridHeaderItem)e.Item;

            //loops through all headercells, which is different than columns
            foreach (GridTableHeaderCell headerCell in header.Controls)
            {
                ImageButton img = new ImageButton();

                //if the headercell has controls, it is a column, not a columngroup
                if (headerCell.HasControls())
                {
                    //get the actual header text
                    string headerText = ((LiteralControl)headerCell.Controls[0]).Text;

                    //make sure it isn't the edit header
                    if (headerText != "&nbsp;")
                    {
                        try
                        {
                            img = setupImgButton(headerText, headerCell.HeaderID);

                            headerCell.Controls.Add(img);
                        }
                        catch (Exception ex)
                        {
                            Logging.WriteLogToFile(_userId, ex);
                        }

                        //loop through actual radgrid columns and compare to the header cells 
                        foreach (GridColumn col in rgBreastBiopsiesSurgeries.Columns)
                        {
                            //test to see if there is currently a sort expression and the header cell matches the column
                            if (rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions.Count != 0
                                && (rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions[0].FieldName == col.UniqueName ||
                                    rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions[0].FieldName == col.SortExpression)
                                && headerText == col.HeaderText)
                            {
                                //check to see if the sort order isn't none
                                if (rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions[0].SortOrder != GridSortOrder.None)
                                {
                                    //if it isn't none, remove the no sort image
                                    try
                                    {
                                        headerCell.Controls.Remove(img);
                                    }
                                    catch (Exception ex)
                                    {
                                        Logging.WriteLogToFile(_userId, ex);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    protected void rgBreastBiopsiesSurgeries_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                string[] fieldNames = { "Report", "NofificationStatement", "FollowUpActionStatement", "TumorRegData", "Pathology" }; //"Biopsy_Result", "Pathology_Report", "brca_status","TumorRegistryData"};
                string reportName = "BB_";

                if (isExport)
                {
                    string columnName;
                    TableCell ctc;

                    // Inspect link fields and format accordingly for the output report

                    foreach (string columnRoot in fieldNames)
                    {
                        item = (GridDataItem)e.Item;
                        columnName = reportName + columnRoot + "_Report";
                        ctc = item[columnName];
                        Helpers.configureLinkForExport(ctc, columnRoot, reportName, "");
                    }

                    // Format cells
                    foreach (TableCell cell in e.Item.Cells)
                    {
                        cell.Style["mso-number-format"] = @"\@";
                    }
                }
                else // Not exporting, just displaying grid
                {
                    Label lbl;
                    string sData;
                    RadToolTip rtt;

                    // Inspect each link field and create links or appropriate message

                    foreach (string columnRoot in fieldNames)
                    {

                        lbl = (Label)item.FindControl("lbl" + reportName + columnRoot);
                        sData = ((System.Data.DataRowView)e.Item.DataItem)[columnRoot].ToString();
                        rtt = (RadToolTip)item.FindControl("rtt" + reportName + columnRoot);

                        Helpers.setupLink(lbl, sData, rtt, columnRoot, reportName);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    protected void rgBreastBiopsiesSurgeries_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgBreastBiopsiesSurgeries.CurrentPageIndex = 0;
        executeSortExpression(e.SortExpression, e.NewSortOrder);
    }

    protected void rgBreastBiopsiesSurgeries_PageIndexChanged(object sender, GridPageChangedEventArgs e)
    {
        foreach (GridSortExpression sortE in rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions)
        {
            executeSortExpression(sortE.FieldName, sortE.SortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    #endregion

    #region Facility Dropdown Events

    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        clearNonFacilityFilters();

        ProcessVisnSelection();
    }

    protected void ProcessVisnSelection()
    {
        r.GetFacilitiesByUserId();

        // Station drop down clear existing items
        rcbFac.Items.Clear();
        rcbFac.Text = "";
        rcbFac.ClearSelection();

        if (rcbVisn.SelectedIndex > -1)
        {
            rcbFac.Enabled = true;

            DataColumn idColumn = new DataColumn();
            idColumn = new DataColumn();
            idColumn.DataType = System.Type.GetType("System.Int32");
            idColumn.ColumnName = "sta3n";
            idColumn.ReadOnly = true;
            idColumn.Unique = true;

            DataColumn nameColumn = new DataColumn();
            nameColumn = new DataColumn();
            nameColumn.DataType = System.Type.GetType("System.String");
            nameColumn.ColumnName = "facText";
            nameColumn.ReadOnly = true;
            nameColumn.Unique = true;

            DataTable facs = new DataTable();
            facs.Columns.Add(idColumn);
            facs.Columns.Add(nameColumn);

            DataRow dr;

            foreach (DataRow drFac in r.dtFac.Rows)
            {
                if (drFac["visnId"].ToString() == rcbVisn.SelectedValue)
                {
                    dr = facs.NewRow();
                    dr["sta3n"] = drFac["sta3n"];
                    dr["facText"] = drFac["facText"];

                    facs.Rows.Add(dr);
                }
            }

            rcbFac.DataSource = facs;
            rcbFac.DataTextField = "facText";
            rcbFac.DataValueField = "sta3n";
            rcbFac.DataBind();

            // If there's only one option, select it for them
            if (rcbFac.Items.Count == 1)
            {
                rcbFac.SelectedIndex = 0;
                //rcbFac.Enabled = false;
                btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;

                rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(rcbFac.SelectedValue));
                rcbPACTTeamsForFacility.DataBind();
                rcbPACTTeamsForFacility.Enabled = true;
            }

            rcbFac.Focus();
        }
        else
        {
            rcbFac.Enabled = false;
        }
    }

    protected void rcbFac_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbFac.SelectedIndex > -1)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }

        // Check to see if facility was selected.
        if (rcbFac.SelectedValue != null && !String.IsNullOrEmpty(rcbFac.SelectedValue))
        {
            // query for available PACT Teams for facility
            //rcbPACTTeamsForFacility.DataSource = 
            rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(rcbFac.SelectedValue));
            rcbPACTTeamsForFacility.DataBind();
            rcbPACTTeamsForFacility.Enabled = true;

            foreach (RadComboBoxItem itm in rcbPACTTeamsForFacility.Items)
            {
                itm.Checked = true;
            }

            rcbPACTTeamsForFacility.Focus();
        }
        else
        {
            // remove items
            rcbPACTTeamsForFacility.ClearSelection();
            rcbPACTTeamsForFacility.DataSource = "";
            rcbPACTTeamsForFacility.DataBind();

            rcbPACTTeamsForFacility.Enabled = true;
        }
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            rgBreastBiopsiesSurgeries.CurrentPageIndex = 0;
            rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions.Clear();

            DateTime start = DateTime.Now;

            FillReportGrid();
            rgBreastBiopsiesSurgeries.DataBind();

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        rgBreastBiopsiesSurgeries.MasterTableView.SortExpressions.Clear();

        // clear search fields -- set date
        rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
        rdpEndDate.SelectedDate = DateTime.Today;
        rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

        rcbFac.ClearSelection();
        rcbVisn.ClearSelection();
        rcbFac.Enabled = false;

        clearNonFacilityFilters();

        FillFACs();
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        rgBreastBiopsiesSurgeries.DataSource = "";
        rgBreastBiopsiesSurgeries.DataBind();

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgBreastBiopsiesSurgeries.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgBreastBiopsiesSurgeries.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgBreastBiopsiesSurgeries.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgBreastBiopsiesSurgeries.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgBreastBiopsiesSurgeries.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgBreastBiopsiesSurgeries.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgBreastBiopsiesSurgeries.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgBreastBiopsiesSurgeries.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {

        rgBreastBiopsiesSurgeries.MasterTableView.AllowPaging = false;
        rgBreastBiopsiesSurgeries.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgBreastBiopsiesSurgeries.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgBreastBiopsiesSurgeries.MasterTableView.Items.Count.ToString();
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.ForceTextWrap = true;

        rgBreastBiopsiesSurgeries.MasterTableView.Caption = reportHeaderString.ToString();
        rgBreastBiopsiesSurgeries.Rebind();
        rgBreastBiopsiesSurgeries.ExportSettings.FileName = hfCustomReportName.Value;
        rgBreastBiopsiesSurgeries.ExportSettings.IgnorePaging = true;
        rgBreastBiopsiesSurgeries.ExportSettings.OpenInNewWindow = true;
        rgBreastBiopsiesSurgeries.ExportSettings.HideStructureColumns = true;

        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgBreastBiopsiesSurgeries.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgBreastBiopsiesSurgeries.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgBreastBiopsiesSurgeries.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgBreastBiopsiesSurgeries.Rebind();
    }

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score, object mammogramSource)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1")
            {
                returnVal = "No Score - Source: " + (mammogramSource != null ? mammogramSource.ToString() : "");
            }
            else if (String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    #endregion
}