﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_BreastCancer_Agg_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;

    protected string reportName = "Breast Cancer Aggregate Report";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > Breast Cancer Aggregate";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            if (!IsPostBack)
            {
                rgBreastCancerAgg.DataSource = "";
                rgBreastCancerAgg.DataBind();

                //Fill the facilities listbox
                FillFACs();

                // Fill the Race list
                FillRaces();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Breast Cancer Aggregate Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgBreastCancerAgg_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgBreastCancerAgg_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        if (e.CommandName == "Details")
        {
            if (e.Item is GridDataItem)
            {
                int rowIndex = Convert.ToInt32(e.CommandArgument);
                string selectedSta3n = e.Item.OwnerTableView.DataKeyValues[rowIndex]["Sta3n"].ToString();

                string ageRestrictString = "";
                ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

                string searchGender = null;
                if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
                {
                    searchGender = null;
                }
                else
                {
                    searchGender = rcbGender.SelectedValue;
                }

                string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

                //item = ((GridDataItem)e.Item);
                //string selectedSta3n = item.GetDataKeyValue("Sta3n").ToString();

                //Response.Redirect("BreastCancer_Agg_ByFacId_Rpt.aspx?sta3n=" + selectedSta3n);
                Response.Redirect(string.Format("BreastCancer_Agg_ByFacId_Rpt.aspx?sta3n={0}&ages={1}&gender={2}&race={3}", selectedSta3n, ageRestrictString, searchGender, raceRestrictString));
            }
        }
    }

    protected void rgBreastCancerAgg_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgBreastCancerAgg_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridFilteringItem)
            {

            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                drillButton.Visible = UserPermittedToViewFacility(item.GetDataKeyValue("Sta3n").ToString());    
            }

            if (isExport && e.Item is GridDataItem)
            {
                foreach (TableCell cell in e.Item.Cells)
                {
                    //Approach 1
                    cell.Style["mso-number-format"] = @"\@";
                }
            }

            // footer
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgBreastCancerAgg_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        //rgBreastCancerAgg.CurrentPageIndex = 0;
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        Aggregate agg = new Aggregate();

        try
        {
            #region Determine VISNs

            var visnList = rcbVisn.CheckedItems;
            string tempVISNString = "";

            if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
            {
                tempVISNString = null; // all
            }
            else
            {
                foreach (var entry in visnList)
                {
                    tempVISNString += entry.Value + ",";
                }

                if (tempVISNString.EndsWith(","))
                {
                    tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
                }

                if (String.IsNullOrEmpty(tempVISNString))
                    tempVISNString = null;
            }

            #endregion

            string ageRestrictString = "";
            ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

            string searchGender = null;
            if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
            {
                searchGender = null;
            }
            else
            {
                searchGender = rcbGender.SelectedValue;
            }

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            DataTable dt = agg.GetBreastCancerByVISNAgg(tempVISNString, ageRestrictString, searchGender, raceRestrictString);

            //bind the data to the grid
            rgBreastCancerAgg.DataSource = dt;

            rhcBreastCancerDist.DataSource = dt;
            rhcBreastCancerDist.DataBind();


            // Get Breakdown by Age
            DataTable ageDT = agg.GetBreastCancerByVISNAge(tempVISNString);
            if (ageDT != null && ageDT.Rows.Count > 0)
            {
                // get first row
                List<BreastCancerAggDisplay_ChartItem> ageBreakdownList = new List<BreastCancerAggDisplay_ChartItem>();
                foreach (DataColumn dc in ageDT.Columns)
                {
                    string temp = ageDT.Rows[0][dc].ToString();
                    ageBreakdownList.Add(new BreastCancerAggDisplay_ChartItem() { DisplayName = dc.ColumnName, PercentValue = Convert.ToDecimal(ageDT.Rows[0][dc]) });
                }

                rhcBreastCancerByAge.DataSource = ageBreakdownList;
                rhcBreastCancerByAge.DataBind();
            }

            // Get Breakdown by Race
            DataTable raceDT = agg.GetBreastCancerByVISNRace(tempVISNString);
            if (raceDT != null && raceDT.Rows.Count > 0)
            {
                // get first row
                List<BreastCancerAggDisplay_ChartItem> raceBreakdownList = new List<BreastCancerAggDisplay_ChartItem>();
                foreach (DataColumn dc in raceDT.Columns)
                {
                    string temp = raceDT.Rows[0][dc].ToString();
                    raceBreakdownList.Add(new BreastCancerAggDisplay_ChartItem() { DisplayName = dc.ColumnName, PercentValue = Convert.ToDecimal(raceDT.Rows[0][dc]) });
                }

                rhcBreastCancerByRace.DataSource = raceBreakdownList;
                rhcBreastCancerByRace.DataBind();
            }

            //rhcComplianceByRace

            //rhcBreastCxDistribution.DataSource = dt;
            //rhcBreastCxDistribution.DataBind();

            //RadHtmlChart1.DataSource = dt;
            //RadHtmlChart1.DataBind();
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            //if error
            rgBreastCancerAgg.DataSource = "";
        }
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            // populate all VISNs
            DataTable dtVisns = r.GetVISNs();

            if (dtVisns != null)
            {
                rcbVisn.DataSource = dtVisns;
                rcbVisn.DataTextField = "visnText";
                rcbVisn.DataValueField = "visnId";
                rcbVisn.DataBind();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISN - rcbVisn
        if (rcbVisn.SelectedValue != null)
        {
            sb.Append(" VISN: " + rcbVisn.SelectedItem.Text + " |");
        }

        // Age Restriction - rddlAgeRestriction
        sb.Append(" Ages: " + ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? "All |" : rddlAgeRestriction.SelectedValue + " |"));

        // Gender - rcbGender
        if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
        {
            sb.Append(" Gender: All |");
        }
        else
        {
            sb.Append(" Gender: " + rcbGender.SelectedValue + " |");
        }

        // Race - rcbRace
        sb.Append(" Race: " + ((rcbRace.SelectedValue == "All") ? "All |" : rcbRace.SelectedValue + " |"));

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }
    
    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbVisn.CheckedItems.Count > 0)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        DateTime start = DateTime.Now;

        pnlCharts.Visible = true;

        FillReportGrid();
        rgBreastCancerAgg.DataBind();

        DateTime end = DateTime.Now;
        lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rgBreastCancerAgg.DataSource = "";
        rgBreastCancerAgg.DataBind();

        rcbRace.ClearSelection();
        rddlAgeRestriction.ClearSelection();
        rcbGender.SelectedIndex = 0;

        rcbVisn.ClearSelection();
        rcbVisn.ClearCheckedItems();
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        rhcBreastCancerDist.DataSource = "";
        rhcBreastCancerDist.DataBind();

        //rhcBreastCxDistribution.DataSource = "";
        //rhcBreastCxDistribution.DataBind();

        pnlCharts.Visible = false;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Return to Dashboard
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnBackToDashboard_Click(object sender, EventArgs e)
    {
        Response.Redirect("Dashboard.aspx");
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgBreastCancerAgg.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgBreastCancerAgg.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgBreastCancerAgg.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgBreastCancerAgg.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgBreastCancerAgg.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgBreastCancerAgg.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgBreastCancerAgg.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgBreastCancerAgg.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {

        rgBreastCancerAgg.MasterTableView.AllowPaging = false;
        rgBreastCancerAgg.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgBreastCancerAgg.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgBreastCancerAgg.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgBreastCancerAgg.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgBreastCancerAgg.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgBreastCancerAgg.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgBreastCancerAgg.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgBreastCancerAgg.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgBreastCancerAgg.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgBreastCancerAgg.MasterTableView.Items.Count.ToString();
        rgBreastCancerAgg.ExportSettings.Pdf.ForceTextWrap = true;

        rgBreastCancerAgg.MasterTableView.Caption = reportHeaderString.ToString();
        rgBreastCancerAgg.Rebind();
        rgBreastCancerAgg.ExportSettings.FileName = hfCustomReportName.Value;
        rgBreastCancerAgg.ExportSettings.IgnorePaging = true;
        rgBreastCancerAgg.ExportSettings.OpenInNewWindow = true;
        rgBreastCancerAgg.ExportSettings.HideStructureColumns = true;

        rgBreastCancerAgg.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgBreastCancerAgg.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgBreastCancerAgg.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgBreastCancerAgg.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgBreastCancerAgg.Rebind();
    }

    #endregion
}

public class BreastCancerAggDisplay_ChartItem
{
    public string DisplayName { get; set; }
    public decimal PercentValue { get; set; }

    public BreastCancerAggDisplay_ChartItem() { }
}