﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammoBiRadDistributionByFacility : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;
    protected string sta3n;
    protected string startDate;
    protected string endDate;
    protected string ages;
    protected string gender;
    protected string race;

    protected string reportName = "All Mammograms with Bi-RADS Distribution By Facility";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > <a href=\"MammoBiRadDistribution_Agg_Rpt.aspx\" title=\"Mammograms with Bi-RADS Distribution\" style=\"color:#FFFFFF;\">All Mammograms with Bi-RADS Distribution</a> > All Mammograms with Bi-RADS Distribution By Facility";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            sta3n = Request.QueryString.Get("sta3n");
            startDate = Request.QueryString.Get("startDate");
            endDate = Request.QueryString.Get("endDate");
            ages = Request.QueryString.Get("ages");
            race = Request.QueryString.Get("race");
            gender = Request.QueryString.Get("gender");

            if (!String.IsNullOrEmpty(sta3n))
            {
                if (!UserPermittedToViewFacility(sta3n))
                {
                    // no access.
                    Response.Redirect("ErrorPage.aspx?id=FAC");
                }
            }
            else
            {
                Response.Redirect("ErrorPage.aspx?id=FAC");
            }

            if (!IsPostBack)
            {
                //set date
                rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
                rdpEndDate.SelectedDate = DateTime.Today;
                rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

                if (!String.IsNullOrEmpty(startDate))
                {
                    try
                    {
                        rdpStartDate.SelectedDate = DateTime.Parse(startDate);
                    }
                    catch (Exception ex)
                    {
                        Logging.WriteLogToFile(_userId, ex);
                    }
                }

                if (!String.IsNullOrEmpty(endDate))
                {
                    try
                    {
                        rdpEndDate.SelectedDate = DateTime.Parse(endDate);
                    }
                    catch (Exception ex)
                    {
                        Logging.WriteLogToFile(_userId, ex);
                    }
                }

                if (!String.IsNullOrEmpty(sta3n))
                {
                    rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(sta3n));
                    rcbPACTTeamsForFacility.DataBind();
                    rcbPACTTeamsForFacility.Enabled = true;

                    foreach (RadComboBoxItem itm in rcbPACTTeamsForFacility.Items)
                    {
                        itm.Checked = true;
                    }

                    rcbPACTTeamsForFacility.Focus();
                }
                else
                {
                    // remove items
                    rcbPACTTeamsForFacility.ClearSelection();
                    rcbPACTTeamsForFacility.DataSource = "";
                    rcbPACTTeamsForFacility.DataBind();

                    rcbPACTTeamsForFacility.Enabled = false;
                }


                rgAllMammos.DataSource = "";
                rgAllMammos.DataBind();

                // Fill the Race list
                FillRaces();

                // set initial values
                if (!String.IsNullOrEmpty(race))
                    rcbRace.SelectedValue = race;

                if (!String.IsNullOrEmpty(ages))
                    rddlAgeRestriction.SelectedValue = ages;

                if (!String.IsNullOrEmpty(gender))
                    rcbGender.SelectedValue = gender;

                // default search
                DateTime start = DateTime.Now;
                FillReportGrid();
                rgAllMammos.DataBind();

                DateTime end = DateTime.Now;

                lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region Protected Helper Methods

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1" || String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, throws error.
    /// </summary>
    /// <param name="facId"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    string permittedSta3n = dr["sta3n"].ToString();

                    if (sta3n == permittedSta3n)
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        PatientData pd = new PatientData();

        try
        {
            if (!String.IsNullOrEmpty(sta3n))
            {
                rgAllMammos.DataSource = getDataset();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            //if error
            rgAllMammos.DataSource = "";
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        PatientData pd = new PatientData();

        try
        {
            if (!String.IsNullOrEmpty(sta3n))
            {

                // get sta3n from institution id
                DateTime tempStartDate = (DateTime)rdpStartDate.SelectedDate;
                DateTime tempEndDate = (DateTime)rdpEndDate.SelectedDate;

                //get the report data
                int showLatest = 1;
                showLatest = chkShowLatest.Checked ? 1 : 0;

                //List<RadComboBoxItem> test = rcbBiRadsList.CheckedItems.;
                var biRadList = rcbBiRadsList.CheckedItems;
                string tempBiradsString = "";

                if (rcbBiRadsList.CheckedItems.Count == rcbBiRadsList.Items.Count)
                {
                    tempBiradsString = null; // all
                }
                else
                {
                    foreach (var entry in biRadList)
                    {
                        tempBiradsString += entry.Value + ",";
                    }

                    if (tempBiradsString.EndsWith(","))
                    {
                        tempBiradsString = tempBiradsString.Substring(0, tempBiradsString.Length - 1);
                    }

                    if (String.IsNullOrEmpty(tempBiradsString))
                        tempBiradsString = null;
                }

                int showOnlyMammos = 0;
                showOnlyMammos = chkShowOnlyMammos.Checked ? 1 : 0;

                string searchPatName = String.IsNullOrEmpty(rtbPatientName.Text) ? null : rtbPatientName.Text;
                string searchProviderName = String.IsNullOrEmpty(rtbProviderName.Text) ? null : rtbProviderName.Text;

                string ageRestrictString = "";
                ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

                string searchGender = null;
                if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
                {
                    searchGender = null;
                }
                else
                {
                    searchGender = rcbGender.SelectedValue;
                }

                string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

                // follow-up action
                string followUpAction = null;
                if (String.IsNullOrEmpty(rcbFollowUpActionStatus.SelectedValue) || rcbFollowUpActionStatus.SelectedValue == "All")
                {
                    followUpAction = null;
                }
                else
                {
                    followUpAction = rcbFollowUpActionStatus.SelectedValue;
                }

                //PACT TEAM STRING CREATION
                var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
                string pactTeamString = "";

                if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
                {
                    pactTeamString = null; // all
                }
                else
                {
                    foreach (var entry in pactTeamList)
                    {
                        pactTeamString += entry.Value + ",";
                    }

                    if (pactTeamString.EndsWith(","))
                    {
                        pactTeamString = pactTeamString.Substring(0, pactTeamString.Length - 1);
                    }

                    if (String.IsNullOrEmpty(pactTeamString))
                        pactTeamString = null;
                }

                //DataTable dt = pd.GetAllMammogramsAbnormalMammo(sta3n, tempStartDate.ToShortDateString(), tempEndDate.ToShortDateString(), showLatest, tempBiradsString, ageRestrictString, searchPatName, searchGender, searchProviderName, raceRestrictString, showOnlyMammos, followUpAction);
                DataTable dt = pd.GetAllMammogramsAbnormalMammo(sta3n, tempStartDate.ToShortDateString(), tempEndDate.ToShortDateString(), showLatest, tempBiradsString, ageRestrictString, searchPatName, searchGender, searchProviderName, raceRestrictString, showOnlyMammos, followUpAction, pactTeamString);
                return dt;
            }
            else
            {
                // error ... handle
                Logging.WriteLogToFile(_userId, "No Sta3n passed");
            }

            return null;
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(string sortExpression, GridSortOrder newSortOrder)
    {
        DataTable dt = getDataset();
        if (dt != null)
        {
            if (sortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (sortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgAllMammos.DataSource = dt.DefaultView.ToTable();
            rgAllMammos.DataBind();
        }
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        sb.Append(" Facility: " + sta3n + " |");

        #region PACT Team

        // PACT Team List
        var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
        string tempPACTString = "";

        if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
        {
            tempPACTString = null; // all
        }
        else
        {
            foreach (var entry in pactTeamList)
            {
                tempPACTString += entry.Text + ",";
            }

            if (tempPACTString.EndsWith(","))
            {
                tempPACTString = tempPACTString.Substring(0, tempPACTString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempPACTString))
                tempPACTString = null;
        }

        if (String.IsNullOrEmpty(tempPACTString))
        {
            sb.Append(" PACT TEAM: All |");
        }
        else
        {
            sb.Append(" PACT TEAM: " + tempPACTString + " |");
        }

        #endregion

        // Start Date - rdpStartDate
        if (rdpStartDate.SelectedDate != null)
        {
            sb.Append(" Start Date: " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + " |");
        }

        // End Date - rdpEndDate
        if (rdpEndDate.SelectedDate != null)
        {
            sb.Append(" End Date: " + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        }

        // Showing only latest - chkShowLatest
        sb.Append(" Showing Only Latest: " + chkShowLatest.Checked.ToString() + " |");

        // Showing only Mammograms - chkShowOnlyMammos
        sb.Append(" Showing Only MMG: " + chkShowOnlyMammos.Checked.ToString() + " |");

        // BIRAD List - rcbBiRadsList
        var biRadList = rcbBiRadsList.CheckedItems;
        string tempBiradsString = "";

        if (rcbBiRadsList.CheckedItems.Count == rcbBiRadsList.Items.Count)
        {
            tempBiradsString = null; // all
        }
        else
        {
            foreach (var entry in biRadList)
            {
                tempBiradsString += entry.Value + ",";
            }

            if (tempBiradsString.EndsWith(","))
            {
                tempBiradsString = tempBiradsString.Substring(0, tempBiradsString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempBiradsString))
                tempBiradsString = null;
        }

        if (String.IsNullOrEmpty(tempBiradsString))
        {
            sb.Append(" BIRADS: All |");
        }
        else
        {
            sb.Append(" BIRADS: " + tempBiradsString + " |");
        }

        // Age Restriction - rddlAgeRestriction
        sb.Append(" Ages: " + ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? "All |" : rddlAgeRestriction.SelectedValue + " |"));

        // Patient Name / SSN - rtbPatientName
        if (!String.IsNullOrEmpty(rtbPatientName.Text))
        {
            sb.Append(" Patient Name/SSN: " + rtbPatientName.Text + " |");
        }

        // Gender - rlbGender
        if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
        {
            sb.Append(" Gender: All |");
        }
        else
        {
            sb.Append(" Gender: " + rcbGender.SelectedValue + " |");
        }

        // Race - rcbRace
        sb.Append(" Race: " + ((rcbRace.SelectedValue == "All") ? "All |" : rcbRace.SelectedValue + " |"));

        // Provider - rtbProviderName
        if (!String.IsNullOrEmpty(rtbProviderName.Text))
        {
            sb.Append(" Provider: " + rtbProviderName.Text + " |");
        }

        // Followup
        sb.Append(" Follow-Up Action: " + ((rcbFollowUpActionStatus.SelectedValue == "All") ? "All |" : rcbFollowUpActionStatus.SelectedValue + " |"));

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            DateTime start = DateTime.Now;
            FillReportGrid();
            rgAllMammos.DataBind();

            DateTime end = DateTime.Now;

            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields

        //set date
        rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
        rdpEndDate.SelectedDate = DateTime.Today;
        rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

        if (!String.IsNullOrEmpty(startDate))
        {
            try
            {
                rdpStartDate.SelectedDate = DateTime.Parse(startDate);
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }

        if (!String.IsNullOrEmpty(endDate))
        {
            try
            {
                rdpEndDate.SelectedDate = DateTime.Parse(endDate);
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }

        chkShowLatest.Checked = false;
        chkShowOnlyMammos.Checked = false;

        foreach (var temp in rcbBiRadsList.Items)
        {
            ((RadComboBoxItem)temp).Checked = true;
        }

        rtbPatientName.Text = "";

        if (!String.IsNullOrEmpty(gender))
        {
            try
            {
                rcbGender.SelectedValue = gender;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rcbGender.SelectedIndex = 0;
        }

        rtbProviderName.Text = "";

        if (!String.IsNullOrEmpty(ages))
        {
            try
            {
                rddlAgeRestriction.SelectedValue = ages;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rddlAgeRestriction.SelectedIndex = 0;
        }

        if (!String.IsNullOrEmpty(race))
        {
            try
            {
                rcbRace.SelectedValue = race;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rcbRace.SelectedIndex = 0;
        }

        rcbFollowUpActionStatus.SelectedIndex = 0;

        if (!String.IsNullOrEmpty(sta3n))
        {
            rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(sta3n));
            rcbPACTTeamsForFacility.DataBind();
            rcbPACTTeamsForFacility.Enabled = true;
        }

        rgAllMammos.DataSource = "";
        rgAllMammos.DataBind();

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgAllMammos.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgAllMammos.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgAllMammos.MasterTableView.AllowPaging = false;
        rgAllMammos.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 11pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgAllMammos.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgAllMammos.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgAllMammos.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgAllMammos.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgAllMammos.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgAllMammos.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgAllMammos.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgAllMammos.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgAllMammos.MasterTableView.Items.Count.ToString();
        rgAllMammos.ExportSettings.Pdf.ForceTextWrap = true;

        rgAllMammos.MasterTableView.Caption = reportHeaderString.ToString();
        rgAllMammos.Rebind();
        rgAllMammos.ExportSettings.FileName = hfCustomReportName.Value;
        rgAllMammos.ExportSettings.IgnorePaging = true;
        rgAllMammos.ExportSettings.OpenInNewWindow = true;
        rgAllMammos.ExportSettings.HideStructureColumns = true;

        rgAllMammos.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgAllMammos.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgAllMammos.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgAllMammos.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgAllMammos.Rebind();
    }

    #endregion

    #region Grid Methods

    protected void rgAllMammos_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        //InitReportGrid();
        FillReportGrid();
    }

    protected void rgAllMammos_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (e.Item is GridHeaderItem && !isExport)
        {

            GridHeaderItem header = (GridHeaderItem)e.Item;

            //loops through all headercells, which is different than columns
            foreach (GridTableHeaderCell headerCell in header.Controls)
            {
                ImageButton img = new ImageButton();

                //if the headercell has controls, it is a column, not a columngroup
                if (headerCell.HasControls())
                {
                    //get the actual header text
                    string headerText = ((LiteralControl)headerCell.Controls[0]).Text;

                    //make sure it isn't the edit header
                    if (headerText != "&nbsp;")
                    {
                        try
                        {
                            img = setupImgButton(headerText, headerCell.HeaderID);

                            headerCell.Controls.Add(img);
                        }
                        catch (Exception ex)
                        {
                            Logging.WriteLogToFile(_userId, ex);
                        }

                        //loop through actual radgrid columns and compare to the header cells 
                        foreach (GridColumn col in rgAllMammos.Columns)
                        {
                            //test to see if there is currently a sort expression and the header cell matches the column
                            if (rgAllMammos.MasterTableView.SortExpressions.Count != 0
                                && (rgAllMammos.MasterTableView.SortExpressions[0].FieldName == col.UniqueName ||
                                    rgAllMammos.MasterTableView.SortExpressions[0].FieldName == col.SortExpression)
                                && headerText == col.HeaderText)
                            {
                                //check to see if the sort order isn't none
                                if (rgAllMammos.MasterTableView.SortExpressions[0].SortOrder != GridSortOrder.None)
                                {
                                    //if it isn't none, remove the no sort image
                                    try
                                    {
                                        headerCell.Controls.Remove(img);
                                    }
                                    catch (Exception ex)
                                    {
                                        Logging.WriteLogToFile(_userId, ex);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    protected void rgAllMammos_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                string[] fieldNames = { "NofificationStatement", "FollowUpActionStatement" };
                string reportName = "BB_";

                if (isExport)
                {
                    string columnName;
                    TableCell ctc;

                    // Inspect link fields and format accordingly for the output report

                    foreach (string columnRoot in fieldNames)
                    {
                        item = (GridDataItem)e.Item;
                        columnName = reportName + columnRoot + "_Report";
                        ctc = item[columnName];
                        Helpers.configureLinkForExport(ctc, columnRoot, reportName, "");
                    }

                    // Format cells
                    foreach (TableCell cell in e.Item.Cells)
                    {
                        cell.Style["mso-number-format"] = @"\@";
                    }
                }
                else // Not exporting, just displaying grid
                {
                    Label lbl;
                    string sData;
                    RadToolTip rtt;

                    // Inspect each link field and create links or appropriate message

                    foreach (string columnRoot in fieldNames)
                    {

                        lbl = (Label)item.FindControl("lbl" + reportName + columnRoot);
                        sData = ((System.Data.DataRowView)e.Item.DataItem)[columnRoot].ToString();
                        rtt = (RadToolTip)item.FindControl("rtt" + reportName + columnRoot);

                        Helpers.setupLink(lbl, sData, rtt, columnRoot, reportName);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// For custom sorting of Team (nulls at bottom)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgAllMammos_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgAllMammos.CurrentPageIndex = 0;
        executeSortExpression(e.SortExpression, e.NewSortOrder);
    }

    /// <summary>
    /// Paging method to keep custom sorting intact.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgAllMammos_PageIndexChanged(object sender, GridPageChangedEventArgs e)
    {
        foreach (GridSortExpression sortE in rgAllMammos.MasterTableView.SortExpressions)
        {
            executeSortExpression(sortE.FieldName, sortE.SortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    #endregion

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR All Mammograms with Bi-RADS Distribution By Facility Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_AGGREGATE_REPORTS";
    }
    #endregion
}