﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammoBiRadDistribution_Agg_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoPDFExport"] = value;
        }
    }

    protected string reportName = "Mammogram BIRAD Score Distribution";

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > All Mammograms with Bi-RADS Distribution";
        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            if (!IsPostBack)
            {
                //set date
                rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
                rdpEndDate.SelectedDate = DateTime.Today;
                rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

                // Fill the Race list
                FillRaces();

                rgAllMammos.DataSource = "";
                rgAllMammos.DataBind();

                //Fill the facilities listbox
                FillFACs();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR All Mammograms with Bi-RADS Distribution Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_AGGREGATE_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgAllMammos_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        //InitReportGrid();
        FillReportGrid();
    }

    protected void rgAllMammos_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            GridDataItem item;

            item = ((GridDataItem)e.Item);

            int rowIndex = Convert.ToInt32(e.CommandArgument);

            string sta3n = e.Item.OwnerTableView.DataKeyValues[rowIndex]["STATIONNUMBER"].ToString();

            string ageRestrictString = "";
            ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

            string searchGender = null;
            if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
            {
                searchGender = null;
            }
            else
            {
                searchGender = rcbGender.SelectedValue;
            }

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            //string sta3n = (e.Item as GridDataItem).GetDataKeyValue("STATIONNUMBER").ToString();
            Response.Redirect(string.Format("MammoBiRadDistributionByFacility.aspx?sta3n={0}&startDate={1}&endDate={2}&ages={3}&gender={4}&race={5}", sta3n, ((DateTime)rdpStartDate.SelectedDate).ToShortDateString(), ((DateTime)rdpEndDate.SelectedDate).ToShortDateString(), ageRestrictString, searchGender, raceRestrictString));
        }
    }

    protected void rgAllMammos_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (e.Item is GridHeaderItem && !isExport)
        {

            GridHeaderItem header = (GridHeaderItem)e.Item;

            //loops through all headercells, which is different than columns
            foreach (GridTableHeaderCell headerCell in header.Controls)
            {
                ImageButton img = new ImageButton();

                //if the headercell has controls, it is a column, not a columngroup
                if (headerCell.HasControls())
                {
                    //get the actual header text
                    string headerText = ((LiteralControl)headerCell.Controls[0]).Text;

                    //make sure it isn't the edit header
                    if (headerText != "&nbsp;")
                    {
                        try
                        {
                            img = setupImgButton(headerText, headerCell.HeaderID);

                            headerCell.Controls.Add(img);
                        }
                        catch (Exception ex)
                        {
                            Logging.WriteLogToFile(_userId, ex);
                        }

                        //loop through actual radgrid columns and compare to the header cells 
                        foreach (GridColumn col in rgAllMammos.Columns)
                        {
                            //test to see if there is currently a sort expression and the header cell matches the column
                            if (rgAllMammos.MasterTableView.SortExpressions.Count != 0
                                && (rgAllMammos.MasterTableView.SortExpressions[0].FieldName == col.UniqueName ||
                                    rgAllMammos.MasterTableView.SortExpressions[0].FieldName == col.SortExpression)
                                && headerText == col.HeaderText)
                            {
                                //check to see if the sort order isn't none
                                if (rgAllMammos.MasterTableView.SortExpressions[0].SortOrder != GridSortOrder.None)
                                {
                                    //if it isn't none, remove the no sort image
                                    try
                                    {
                                        headerCell.Controls.Remove(img);
                                    }
                                    catch (Exception ex)
                                    {
                                        Logging.WriteLogToFile(_userId, ex);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    protected void rgAllMammos_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridFilteringItem)
            {

            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                drillButton.Visible = UserPermittedToViewFacility(item.GetDataKeyValue("STATIONNUMBER").ToString());
            }

            if (isExport && e.Item is GridDataItem)
            {
                foreach (TableCell cell in e.Item.Cells)
                {
                    //Approach 1
                    cell.Style["mso-number-format"] = @"\@";
                }
            }

            // footer
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgAllMammos_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgAllMammos.CurrentPageIndex = 0;
    }

    /// <summary>
    /// Helper Method to return VISN Name from ID
    /// </summary>
    /// <param name="visnId"></param>
    /// <returns></returns>
    protected string GetVISNNameFromID(object visnId)
    {
        if (visnId != null)
        {
            // rather than calling database - find VISN name in VISN list.
            RadComboBoxItem item = rcbVisn.FindItemByValue(visnId.ToString());
            if (item != null)
            {
                return item.Text;
            }
        }

        return "";
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string facId)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (facId == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        DataTable dt = getDataset();
        rgAllMammos.DataSource = dt;
        //RadHtmlChart1.DataSource = dt;
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        Aggregate agg = new Aggregate();

        try
        {
            #region Determine VISNs

            var visnList = rcbVisn.CheckedItems;
            string tempVISNString = "";

            if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
            {
                tempVISNString = null; // all
            }
            else
            {
                foreach (var entry in visnList)
                {
                    tempVISNString += entry.Value + ",";
                }

                if (tempVISNString.EndsWith(","))
                {
                    tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
                }

                if (String.IsNullOrEmpty(tempVISNString))
                    tempVISNString = null;
            }

            #endregion

            string ageRestrictString = "";
            ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

            string searchGender = null;
            if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
            {
                searchGender = null;
            }
            else
            {
                searchGender = rcbGender.SelectedValue;
            }

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            DataTable dt = agg.GetMammogramsWithBiRadDistrAgg(tempVISNString, rdpStartDate.SelectedDate.ToString(), rdpEndDate.SelectedDate.ToString(), ageRestrictString, searchGender, raceRestrictString);

            if (dt != null)
            {
                return dt;
            }

            return null;
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            return null;
        }
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            rcbVisn.Items.Clear();
            rcbVisn.EmptyMessage = "Select a VISN";

            DataTable visns = r.GetVISNs();

            rcbVisn.DataSource = visns;
            rcbVisn.DataTextField = "visnText";
            rcbVisn.DataValueField = "visnId";
            rcbVisn.DataBind();

        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        // telerik hack - ensure event is being fired by the actual dropdown list
        if (this.Request.Params.Get("__EVENTTARGET") == "rcbVisn")
        {
            if (rcbVisn.CheckedItems.Count > 0)
            {
                rgAllMammos.DataSource = "";
                rgAllMammos.DataBind();

                //RadHtmlChart1.DataSource = "";
                //RadHtmlChart1.DataBind();

                rhcBiRadDistChart.DataSource = "";
                rhcBiRadDistChart.DataBind();

                pnlCharts.Visible = false;
            }
        }

        if (rcbVisn.CheckedItems.Count > 0)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISN - rcbVisn

        #region VISNs

        var visnList = rcbVisn.CheckedItems;
        string tempVISNString = "";

        if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
        {
            tempVISNString = null; // all
        }
        else
        {
            foreach (var entry in visnList)
            {
                tempVISNString += entry.Text + ",";
            }

            if (tempVISNString.EndsWith(","))
            {
                tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempVISNString))
                tempVISNString = null;
        }

        if (String.IsNullOrEmpty(tempVISNString))
        {
            sb.Append(" VISNs: All |");
        }
        else
        {
            sb.Append(" VISNs: " + tempVISNString + " |");
        }

        #endregion

        // Start Date - rdpStartDate
        if (rdpStartDate.SelectedDate != null)
        {
            sb.Append(" Start Date: " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + " |");
        }

        // End Date - rdpEndDate
        if (rdpEndDate.SelectedDate != null)
        {
            sb.Append(" End Date: " + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        }

        // Age Restriction - rddlAgeRestriction
        sb.Append(" Ages: " + ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? "All |" : rddlAgeRestriction.SelectedValue + " |"));

        // Gender - rcbGender
        if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
        {
            sb.Append(" Gender: All |");
        }
        else
        {
            sb.Append(" Gender: " + rcbGender.SelectedValue + " |");
        }

        // Race - rcbRace
        sb.Append(" Race: " + ((rcbRace.SelectedValue == "All") ? "All |" : rcbRace.SelectedValue + " |"));

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            rgAllMammos.MasterTableView.SortExpressions.Clear();

            DateTime start = DateTime.Now;

            FillReportGrid();
            rgAllMammos.DataBind();
            //RadHtmlChart1.DataBind();

            #region Get Aggregate Data for Bar Chart

            int biRad0Count = 0;
            int biRad1Count = 0;
            int biRad2Count = 0;
            int biRad3Count = 0;
            int biRad4Count = 0;
            int biRad5Count = 0;
            int biRad6Count = 0;
            int biRadNoScoreCount = 0;
            int totalCount = 0;

            GridFooterItem footer = (GridFooterItem)rgAllMammos.MasterTableView.GetItems(GridItemType.Footer)[0];

            try
            {
                totalCount = Convert.ToInt32(footer.Cells[6].Text);
                biRad0Count = Convert.ToInt32(footer.Cells[7].Text);
                biRad1Count = Convert.ToInt32(footer.Cells[8].Text);
                biRad2Count = Convert.ToInt32(footer.Cells[9].Text);
                biRad3Count = Convert.ToInt32(footer.Cells[10].Text);
                biRad4Count = Convert.ToInt32(footer.Cells[11].Text);
                biRad5Count = Convert.ToInt32(footer.Cells[12].Text);
                biRad6Count = Convert.ToInt32(footer.Cells[13].Text);
                biRadNoScoreCount = Convert.ToInt32(footer.Cells[14].Text);


                decimal biRad0Percent = Math.Round(((Convert.ToDecimal(biRad0Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad1Percent = Math.Round(((Convert.ToDecimal(biRad1Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad2Percent = Math.Round(((Convert.ToDecimal(biRad2Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad3Percent = Math.Round(((Convert.ToDecimal(biRad3Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad4Percent = Math.Round(((Convert.ToDecimal(biRad4Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad5Percent = Math.Round(((Convert.ToDecimal(biRad5Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRad6Percent = Math.Round(((Convert.ToDecimal(biRad6Count) / Convert.ToDecimal(totalCount)) * 100), 2);
                decimal biRadNoScorePercent = Math.Round(((Convert.ToDecimal(biRadNoScoreCount) / Convert.ToDecimal(totalCount)) * 100), 2);

                List<BiRADDistribution_Display> biRadList = new List<BiRADDistribution_Display>();
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 0", BiRADCount = biRad0Count, PercentageValue = biRad0Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 1", BiRADCount = biRad1Count, PercentageValue = biRad1Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 2", BiRADCount = biRad2Count, PercentageValue = biRad2Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 3", BiRADCount = biRad3Count, PercentageValue = biRad3Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 4", BiRADCount = biRad4Count, PercentageValue = biRad4Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 5", BiRADCount = biRad5Count, PercentageValue = biRad5Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD 6", BiRADCount = biRad6Count, PercentageValue = biRad6Percent, TotalCount = totalCount });
                biRadList.Add(new BiRADDistribution_Display() { Name = "BIRAD No Score", BiRADCount = biRadNoScoreCount, PercentageValue = biRadNoScorePercent, TotalCount = totalCount });

                rhcBiRadDistChart.DataSource = biRadList;
                rhcBiRadDistChart.DataBind();

                pnlCharts.Visible = true;

            }
            catch (Exception ex)
            {
                // fail silently - no data returned to create bar chart and get footer values.
            }

            #endregion

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        //set date
        rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
        rdpEndDate.SelectedDate = DateTime.Today;
        rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

        rgAllMammos.MasterTableView.SortExpressions.Clear();

        rgAllMammos.DataSource = "";
        rgAllMammos.DataBind();

        //RadHtmlChart1.DataSource = "";
        //RadHtmlChart1.DataBind();

        rcbVisn.ClearSelection();
        rcbVisn.ClearCheckedItems();

        rcbGender.SelectedIndex = 0;
        rddlAgeRestriction.ClearSelection();
        rcbRace.ClearSelection();

        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        pnlCharts.Visible = false;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgAllMammos.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgAllMammos.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgAllMammos.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgAllMammos.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgAllMammos.MasterTableView.AllowPaging = false;
        rgAllMammos.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgAllMammos.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgAllMammos.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgAllMammos.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgAllMammos.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgAllMammos.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgAllMammos.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgAllMammos.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgAllMammos.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgAllMammos.MasterTableView.Items.Count.ToString();
        rgAllMammos.ExportSettings.Pdf.ForceTextWrap = true;

        rgAllMammos.MasterTableView.Caption = reportHeaderString.ToString();
        rgAllMammos.Rebind();
        rgAllMammos.ExportSettings.FileName = hfCustomReportName.Value;
        rgAllMammos.ExportSettings.IgnorePaging = true;
        rgAllMammos.ExportSettings.OpenInNewWindow = true;
        rgAllMammos.ExportSettings.HideStructureColumns = true;

        rgAllMammos.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgAllMammos.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgAllMammos.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgAllMammos.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgAllMammos.Rebind();
    }

    #endregion
}

public class BiRADDistribution_Display
{
    public string Name { get; set; }
    public decimal PercentageValue { get; set; }
    public int BiRADCount { get; set; }
    public int TotalCount { get; set; }

    /// <summary>
    /// Default Constructor
    /// </summary>
    public BiRADDistribution_Display() { }
}