﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammogramCompliance_Agg_By_Race : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;
    private Aggregate agg;

    protected string reportName = "Mammograms Compliance Aggregate By Race Report";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoCompliancePDFExport"] != null)
            {
                return (bool)ViewState["IsMammoCompliancePDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoCompliancePDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > <a href=\"MammogramCompliance_Agg_Rpt.aspx\" title=\"Navigate to Mammogram Compliance Report\" style=\"color:#FFFFFF;\">Mammogram Compliance</a> > Mammograms Compliance By Race";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            agg = new Aggregate();

            if (!IsPostBack)
            {
                rgMammoCompliance.DataSource = "";
                rgMammoCompliance.DataBind();

                //Fill the facilities listbox
                FillFACs();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Mammogram Compliance By Race Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgMammoCompliance_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgMammoCompliance_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        if (e.CommandName == "Details")
        {
            if (e.Item is GridDataItem)
            {

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                string selectedSta3n = e.Item.OwnerTableView.DataKeyValues[rowIndex]["Sta3n"].ToString();

                //item = ((GridDataItem)e.Item);
                //string selectedSta3n = item.GetDataKeyValue("Sta3n").ToString();

                Response.Redirect("MammogramCompliance_ByFacility_Agg_Rpt.aspx?sta3n=" + selectedSta3n);
            }
        }
    }

    protected void rgMammoCompliance_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgMammoCompliance_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridFilteringItem)
            {

            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                drillButton.Visible = UserPermittedToViewFacility(item.GetDataKeyValue("Sta3n").ToString());
            }

            if (isExport && e.Item is GridDataItem)
            {
                foreach (TableCell cell in e.Item.Cells)
                {
                    //Approach 1
                    cell.Style["mso-number-format"] = @"\@";
                }
            }

            // footer
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgMammoCompliance_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        //rgMammoCompliance.CurrentPageIndex = 0;
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// Helper Method to return VISN Name from ID
    /// </summary>
    /// <param name="visnId"></param>
    /// <returns></returns>
    protected string GetVISNNameFromID(object visnId)
    {
        if (visnId != null)
        {
            // rather than calling database - find VISN name in VISN list.
            RadComboBoxItem item = rcbVisn.FindItemByValue(visnId.ToString());
            if (item != null)
            {
                return item.Text;
            }
        }

        return "";
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        try
        {
            DataTable dt;

            #region Determine VISNs

            var visnList = rcbVisn.CheckedItems;
            string tempVISNString = "";

            if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
            {
                tempVISNString = null; // all
            }
            else
            {
                foreach (var entry in visnList)
                {
                    tempVISNString += entry.Value + ",";
                }

                if (tempVISNString.EndsWith(","))
                {
                    tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
                }

                if (String.IsNullOrEmpty(tempVISNString))
                    tempVISNString = null;
            }

            #endregion

            // Current
            dt = agg.GetMammogramComplianceByVISNAndRaceAgg(tempVISNString);

            //bind the data to the grid
            rgMammoCompliance.DataSource = dt;
        }
        catch (Exception ex)
        {
            //if error
            rgMammoCompliance.DataSource = "";
        }
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            // populate all VISNs
            DataTable dtVisns = r.GetVISNs();

            if (dtVisns != null)
            {
                rcbVisn.DataSource = dtVisns;
                rcbVisn.DataTextField = "visnText";
                rcbVisn.DataValueField = "visnId";
                rcbVisn.DataBind();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISNs
        var visnList = rcbVisn.CheckedItems;
        string tempVISNString = "";

        if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
        {
            tempVISNString = null; // all
        }
        else
        {
            foreach (var entry in visnList)
            {
                tempVISNString += entry.Text + ",";
            }

            if (tempVISNString.EndsWith(","))
            {
                tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempVISNString))
                tempVISNString = null;
        }

        if (String.IsNullOrEmpty(tempVISNString))
        {
            sb.Append(" VISNs: All |");
        }
        else
        {
            sb.Append(" VISNs: " + tempVISNString + " |");
        }

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    /// <summary>
    /// Selected Index changed on VISN list.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbVisn.CheckedItems.Count > 0)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            // disable the run button
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        rgMammoCompliance.MasterTableView.SortExpressions.Clear();

        DateTime start = DateTime.Now;

        FillReportGrid();
        rgMammoCompliance.DataBind();

        DateTime end = DateTime.Now;
        lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rgMammoCompliance.MasterTableView.SortExpressions.Clear();

        rgMammoCompliance.DataSource = "";
        rgMammoCompliance.DataBind();

        rcbVisn.ClearSelection();
        rcbVisn.ClearCheckedItems();

        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgMammoCompliance.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgMammoCompliance.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgMammoCompliance.MasterTableView.AllowPaging = false;
        rgMammoCompliance.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgMammoCompliance.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgMammoCompliance.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgMammoCompliance.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgMammoCompliance.MasterTableView.Items.Count.ToString();
        rgMammoCompliance.ExportSettings.Pdf.ForceTextWrap = true;

        rgMammoCompliance.MasterTableView.Caption = reportHeaderString.ToString();
        rgMammoCompliance.Rebind();
        rgMammoCompliance.ExportSettings.FileName = hfCustomReportName.Value;
        rgMammoCompliance.ExportSettings.IgnorePaging = true;
        rgMammoCompliance.ExportSettings.OpenInNewWindow = true;
        rgMammoCompliance.ExportSettings.HideStructureColumns = true;

        rgMammoCompliance.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgMammoCompliance.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgMammoCompliance.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgMammoCompliance.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgMammoCompliance.Rebind();
    }

    #endregion
}