﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammogramCompliance_ByFacility_Agg_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;
    protected string sta3n;
    protected string ages;
    protected string race;
    private Aggregate agg;

    protected string reportName = "Mammograms Compliance by Facility Aggregate Report";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoCompBySta3nPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoCompBySta3nPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoCompBySta3nPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            sta3n = Request.QueryString.Get("sta3n");
            ages = Request.QueryString.Get("ages");
            race = Request.QueryString.Get("race");

            LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > <a href=\"MammogramCompliance_Agg_Rpt.aspx\" title=\"Click to navigate to the Mammogram Compliance Report page\" style=\"color:#FFFFFF;\">Mammogram Compliance</a> > Mammogram Compliance by Facility (" + sta3n + ")";

            UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            agg = new Aggregate();

            if (UserPermittedToViewFacility(sta3n))
            {

                if (!IsPostBack)
                {
                    // Fill the Race list
                    FillRaces();

                    // set initial values
                    if (!String.IsNullOrEmpty(race))
                        rcbRace.SelectedValue = race;

                    if (!String.IsNullOrEmpty(ages))
                        rddlAgeRestriction.SelectedValue = ages;

                    FillReportGrid();
                    rgMammoCompliance.DataBind();
                }
            }
            else
            {
                Logging.WriteLogToFile(_userId, "User Not Authorized Exception.  Facility " + sta3n + " is not in the current user's available facility list.  Please contact a system administrator to troubleshoot.");

                // no access.
                Response.Redirect("ErrorPage.aspx?id=FAC");

                //Response.Flush();
                //Response.Write("User Not Authorized Exception.  Facility " + sta3n + " is not in the current user's available facility list.  Please contact a system administrator to troubleshoot.");
                //Response.End();
                //throw new Exception("User Not Authorized Exception.  Facility " + sta3n + " is not in the current user's available facility list.  Please contact a system administrator to troubleshoot.");
            }
        }
        catch (Exception ex)
        {
            //TODO: notify user
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Mammogram Compliance Report By Facility";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgMammoCompliance_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgMammoCompliance_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        if (e.CommandName == "Details")
        {
            if (e.Item is GridDataItem)
            {
                //item = ((GridDataItem)e.Item);
                //int patientSID = (int)item.GetDataKeyValue("PatientSID");

                int rowIndex = Convert.ToInt32(e.CommandArgument);
                int patientSID = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["PatientSID"];

                Response.Redirect("PatientDetails.aspx?id=" + patientSID);
            }
        }
    }

    protected void rgMammoCompliance_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgMammoCompliance_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                string[] fieldNames = { "NofificationStatement", "FollowUpActionStatement" };
                string reportName = "BB_";

                if (isExport)
                {
                    string columnName;
                    TableCell ctc;

                    // Inspect link fields and format accordingly for the output report

                    foreach (string columnRoot in fieldNames)
                    {
                        item = (GridDataItem)e.Item;
                        columnName = reportName + columnRoot + "_Report";
                        ctc = item[columnName];
                        Helpers.configureLinkForExport(ctc, columnRoot, reportName, "");
                    }

                    // Format cells
                    foreach (TableCell cell in e.Item.Cells)
                    {
                        cell.Style["mso-number-format"] = @"\@";
                    }
                }
                else // Not exporting, just displaying grid
                {
                    Label lbl;
                    string sData;
                    RadToolTip rtt;

                    // Inspect each link field and create links or appropriate message

                    foreach (string columnRoot in fieldNames)
                    {

                        lbl = (Label)item.FindControl("lbl" + reportName + columnRoot);
                        sData = ((System.Data.DataRowView)e.Item.DataItem)[columnRoot].ToString();
                        rtt = (RadToolTip)item.FindControl("rtt" + reportName + columnRoot);

                        Helpers.setupLink(lbl, sData, rtt, columnRoot, reportName);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgMammoCompliance_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgMammoCompliance.CurrentPageIndex = 0;
        executeSortExpression(e.SortExpression, e.NewSortOrder);
    }

    /// <summary>
    /// Paging method to keep custom sorting intact.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMammoCompliance_PageIndexChanged(object sender, GridPageChangedEventArgs e)
    {
        foreach (GridSortExpression sortE in rgMammoCompliance.MasterTableView.SortExpressions)
        {
            executeSortExpression(sortE.FieldName, sortE.SortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// returns plain vistaname if not "current" ... if current, returns hyperlink for drill down.
    /// </summary>
    /// <param name="vistaname"></param>
    /// <returns></returns>
    protected string GetVISTANAMEText(object vistaname, object sta3n)
    {
        //if (vistaname != null)
        //{
        //    if (String.IsNullOrEmpty(rcbDatePicker.SelectedValue))
        //    {
        //        // current ... provide link
        //        return "<a href=\"MammogramCompliance_ByFacility_Agg_Rpt.aspx?sta3n=" + sta3n.ToString() + "\" title=\"Click to View Report by Facility\">" + vistaname.ToString() + "</a>";
        //    }
        //    else
        //    {
        // historical .. .no link
        return vistaname.ToString();
        //    }
        //}

        //return "";
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        try
        {
            DateTime start = DateTime.Now;
            DataTable dt;

            //bind the data to the grid
            rgMammoCompliance.DataSource = getDataset();

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
        catch (Exception ex)
        {
            //if error
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        try
        {
            string complianceStatus = rcbComplianceStatus.SelectedValue;

            if (complianceStatus == "all")
            {
                complianceStatus = null;
            }

            string ageRestrictString = "";
            ageRestrictString = ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? null : rddlAgeRestriction.SelectedValue);

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            return agg.GetMammogramComplianceBySta3nAgg(sta3n, complianceStatus, ageRestrictString, raceRestrictString);
        }
        catch (Exception ex)
        {
            //if error
            Logging.WriteLogToFile(_userId, ex);

            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(string sortExpression, GridSortOrder newSortOrder)
    {
        DataTable dt = getDataset();
        if (dt != null)
        {
            if (sortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (sortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgMammoCompliance.DataSource = dt.DefaultView.ToTable();
            rgMammoCompliance.DataBind();
        }
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        sb.Append(" Facility: " + sta3n + " |");

        sb.Append(" Compliance Status: " + rcbComplianceStatus.SelectedItem.Text + " |");

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Methods / Helpers

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1" || String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbtnRunReport_Click(object sender, EventArgs e)
    {
        FillReportGrid();
        rgMammoCompliance.DataBind();
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rgMammoCompliance.DataSource = "";
        rgMammoCompliance.DataBind();

        rcbComplianceStatus.SelectedIndex = 0;

        if (!String.IsNullOrEmpty(ages))
        {
            try
            {
                rddlAgeRestriction.SelectedValue = ages;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rddlAgeRestriction.SelectedIndex = 0;
        }

        if (!String.IsNullOrEmpty(race))
        {
            try
            {
                rcbRace.SelectedValue = race;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rcbRace.SelectedIndex = 0;
        }

        rbtnRunReport.Enabled = true;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgMammoCompliance.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgMammoCompliance.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgMammoCompliance.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgMammoCompliance.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgMammoCompliance.MasterTableView.AllowPaging = false;
        rgMammoCompliance.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgMammoCompliance.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgMammoCompliance.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgMammoCompliance.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgMammoCompliance.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgMammoCompliance.MasterTableView.Items.Count.ToString();
        rgMammoCompliance.ExportSettings.Pdf.ForceTextWrap = true;

        rgMammoCompliance.MasterTableView.Caption = reportHeaderString.ToString();
        rgMammoCompliance.Rebind();
        rgMammoCompliance.ExportSettings.FileName = hfCustomReportName.Value;
        rgMammoCompliance.ExportSettings.IgnorePaging = true;
        rgMammoCompliance.ExportSettings.OpenInNewWindow = true;
        rgMammoCompliance.ExportSettings.HideStructureColumns = true;

        rgMammoCompliance.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgMammoCompliance.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgMammoCompliance.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgMammoCompliance.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgMammoCompliance.Rebind();
    }

    #endregion
}