﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammogramFollowupComp_BySta3n_Agg_Rpt : BasePage
{
    #region Member Variables

    protected string sta3n = "";
    protected string race;

    private string _userId;
    private Aggregate agg;
    bool isExport = false;
    private Reports r;

    protected string reportName = "Mammogram Follow-Up Compliance By Facility";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoFollowUpPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoFollowUpPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoFollowUpPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        Response.Cache.SetCacheability(HttpCacheability.NoCache);
        Response.Cache.SetExpires(DateTime.UtcNow.AddHours(-1));
        Response.Cache.SetNoStore();

        rbtnRunReportTop.Enabled = rbtnRunReportBottom.Enabled = true;
        try
        {
            sta3n = Request.QueryString.Get("sta3n");
            race = Request.QueryString.Get("race");

            LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > <a href=\"MammogramFollowupCompliance_Agg_Rpt.aspx\" title=\"Click to navigate to the Mammogram Follow-Up Compliance Report page\" style=\"color:#FFFFFF;\">Mammogram Follow-Up Compliance</a> > Mammogram Follow-Up Compliance by Facility (" + sta3n + ")";

            UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;
            
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            agg = new Aggregate();

            if (UserPermittedToViewFacility(sta3n))
            {

                if (!IsPostBack)
                {
                    //set date - if dates are not on request string ... default
                    rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
                    rdpEndDate.SelectedDate = DateTime.Today;
                    rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

                    // Fill the Race list
                    FillRaces();

                    // set initial values
                    if (!String.IsNullOrEmpty(race))
                        rcbRace.SelectedValue = race;

                    FillReportGrid();
                    //rgMammoFollowUpCompliance.DataSource = "";
                    rgMammoFollowUpCompliance.DataBind();
                }
            }
            else
            {
                //throw new Exception("User Not Authorized Exception.  Facility " + sta3n + " is not in the current user's available facility list.  Please contact a system administrator to troubleshoot.");
                Logging.WriteLogToFile(_userId, "User Not Authorized Exception.  Facility " + sta3n + " is not in the current user's available facility list.  Please contact a system administrator to troubleshoot.");

                // no access.
                Response.Redirect("ErrorPage.aspx?id=FAC");
            }
        }
        catch (Exception ex)
        {
            //TODO: notify user
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Mammogram Follow-Up Compliance by Facility Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgMammoFollowUpCompliance_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgMammoFollowUpCompliance_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        if (e.CommandName == "Details")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);
                int patientSID = (int)item.GetDataKeyValue("PatientSID");

                Response.Redirect("PatientDetails.aspx?id=" + patientSID);
            }
        }
    }

    protected void rgMammoFollowUpCompliance_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgMammoFollowUpCompliance_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                string[] fieldNames = { "NotificationStatement", "FollowUpActionStatement" };
                string reportName = "BB_";

                if (isExport)
                {
                    string columnName;
                    TableCell ctc;

                    // Inspect link fields and format accordingly for the output report

                    foreach (string columnRoot in fieldNames)
                    {
                        item = (GridDataItem)e.Item;
                        columnName = reportName + columnRoot + "_Report";
                        ctc = item[columnName];
                        Helpers.configureLinkForExport(ctc, columnRoot, reportName, "");
                    }

                    // Format cells
                    foreach (TableCell cell in e.Item.Cells)
                    {
                        cell.Style["mso-number-format"] = @"\@";
                    }
                }
                else // Not exporting, just displaying grid
                {
                    Label lbl;
                    string sData;
                    RadToolTip rtt;

                    // Inspect each link field and create links or appropriate message

                    foreach (string columnRoot in fieldNames)
                    {

                        lbl = (Label)item.FindControl("lbl" + reportName + columnRoot);
                        sData = ((System.Data.DataRowView)e.Item.DataItem)[columnRoot].ToString();
                        rtt = (RadToolTip)item.FindControl("rtt" + reportName + columnRoot);

                        Helpers.setupLink(lbl, sData, rtt, columnRoot, reportName);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgMammoFollowUpCompliance_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgMammoFollowUpCompliance.CurrentPageIndex = 0;

        DataTable dt = getDataset();
        if (dt != null)
        {
            if (e.SortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (e.NewSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (e.SortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (e.NewSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (e.NewSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = e.SortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = e.SortExpression + " DESC";
                        break;
                }
            }

            rgMammoFollowUpCompliance.DataSource = dt.DefaultView.ToTable();
            rgMammoFollowUpCompliance.DataBind();
        }
    }

    /// <summary>
    /// Paging method to keep custom sorting intact.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMammoFollowUpCompliance_PageIndexChanged(object sender, GridPageChangedEventArgs e)
    {
        foreach (GridSortExpression sortE in rgMammoFollowUpCompliance.MasterTableView.SortExpressions)
        {
            executeSortExpression(sortE.FieldName, sortE.SortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// returns plain vistaname if not "current" ... if current, returns hyperlink for drill down.
    /// </summary>
    /// <param name="vistaname"></param>
    /// <returns></returns>
    protected string GetVISTANAMEText(object vistaname, object sta3n)
    {
        //if (vistaname != null)
        //{
        //    if (String.IsNullOrEmpty(rcbDatePicker.SelectedValue))
        //    {
        //        // current ... provide link
        //        return "<a href=\"MammogramCompliance_ByFacility_Agg_Rpt.aspx?sta3n=" + sta3n.ToString() + "\" title=\"Click to View Report by Facility\">" + vistaname.ToString() + "</a>";
        //    }
        //    else
        //    {
        // historical .. .no link
        return vistaname.ToString();
        //    }
        //}

        //return "";
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1" || String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbtnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            FillReportGrid();
            rgMammoFollowUpCompliance.DataBind();
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rgMammoFollowUpCompliance.DataSource = "";
        rgMammoFollowUpCompliance.DataBind();

        rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
        rdpEndDate.SelectedDate = DateTime.Today;
        rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

        rcbFollowUpActionStatus.ClearSelection();

        if (!String.IsNullOrEmpty(race))
        {
            try
            {
                rcbRace.SelectedValue = race;
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_userId, ex);
            }
        }
        else
        {
            rcbRace.SelectedIndex = 0;
        }

        rbtnRunReportTop.Enabled = rbtnRunReportBottom.Enabled = true;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgMammoFollowUpCompliance.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgMammoFollowUpCompliance.MasterTableView.AllowPaging = false;
        rgMammoFollowUpCompliance.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgMammoFollowUpCompliance.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgMammoFollowUpCompliance.MasterTableView.Items.Count.ToString();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.ForceTextWrap = true;

        rgMammoFollowUpCompliance.MasterTableView.Caption = reportHeaderString.ToString();
        rgMammoFollowUpCompliance.Rebind();
        rgMammoFollowUpCompliance.ExportSettings.FileName = hfCustomReportName.Value;
        rgMammoFollowUpCompliance.ExportSettings.IgnorePaging = true;
        rgMammoFollowUpCompliance.ExportSettings.OpenInNewWindow = true;
        rgMammoFollowUpCompliance.ExportSettings.HideStructureColumns = true;

        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgMammoFollowUpCompliance.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgMammoFollowUpCompliance.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgMammoFollowUpCompliance.Rebind();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        try
        {
            rgMammoFollowUpCompliance.DataSource = getDataset();
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            //if error
            rgMammoFollowUpCompliance.DataSource = "";
        }
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset() {
        try
        {
            DateTime start = DateTime.Now;
            DataTable dt;

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            string followUpStatusString = (rcbFollowUpActionStatus.SelectedValue == "all") ? null : rcbFollowUpActionStatus.SelectedValue;

            dt = agg.GetMammogramFollowupComplianceBySta3nAgg(sta3n, ((DateTime)rdpStartDate.SelectedDate).ToShortDateString(), ((DateTime)rdpEndDate.SelectedDate).ToShortDateString(), raceRestrictString, followUpStatusString);

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";

            //bind the data to the grid
            return dt;
        }
        catch (Exception ex)
        {
            //if error
            Logging.WriteLogToFile(_userId, ex);

            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(string sortExpression, GridSortOrder newSortOrder)
    {
        DataTable dt = getDataset();
        if (dt != null)
        {
            if (sortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (sortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgMammoFollowUpCompliance.DataSource = dt.DefaultView.ToTable();
            rgMammoFollowUpCompliance.DataBind();
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        sb.Append(" Facility: " + sta3n + " |");

        // Start Date - rdpStartDate
        if (rdpStartDate.SelectedDate != null)
        {
            sb.Append(" Start Date: " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + " |");
        }

        // End Date - rdpEndDate
        if (rdpEndDate.SelectedDate != null)
        {
            sb.Append(" End Date: " + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        }

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion
}