﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammogramFollowupCompliance_Agg_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;
    private Aggregate agg;

    protected string reportName = "Mammogram Follow-Up Compliance Aggregate Report";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsMammoFollowUpPDFExport"] != null)
            {
                return (bool)ViewState["IsMammoFollowUpPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsMammoFollowUpPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > Mammograms Follow-Up Compliance";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            agg = new Aggregate();

            if (!IsPostBack)
            {
                rgMammoFollowUpCompliance.DataSource = "";
                rgMammoFollowUpCompliance.DataBind();

                //Fill the facilities listbox
                FillFACs();

                // Fill the Race list
                FillRaces();

                //set date
                rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
                rdpEndDate.SelectedDate = DateTime.Today;
                rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

                // history?
                //rcbDatePicker.DataSource = agg.GetMammoComplianceHistoricalMonthSIDs();
                //rcbDatePicker.DataBind();

                //rcbDatePicker.Items.Insert(0, new RadComboBoxItem("Current", null));
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Mammogram Compliance Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgMammoFollowUpCompliance_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgMammoFollowUpCompliance_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        if (e.CommandName == "Details")
        {
            if (e.Item is GridDataItem)
            {
                int rowIndex = Convert.ToInt32(e.CommandArgument);
                string selectedSta3n = e.Item.OwnerTableView.DataKeyValues[rowIndex]["Sta3n"].ToString();

                string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

                //item = ((GridDataItem)e.Item);
                //string selectedSta3n = item.GetDataKeyValue("Sta3n").ToString();

                //Response.Redirect("MammogramFollowupComp_BySta3n_Agg_Rpt.aspx?sta3n=" + selectedSta3n);
                Response.Redirect(string.Format("MammogramFollowupComp_BySta3n_Agg_Rpt.aspx?sta3n={0}&race={1}", selectedSta3n, raceRestrictString));
            }
        }
    }

    protected void rgMammoFollowUpCompliance_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }
    }

    protected void rgMammoFollowUpCompliance_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridFilteringItem)
            {

            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                // Get drill down button and check permissions - if user does not have access to see a VISN, it hides the drill-down option.
                LinkButton drillButton = (LinkButton)item["DetailButton"].Controls[0];
                drillButton.Visible = UserPermittedToViewFacility(item.GetDataKeyValue("Sta3n").ToString());
            }

            if (isExport && e.Item is GridDataItem)
            {
                foreach (TableCell cell in e.Item.Cells)
                {
                    //Approach 1
                    cell.Style["mso-number-format"] = @"\@";
                }
            }

            // footer
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgMammoFollowUpCompliance_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgMammoFollowUpCompliance.CurrentPageIndex = 0;
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    /// <summary>
    /// Helper Method to return VISN Name from ID
    /// </summary>
    /// <param name="visnId"></param>
    /// <returns></returns>
    protected string GetVISNNameFromID(object visnId)
    {
        if (visnId != null)
        {
            // rather than calling database - find VISN name in VISN list.
            RadComboBoxItem item = rcbVisn.FindItemByValue(visnId.ToString());
            if (item != null)
            {
                return item.Text;
            }
        }

        return "";
    }

    /// <summary>
    /// returns plain vistaname if not "current" ... if current, returns hyperlink for drill down.
    /// </summary>
    /// <param name="vistaname"></param>
    /// <returns></returns>
    protected string GetVISTANAMEText(object vistaname, object sta3n)
    {
        //if (vistaname != null)
        //{
        //    if (String.IsNullOrEmpty(rcbDatePicker.SelectedValue))
        //    {
        //        // current ... provide link
        //        return "<a href=\"MammogramCompliance_ByFacility_Agg_Rpt.aspx?sta3n=" + sta3n.ToString() + "\" title=\"Click to View Report by Facility\">" + vistaname.ToString() + "</a>";
        //    }
        //    else
        //    {
                // historical .. .no link
                return vistaname.ToString();
        //    }
        //}

        //return "";
    }

    /// <summary>
    /// Determines if use has permissions to see facility ... if not, hides option to drill down.
    /// </summary>
    /// <param name="sta3n"></param>
    /// <returns></returns>
    protected bool UserPermittedToViewFacility(string sta3n)
    {
        if (r.GetFacilitiesByUserId())
        {
            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (dr["sta3n"] != null)
                {
                    if (sta3n == dr["sta3n"].ToString())
                        return true;
                }
            }
        }
        return false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        rgMammoFollowUpCompliance.DataSource = getDataset();
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        try
        {
            DataTable dt;

            #region Determine VISNs

            var visnList = rcbVisn.CheckedItems;
            string tempVISNString = "";

            if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
            {
                tempVISNString = null; // all
            }
            else
            {
                foreach (var entry in visnList)
                {
                    tempVISNString += entry.Value + ",";
                }

                if (tempVISNString.EndsWith(","))
                {
                    tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
                }

                if (String.IsNullOrEmpty(tempVISNString))
                    tempVISNString = null;
            }

            #endregion

            string raceRestrictString = (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

            dt = agg.GetMammogramFollowupComplianceByVISNAgg(tempVISNString, ((DateTime)rdpStartDate.SelectedDate).ToShortDateString(), ((DateTime)rdpEndDate.SelectedDate).ToShortDateString(), raceRestrictString);

            // Create new DataTable for chart - limit number of VISNs allowed.
            //if (rcbVisn.CheckedItems.Count > 1)
            //{
            //    DataTable newDT = dt.AsEnumerable().Skip(0).Take(Convert.ToInt32(ConfigurationManager.AppSettings["NumMMGComplianceFacOnChart"].ToString())).CopyToDataTable();

            //    lblShowingNumber.Text = "Showing the first " + ConfigurationManager.AppSettings["NumMMGComplianceFacOnChart"].ToString() + " facilities of report";

            //    pnlChart.Visible = true;
            //    rhcComplianceChart.DataSource = newDT;
            //    rhcComplianceChart.DataBind();
            //}
            //else
            //{
                // set chart data source

                lblShowingNumber.Text = "Showing all facilities in report";

                pnlChart.Visible = true;
                rhcComplianceChart.DataSource = dt;
                rhcComplianceChart.DataBind();
            //}

            if (dt != null)
            {
                return dt;
            }

            return null;
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            return null;
        }
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            // populate all VISNs
            DataTable dtVisns = r.GetVISNs();

            if (dtVisns != null)
            {
                rcbVisn.DataSource = dtVisns;
                rcbVisn.DataTextField = "visnText";
                rcbVisn.DataValueField = "visnId";
                rcbVisn.DataBind();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    private void FillRaces()
    {
        rcbRace.Items.Clear();
        DataTable tempDataTable = r.GetDistinctPatientRaces();
        DataRow dr = tempDataTable.NewRow();
        dr["RACE"] = "All";
        tempDataTable.Rows.Add(dr);

        DataView vw = new DataView(tempDataTable);
        vw.Sort = "[RACE] asc";

        rcbRace.DataSource = vw;
        rcbRace.DataTextField = "RACE";
        rcbRace.DataValueField = "RACE";
        rcbRace.DataBind();
    }

    /// <summary>
    /// Selected Index changed on VISN list.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbVisn.CheckedItems.Count > 0)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            // disable the run button
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISNs
        var visnList = rcbVisn.CheckedItems;
        string tempVISNString = "";

        if (rcbVisn.CheckedItems.Count == rcbVisn.Items.Count)
        {
            tempVISNString = null; // all
        }
        else
        {
            foreach (var entry in visnList)
            {
                tempVISNString += entry.Text + ",";
            }

            if (tempVISNString.EndsWith(","))
            {
                tempVISNString = tempVISNString.Substring(0, tempVISNString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempVISNString))
                tempVISNString = null;
        }

        if (String.IsNullOrEmpty(tempVISNString))
        {
            sb.Append(" VISNs: All |");
        }
        else
        {
            sb.Append(" VISNs: " + tempVISNString + " |");
        }

        // Start Date - rdpStartDate
        if (rdpStartDate.SelectedDate != null)
        {
            sb.Append(" Start Date: " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + " |");
        }

        // End Date - rdpEndDate
        if (rdpEndDate.SelectedDate != null)
        {
            sb.Append(" End Date: " + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        }

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            rgMammoFollowUpCompliance.MasterTableView.SortExpressions.Clear();

            DateTime start = DateTime.Now;

            FillReportGrid();
            rgMammoFollowUpCompliance.DataBind();

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rgMammoFollowUpCompliance.MasterTableView.SortExpressions.Clear();

        rdpStartDate.SelectedDate = DateTime.Today.AddYears(-1);
        rdpEndDate.SelectedDate = DateTime.Today;
        rdpEndDate.MinDate = (DateTime)rdpStartDate.SelectedDate;

        rgMammoFollowUpCompliance.DataSource = "";
        rgMammoFollowUpCompliance.DataBind();

        rhcComplianceChart.DataSource = "";
        rhcComplianceChart.DataBind();
        pnlChart.Visible = false;

        rcbVisn.ClearSelection();
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgMammoFollowUpCompliance.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgMammoFollowUpCompliance.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgMammoFollowUpCompliance.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgMammoFollowUpCompliance.MasterTableView.AllowPaging = false;
        rgMammoFollowUpCompliance.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgMammoFollowUpCompliance.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgMammoFollowUpCompliance.MasterTableView.Items.Count.ToString();
        rgMammoFollowUpCompliance.ExportSettings.Pdf.ForceTextWrap = true;

        rgMammoFollowUpCompliance.MasterTableView.Caption = reportHeaderString.ToString();
        rgMammoFollowUpCompliance.Rebind();
        rgMammoFollowUpCompliance.ExportSettings.FileName = hfCustomReportName.Value;
        rgMammoFollowUpCompliance.ExportSettings.IgnorePaging = true;
        rgMammoFollowUpCompliance.ExportSettings.OpenInNewWindow = true;
        rgMammoFollowUpCompliance.ExportSettings.HideStructureColumns = true;

        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgMammoFollowUpCompliance.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgMammoFollowUpCompliance.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgMammoFollowUpCompliance.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("GUI") || col.UniqueName == "EditCommandColumn")
                col.Visible = false;
        }
        rgMammoFollowUpCompliance.Rebind();
    }

    #endregion
}