﻿using System;
using System.Data;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_MammosDue_Std_Rpt : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private Reports r;
    protected string reportName = "Mammograms Due / Ordered Pending";

    private bool isPdfExport;
    public bool IsPDFExport
    {
        get
        {
            if (ViewState["IsPDFExport"] != null)
            {
                return (bool)ViewState["IsPDFExport"];
            }
            else
            {
                return false;
            }
        }

        set
        {
            isPdfExport = value;
            ViewState["IsPDFExport"] = value;
        }
    }

    #endregion

    /// <summary>
    /// Default Page_Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > Mammograms Due";

        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            r = new Reports(_userId);

            // Sets Last Refresh Date.
            lblLastRefreshDate.Text = Reports.GetETLRefreshDate().ToString();

            if (!IsPostBack)
            {
                rgMammosDueOrderedPending.DataSource = "";
                rgMammosDueOrderedPending.DataBind();

                //set date
                //rdpStartDate.SelectedDate = DateTime.Today;
                //rdpEndDate.SelectedDate = DateTime.Today.AddDays(7);
                //rdpEndDate.MinDate = DateTime.Today;

                //Fill the facilities listbox
                FillFACs();

                // Fill the Race list
                //FillRaces();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region Private Methods

    /// <summary>
    /// Perform actual search. Call stored procedure with parameters.
    /// </summary>
    private void FillReportGrid()
    {
        rgMammosDueOrderedPending.DataSource = getDataset();
    }

    /// <summary>
    /// Returns a DataTable Dataset for the parameters.
    /// </summary>
    /// <returns></returns>
    private DataTable getDataset()
    {
        PatientData pd = new PatientData();

        try
        {
            if (rcbFac.SelectedItem != null)
            {
                //get the report data

                string searchPatName = String.IsNullOrEmpty(rtbPatientName.Text) ? null : rtbPatientName.Text;
                string searchProviderName = String.IsNullOrEmpty(rtbProviderName.Text) ? null : rtbProviderName.Text;

                string searchGender = null;
                if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
                {
                    searchGender = null;
                }
                else
                {
                    searchGender = rcbGender.SelectedValue;
                }

                string ageRestrictString = "";
                ageRestrictString = (String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) ? null : rddlAgeRestriction.SelectedValue);

                string mammotypeString = "";
                mammotypeString = (String.IsNullOrEmpty(rddlMammogramType.SelectedValue) ? null : rddlMammogramType.SelectedValue);

                DateTime tempStartDate;
                DateTime tempEndDate;

                // Determine Date Range
                //switch (rcbDateRangeSelection.SelectedValue)
                //{
                //    case "custom":
                //        tempStartDate = (DateTime)rdpStartDate.SelectedDate;
                //        tempEndDate = (DateTime)rdpEndDate.SelectedDate;
                //        break;
                //    case "overdueLessThan60":
                //        tempStartDate = DateTime.Now.AddDays(-60);
                //        tempEndDate = DateTime.Now.AddDays(-1);
                //        break;
                //    case "overdueOver60":
                //        tempStartDate = DateTime.Now.AddMonths(-6);
                //        tempEndDate = DateTime.Now.AddDays(-60);
                //        break;
                //    default:
                //        tempStartDate = DateTime.Now.AddDays(-60);
                //        tempEndDate = DateTime.Now.AddDays(-1);
                //        break;
                //}

                // Determine Date Range
                switch (rcbDateRange.SelectedValue)
                {
                    case "1week":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddDays(7);
                        break;
                    case "2week":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddDays(14);
                        break;
                    case "1month":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddMonths(1);
                        break;
                    case "2month":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddMonths(2);
                        break;
                    case "3month":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddMonths(3);
                        break;
                    case "6month":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddMonths(6);
                        break;
                    case "1year":
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddYears(1);
                        break;
                    case "due":
                        tempStartDate = DateTime.Now.AddYears(-2);
                        tempEndDate = DateTime.Now.AddDays(0);
                        break;
                    case "custom":
                        tempStartDate = (DateTime)rdpStartDate.SelectedDate;
                        tempEndDate = (DateTime)rdpEndDate.SelectedDate;
                        break;
                    case "overdueLessThan60": // up to 3 months.
                        tempStartDate = DateTime.Now.AddDays(-60);
                        tempEndDate = DateTime.Now.AddDays(-1);
                        break;
                    case "overdueOver60": // up to 3 months.
                        tempStartDate = DateTime.Now.AddMonths(-6);
                        tempEndDate = DateTime.Now.AddDays(-60);
                        break;
                    default:
                        // 1 week
                        tempStartDate = DateTime.Now;
                        tempEndDate = tempStartDate.AddDays(7);
                        break;
                }

                string raceRestrictString = null; //= (rcbRace.SelectedValue == "All") ? null : rcbRace.SelectedValue;

                //PACT TEAM STRING CREATION
                var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
                string pactTeamString = "";

                if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
                {
                    pactTeamString = null; // all
                }
                else
                {
                    foreach (var entry in pactTeamList)
                    {
                        pactTeamString += entry.Value + ",";
                    }

                    if (pactTeamString.EndsWith(","))
                    {
                        pactTeamString = pactTeamString.Substring(0, pactTeamString.Length - 1);
                    }

                    if (String.IsNullOrEmpty(pactTeamString))
                        pactTeamString = null;
                }

                DataTable dt = pd.GetMammogramsDue(rcbFac.SelectedValue, tempStartDate.ToShortDateString(), tempEndDate.ToShortDateString(), ageRestrictString, searchPatName, searchGender, searchProviderName, mammotypeString, raceRestrictString, pactTeamString);

                //bind the data to the grid
                return dt;
            }
            else
            {
                //if no selected facility
                return null;
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);

            //if error
            return null;
        }
    }

    /// <summary>
    /// executes custom sort
    /// </summary>
    /// <param name="sortExpression"></param>
    /// <param name="newSortOrder"></param>
    private void executeSortExpression(string sortExpression, GridSortOrder newSortOrder)
    {
        DataTable dt = getDataset();
        if (dt != null)
        {
            if (sortExpression == "PCMMTeam")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMTeam is NULL OR PCMMTeam = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam DESC";
                        break;
                }
            }
            else if (sortExpression == "PCMMStaffName")
            {
                dt.Columns.Add("NullEmptyCheck", typeof(int), "PCMMStaffName is NULL OR PCMMStaffName = ''");

                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMStaffName DESC";
                        break;
                }
            }
            else
            {
                switch (newSortOrder)
                {
                    case GridSortOrder.None:
                        //                    dt.DefaultView.Sort = "NullEmptyCheck ASC, PCMMTeam ASC";
                        break;
                    case GridSortOrder.Ascending:
                        dt.DefaultView.Sort = sortExpression + " ASC";
                        break;
                    case GridSortOrder.Descending:
                        dt.DefaultView.Sort = sortExpression + " DESC";
                        break;
                }
            }

            rgMammosDueOrderedPending.DataSource = dt.DefaultView.ToTable();
            rgMammosDueOrderedPending.DataBind();
        }
    }

    /// <summary>
    /// Fills the facility drop down with the facilities the user has access to see.
    /// </summary>
    private void FillFACs()
    {
        try
        {
            //get the user's facilities and populate listboxs
            if (r.GetFacilitiesByUserId())
            {
                // Station drop down clear existing items

                rcbFac.Items.Clear();
                rcbFac.EmptyMessage = "Select a Station";

                // VISN drop down clear existing items

                rcbVisn.Items.Clear();
                rcbVisn.EmptyMessage = "Select a VISN";

                System.Data.DataTable tVisns = new DataTable();

                SetupVisnList(tVisns);

                DataView vw = new DataView(tVisns);
                vw.Sort = "[visnId] asc";

                rcbVisn.DataSource = vw;
                rcbVisn.DataTextField = "visnText";
                rcbVisn.DataValueField = "visnId";
                rcbVisn.DataBind();

                // If there's only one option, select it for them
                if (rcbVisn.Items.Count == 1)
                {
                    rcbVisn.SelectedIndex = 0;
                    rcbVisn.Enabled = false;
                    ProcessVisnSelection();
                }
                else if (rcbVisn.Items.Count == 0)
                {
                    rcbVisn.EmptyMessage = "You do not have access to any VISNs to select";
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Fills the dropdown list for race filters.
    /// </summary>
    //private void FillRaces()
    //{
    //    rcbRace.Items.Clear();
    //    DataTable tempDataTable = r.GetDistinctPatientRaces();
    //    DataRow dr = tempDataTable.NewRow();
    //    dr["RACE"] = "All";
    //    tempDataTable.Rows.Add(dr);

    //    DataView vw = new DataView(tempDataTable);
    //    vw.Sort = "[RACE] asc";

    //    rcbRace.DataSource = vw;
    //    rcbRace.DataTextField = "RACE";
    //    rcbRace.DataValueField = "RACE";
    //    rcbRace.DataBind();
    //}

    /// <summary>
    /// Populates VISN list based on the facilities that a user has access to
    /// </summary>
    /// <param name="tVisns"></param>
    private void SetupVisnList(System.Data.DataTable tVisns)
    {
        try
        {
            DataColumn idColumn = new DataColumn();
            idColumn = new DataColumn();
            idColumn.DataType = System.Type.GetType("System.Int32");
            idColumn.ColumnName = "visnId";
            idColumn.ReadOnly = true;
            idColumn.Unique = true;

            DataColumn nameColumn = new DataColumn();
            nameColumn = new DataColumn();
            nameColumn.DataType = System.Type.GetType("System.String");
            nameColumn.ColumnName = "visnText";
            nameColumn.ReadOnly = true;
            nameColumn.Unique = true;

            tVisns.Columns.Add(idColumn);
            tVisns.Columns.Add(nameColumn);

            tVisns.PrimaryKey = new DataColumn[] { tVisns.Columns["visnId"] };

            DataRow drVisn;

            foreach (System.Data.DataRow dr in r.dtFac.Rows)
            {
                if (!tVisns.Rows.Contains(dr["visnId"]))
                {
                    drVisn = tVisns.NewRow();
                    drVisn["visnId"] = dr["visnId"];
                    drVisn["visnText"] = dr["visnText"];

                    tVisns.Rows.Add(drVisn);
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// reset filters (non-location)
    /// </summary>
    private void clearNonFacilityFilters()
    {
        //rcbDateRangeSelection.SelectedIndex = 0;
        rfvStartDate.Enabled = rfvEndDate.Enabled = false;
        pnlCustomDateRanges.Visible = false;

        rcbDateRange.SelectedIndex = 0;

        rddlAgeRestriction.SelectedIndex = 0;
        rddlMammogramType.SelectedIndex = 0;
        rtbPatientName.Text = "";
        rcbGender.SelectedIndex = 0;
        rtbProviderName.Text = "";

        rdpStartDate.SelectedDate = DateTime.Today;
        rdpEndDate.SelectedDate = DateTime.Today.AddDays(7);
        rdpEndDate.MinDate = DateTime.Today;

        rcbPACTTeamsForFacility.ClearCheckedItems();
        rcbPACTTeamsForFacility.ClearSelection();
        rcbPACTTeamsForFacility.DataSource = "";
        rcbPACTTeamsForFacility.DataBind();
        rcbPACTTeamsForFacility.Enabled = false;
    }

    /// <summary>
    /// Determines what filters are applied and what the values are
    /// </summary>
    /// <returns></returns>
    private string getCurrentFiltersAndValues()
    {
        StringBuilder sb = new StringBuilder();

        // VISN - rcbVisn
        if (rcbVisn.SelectedValue != null)
        {
            sb.Append(" VISN: " + rcbVisn.SelectedItem.Text + " |");
        }

        // Facility - rcbFac
        if (rcbFac.SelectedValue != null)
        {
            sb.Append(" Facility: " + rcbFac.SelectedItem.Text + " |");
        }

        // PACT Team List - rcbBiRadsList
        var pactTeamList = rcbPACTTeamsForFacility.CheckedItems;
        string tempPACTString = "";

        if (rcbPACTTeamsForFacility.CheckedItems.Count == rcbPACTTeamsForFacility.Items.Count)
        {
            tempPACTString = null; // all
        }
        else
        {
            foreach (var entry in pactTeamList)
            {
                tempPACTString += entry.Text + ",";
            }

            if (tempPACTString.EndsWith(","))
            {
                tempPACTString = tempPACTString.Substring(0, tempPACTString.Length - 1);
            }

            if (String.IsNullOrEmpty(tempPACTString))
                tempPACTString = null;
        }

        if (String.IsNullOrEmpty(tempPACTString))
        {
            sb.Append(" PACT TEAM: All |");
        }
        else
        {
            sb.Append(" PACT TEAM: " + tempPACTString + " |");
        }

        // Date Range - rcbDateRangeSelection
        //if (rcbDateRangeSelection.SelectedValue != null)
        //{
        //    if (rcbDateRangeSelection.SelectedValue == "custom")
        //    {
        //        sb.Append(" Date Range: (Custom) " + ((DateTime)rdpStartDate.SelectedDate).ToShortDateString() + "-" + ((DateTime)rdpEndDate.SelectedDate).ToShortDateString() + " |");
        //    }
        //    else
        //    {
        //        sb.Append(" Date Range: " + rcbDateRangeSelection.SelectedItem.Text + " |");
        //    }
        //}

        if (rcbDateRange.SelectedValue != null)
        {
            sb.Append(" Date Range: " + rcbDateRange.SelectedItem.Text + " |");
        }

        // Age Restriction - rddlAgeRestriction
        sb.Append(" Ages: " + ((String.IsNullOrEmpty(rddlAgeRestriction.SelectedValue) || rddlAgeRestriction.SelectedValue == "All") ? "All |" : rddlAgeRestriction.SelectedValue + " |"));

        // Mammogram Type: rddlMammogramType
        if (rddlMammogramType.SelectedValue != null)
        {
            sb.Append(" MMG Type: " + rddlMammogramType.SelectedItem.Text + " |");
        }

        // Patient Name / SSN - rtbPatientName
        if (!String.IsNullOrEmpty(rtbPatientName.Text))
        {
            sb.Append(" Patient Name/SSN: " + rtbPatientName.Text + " |");
        }

        // Gender - rcbGender
        if (String.IsNullOrEmpty(rcbGender.SelectedValue) || rcbGender.SelectedValue == "All")
        {
            sb.Append(" Gender: All |");
        }
        else
        {
            sb.Append(" Gender: " + rcbGender.SelectedValue + " |");
        }

        // Provider - rtbProviderName
        if (!String.IsNullOrEmpty(rtbProviderName.Text))
        {
            sb.Append(" Provider: " + rtbProviderName.Text + " |");
        }

        string result = sb.ToString();

        if (result.EndsWith("|"))
        {
            result = result.Substring(0, result.Length - 1);
        }

        return result;
    }

    #endregion

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Mammograms Due / Ordered Pending Report";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Methods

    protected void rgMammosDueOrderedPending_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        FillReportGrid();
    }

    protected void rgMammosDueOrderedPending_ItemCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
    {
    }

    protected void rgMammosDueOrderedPending_ItemCreated(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridFilteringItem && isExport)
            e.Item.Visible = false;

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridHeaderItem)
        {
            GridHeaderItem hItem = (GridHeaderItem)e.Item;
            hItem.Style["font-size"] = "12pt";
        }

        if (IsPDFExport && e.Item is Telerik.Web.UI.GridDataItem)
        {
            GridDataItem dataItem = (GridDataItem)e.Item;
            dataItem.Style["font-size"] = "9pt";
        }

        if (e.Item is GridHeaderItem && !isExport)
        {

            GridHeaderItem header = (GridHeaderItem)e.Item;

            //loops through all headercells, which is different than columns
            foreach (GridTableHeaderCell headerCell in header.Controls)
            {
                ImageButton img = new ImageButton();

                //if the headercell has controls, it is a column, not a columngroup
                if (headerCell.HasControls())
                {
                    //get the actual header text
                    string headerText = ((LiteralControl)headerCell.Controls[0]).Text;

                    //make sure it isn't the edit header
                    if (headerText != "&nbsp;")
                    {
                        try
                        {
                            img = setupImgButton(headerText, headerCell.HeaderID);

                            headerCell.Controls.Add(img);
                        }
                        catch (Exception ex)
                        {
                            Logging.WriteLogToFile(_userId, ex);
                        }

                        //loop through actual radgrid columns and compare to the header cells 
                        foreach (GridColumn col in rgMammosDueOrderedPending.Columns)
                        {
                            //test to see if there is currently a sort expression and the header cell matches the column
                            if (rgMammosDueOrderedPending.MasterTableView.SortExpressions.Count != 0
                                && (rgMammosDueOrderedPending.MasterTableView.SortExpressions[0].FieldName == col.UniqueName ||
                                    rgMammosDueOrderedPending.MasterTableView.SortExpressions[0].FieldName == col.SortExpression)
                                && headerText == col.HeaderText)
                            {
                                //check to see if the sort order isn't none
                                if (rgMammosDueOrderedPending.MasterTableView.SortExpressions[0].SortOrder != GridSortOrder.None)
                                {
                                    //if it isn't none, remove the no sort image
                                    try
                                    {
                                        headerCell.Controls.Remove(img);
                                    }
                                    catch (Exception ex)
                                    {
                                        Logging.WriteLogToFile(_userId, ex);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    protected void rgMammosDueOrderedPending_ItemDataBound(object sender, GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridHeaderItem)
            {
                //add access keys to header items for 508
                GridHeaderItem hi = (GridHeaderItem)e.Item;
            }

            if (e.Item is GridFilteringItem)
            {

            }

            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");
            }

            if (isExport && e.Item is GridDataItem)
            {
                foreach (TableCell cell in e.Item.Cells)
                {
                    //Approach 1
                    cell.Style["mso-number-format"] = @"\@";
                }
            }
        }
        catch (Exception ex)
        {
            //Logging.WriteLogToFile(_userId, ex);
        }

    }

    protected void rgMammosDueOrderedPending_SortCommand(object sender, GridSortCommandEventArgs e)
    {
        // return to page 1 of results
        rgMammosDueOrderedPending.CurrentPageIndex = 0;
        executeSortExpression(e.SortExpression, e.NewSortOrder);
    }

    /// <summary>
    /// Paging method to keep custom sorting intact.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgMammosDueOrderedPending_PageIndexChanged(object sender, GridPageChangedEventArgs e)
    {
        foreach (GridSortExpression sortE in rgMammosDueOrderedPending.MasterTableView.SortExpressions)
        {
            executeSortExpression(sortE.FieldName, sortE.SortOrder);
        }
    }

    ImageButton setupImgButton(string headerText, string headerID)
    {
        //create the sort button image
        ImageButton img = new ImageButton();

        img.ID = "Sort_" + headerID;
        img.AlternateText = "Sort " + headerText + " asc/desc";
        img.ImageUrl = "~/images/sort_none.gif";
        img.TabIndex = 25;
        img.ToolTip = img.AlternateText;

        switch (headerText)
        {
            case "PACT Team":
                img.AccessKey = "t";
                break;
            case "Provider Name":
                img.AccessKey = "p";
                break;
            case "Last Name, First":
                img.AccessKey = "n";
                break;
            case "Last 4":
                img.AccessKey = "s";
                break;
            case "Age":
                img.AccessKey = "a";
                break;
            case "Race":
                img.AccessKey = "r";
                break;
            case "Gender":
                img.AccessKey = "g";
                break;
            case "Patient Zip":
                img.AccessKey = "z";
                break;
            case "Date Performed":  // Mammogram
                img.AccessKey = "m";
                break;
            case "BI-RADS Code":
                img.AccessKey = "b";
                break;
            case "Procedure Location":
                img.AccessKey = "l";
                break;
            default:
                break;
        }

        return img;
    }

    #endregion

    #region Facility Dropdown Events

    protected void rcbVisn_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        clearNonFacilityFilters();

        ProcessVisnSelection();
    }

    protected void ProcessVisnSelection()
    {
        r.GetFacilitiesByUserId();

        // Station drop down clear existing items
        rcbFac.Items.Clear();
        rcbFac.Text = "";
        rcbFac.ClearSelection();

        if (rcbVisn.SelectedIndex > -1)
        {
            rcbFac.Enabled = true;

            DataColumn idColumn = new DataColumn();
            idColumn = new DataColumn();
            idColumn.DataType = System.Type.GetType("System.Int32");
            idColumn.ColumnName = "sta3n";
            idColumn.ReadOnly = true;
            idColumn.Unique = true;

            DataColumn nameColumn = new DataColumn();
            nameColumn = new DataColumn();
            nameColumn.DataType = System.Type.GetType("System.String");
            nameColumn.ColumnName = "facText";
            nameColumn.ReadOnly = true;
            nameColumn.Unique = true;

            DataTable facs = new DataTable();
            facs.Columns.Add(idColumn);
            facs.Columns.Add(nameColumn);

            DataRow dr;

            foreach (DataRow drFac in r.dtFac.Rows)
            {
                if (drFac["visnId"].ToString() == rcbVisn.SelectedValue)
                {
                    dr = facs.NewRow();
                    dr["sta3n"] = drFac["sta3n"];
                    dr["facText"] = drFac["facText"];

                    facs.Rows.Add(dr);
                }
            }

            rcbFac.DataSource = facs;
            rcbFac.DataTextField = "facText";
            rcbFac.DataValueField = "sta3n";
            rcbFac.DataBind();

            // If there's only one option, select it for them
            if (rcbFac.Items.Count == 1)
            {
                rcbFac.SelectedIndex = 0;
                //rcbFac.Enabled = false;
                btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;

                rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(rcbFac.SelectedValue));
                rcbPACTTeamsForFacility.DataBind();
                rcbPACTTeamsForFacility.Enabled = true;
            }

            rcbFac.Focus();
        }
        else
        {
            rcbFac.Enabled = false;
        }
    }

    protected void rcbFac_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbFac.SelectedIndex > -1)
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = true;
        }
        else
        {
            btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;
        }

        // Check to see if facility was selected.
        if (rcbFac.SelectedValue != null && !String.IsNullOrEmpty(rcbFac.SelectedValue))
        {
            // query for available PACT Teams for facility
            //rcbPACTTeamsForFacility.DataSource = 
            rcbPACTTeamsForFacility.DataSource = Reports.GetPACTTeamsBySta3n(Convert.ToInt32(rcbFac.SelectedValue));
            rcbPACTTeamsForFacility.DataBind();
            rcbPACTTeamsForFacility.Enabled = true;

            foreach (RadComboBoxItem itm in rcbPACTTeamsForFacility.Items)
            {
                itm.Checked = true;
            }

            rcbPACTTeamsForFacility.Focus();
        }
        else
        {
            // remove items
            rcbPACTTeamsForFacility.ClearSelection();
            rcbPACTTeamsForFacility.DataSource = "";
            rcbPACTTeamsForFacility.DataBind();

            rcbPACTTeamsForFacility.Enabled = true;
        }
    }

    #endregion

    #region GUI Event Methods

    /// <summary>
    /// Run report and call FillReportGrid
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnRunReport_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            rgMammosDueOrderedPending.CurrentPageIndex = 0;
            rgMammosDueOrderedPending.MasterTableView.SortExpressions.Clear();

            DateTime start = DateTime.Now;

            FillReportGrid();
            rgMammosDueOrderedPending.DataBind();

            DateTime end = DateTime.Now;
            lblExecutionTime.Text = Math.Round((end - start).TotalSeconds, 2).ToString() + " seconds";
        }
    }

    /// <summary>
    /// Removes all search criteria and clears grid.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnClearReport_Click(object sender, EventArgs e)
    {
        // clear search fields
        rcbFac.ClearSelection();
        rcbVisn.ClearSelection();
        rcbFac.Enabled = false;

        clearNonFacilityFilters();

        FillFACs();
        btnRunReportTop.Enabled = btnRunReportBottom.Enabled = false;

        rgMammosDueOrderedPending.MasterTableView.SortExpressions.Clear();

        rgMammosDueOrderedPending.DataSource = "";
        rgMammosDueOrderedPending.DataBind();

        lblExecutionTime.Text = "";
    }

    /// <summary>
    /// Export
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnExport_Click(object sender, EventArgs e)
    {
        isExport = true;

        #region Set Export Only Data

        switch (((Button)sender).CommandName)
        {
            case "XLS":
                rgMammosDueOrderedPending.ExportSettings.ExportOnlyData = true;
                break;
            case "CSV":
                rgMammosDueOrderedPending.ExportSettings.ExportOnlyData = false;
                break;
            case "PDF":
                rgMammosDueOrderedPending.ExportSettings.ExportOnlyData = true;
                break;
            case "DOC":
                rgMammosDueOrderedPending.ExportSettings.ExportOnlyData = true;
                break;
        }

        #endregion

        ConfigureExport();
        switch (((Button)sender).CommandName)
        {
            case "XLS":
                IsPDFExport = false;
                rgMammosDueOrderedPending.MasterTableView.ExportToExcel();
                break;
            case "CSV":
                IsPDFExport = false;
                rgMammosDueOrderedPending.MasterTableView.ExportToCSV();
                break;
            case "PDF":
                IsPDFExport = true;
                rgMammosDueOrderedPending.MasterTableView.ExportToPdf();
                break;
            case "DOC":
                IsPDFExport = false;
                rgMammosDueOrderedPending.MasterTableView.ExportToWord();
                break;
        }
    }

    /// <summary>
    /// Inspect what value was selected in the date range selection ... if custom, show date range pickers, else hide.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbDateRange_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        if (rcbDateRange.SelectedValue == "custom")
        {
            // show date range pickers
            pnlCustomDateRanges.Visible = true;
            rfvStartDate.Enabled = rfvEndDate.Enabled = true;
        }
        else
        {
            // hide date range pickers
            pnlCustomDateRanges.Visible = false;
            rfvStartDate.Enabled = rfvEndDate.Enabled = false;
        }
    }

    /// <summary>
    /// Setup the Export
    /// </summary>
    protected void ConfigureExport()
    {
        rgMammosDueOrderedPending.MasterTableView.AllowPaging = false;
        rgMammosDueOrderedPending.Rebind();

        StringBuilder reportHeaderString = new StringBuilder();
        reportHeaderString.Append("<div style=\"width:100%; font-size: 15pt; clear:both; text-align:left;\">");
        reportHeaderString.Append(hfCustomReportName.Value);
        reportHeaderString.Append("</div>");
        reportHeaderString.Append("<div style=\"width:100%; font-size: 12pt; clear:both; text-align:left;\">");
        reportHeaderString.Append("Exported:" + DateTime.Now.ToString() + " - Total Number of Records: " + rgMammosDueOrderedPending.MasterTableView.Items.Count.ToString() + "<br />");
        reportHeaderString.Append("Filters: " + getCurrentFiltersAndValues() + "<br />");
        reportHeaderString.Append("</div>");

        rgMammosDueOrderedPending.ExportSettings.Pdf.PageTitle = "Filters: " + getCurrentFiltersAndValues();
        rgMammosDueOrderedPending.ExportSettings.Pdf.DefaultFontFamily = "Arial Unicode MS";
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageFooter.LeftCell.Text = hfCustomReportName.Value;
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageFooter.MiddleCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Center;
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageFooter.MiddleCell.Text = "Exported:" + DateTime.Now.ToString();
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageFooter.RightCell.TextAlign = GridPdfPageHeaderFooterCell.CellTextAlign.Right;
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageFooter.RightCell.Text = "Total Number of Records: " + rgMammosDueOrderedPending.MasterTableView.Items.Count.ToString();
        rgMammosDueOrderedPending.ExportSettings.Pdf.ForceTextWrap = true;

        rgMammosDueOrderedPending.MasterTableView.Caption = reportHeaderString.ToString();
        rgMammosDueOrderedPending.Rebind();
        rgMammosDueOrderedPending.ExportSettings.FileName = hfCustomReportName.Value;
        rgMammosDueOrderedPending.ExportSettings.IgnorePaging = true;
        rgMammosDueOrderedPending.ExportSettings.OpenInNewWindow = true;
        rgMammosDueOrderedPending.ExportSettings.HideStructureColumns = true;

        rgMammosDueOrderedPending.ExportSettings.Pdf.PageHeight = Unit.Parse("210mm");
        rgMammosDueOrderedPending.ExportSettings.Pdf.PageWidth = Unit.Parse("400mm");

        foreach (GridColumn gc in rgMammosDueOrderedPending.MasterTableView.Columns)
        {
            gc.ColumnGroupName = null;
        }

        foreach (GridColumn col in rgMammosDueOrderedPending.MasterTableView.RenderColumns)
        {
            if (col.UniqueName.EndsWith("_Report"))
                col.Visible = true;
            else if (col.UniqueName.EndsWith("_GUI"))
                col.Visible = false;
        }
        rgMammosDueOrderedPending.Rebind();
    }

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score, object mammogramSource)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1")
            {
                returnVal = "No Score - Source: " + (mammogramSource != null ? mammogramSource.ToString() : "");
            }
            else if (String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    #endregion
}