﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_PatientDetails : BasePage
{
    #region Member Variables

    private string _userId;
    bool isExport = false;
    private PatientData patData;
    private PatientDetails_Patient patientInformation = new PatientDetails_Patient();

    protected string patientSID;

    private List<PatientDetails_Chemotherapy> chemoTherapyList = new List<PatientDetails_Chemotherapy>();
    private List<PatientDetails_RadiationTherapy> radTherapyList = new List<PatientDetails_RadiationTherapy>();
    private List<PatientDetails_HormoneTherapy> hormonalTherapyList = new List<PatientDetails_HormoneTherapy>();

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            LabelBreadCrumb.Text = "Patient Details";
            UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;

            //set the user ID variable
            patData = new PatientData();

            if (!String.IsNullOrEmpty(Request.QueryString.Get("id")))
            {
                // TODO: Check if user can see patient
                patientSID = Request.QueryString.Get("id");

                // Load Data for Patient ID
                loadPatientDetails(patientSID);
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// Helper method for returning the BIRAD score.
    /// </summary>
    /// <param name="score"></param>
    /// <param name="mammogramSource"></param>
    /// <returns></returns>
    protected string GetBiradScore(object score)
    {
        string returnVal = "";

        if (score != null)
        {
            if (score.ToString() == "-1" || String.IsNullOrEmpty(score.ToString()))
            {
                returnVal = "No Score";
            }
            else
            {
                returnVal = score.ToString();
            }
        }
        else
        {
            returnVal = "No Score";
        }

        return returnVal;
    }

    private void loadPatientDetails(string _patientSid)
    {
        DataSet ds = patData.GetPatientDetails(_patientSid);

        if (ds.Tables[0] != null)
        {
            mapPatientDataTable(ds.Tables[0]);
        }

        // Diagnosis
        if (ds.Tables[1] != null)
        {
            rgDiagnosisHx.DataSource = ds.Tables[1];
            rgDiagnosisHx.DataBind();
        }

        // Tumor Reg Data / ONC
        if (ds.Tables[2] != null)
        {
            rgTumorRegData.DataSource = ds.Tables[2];
            rgTumorRegData.DataBind();

            rgChemotherapy.DataSource = ds.Tables[2];
            rgChemotherapy.DataBind();

            rgRadiationTherapy.DataSource = ds.Tables[2];
            rgRadiationTherapy.DataBind();

            rgHormoneTherapy.DataSource = ds.Tables[2];
            rgHormoneTherapy.DataBind();
        }

        // Radiology Data
        if (ds.Tables[3] != null)
        {
            rgHistory.DataSource = ds.Tables[3];
            rgHistory.DataBind();
        }

        // Biopsies / Surgeries
        if (ds.Tables[4] != null)
        {
            rgBiopsySurgery.DataSource = ds.Tables[4];
            rgBiopsySurgery.DataBind();
        }

        // Co-Morbidities
        if (ds.Tables[5] != null)
        {
            rgCoMorbidities.DataSource = ds.Tables[5];
            rgCoMorbidities.DataBind();
        }

        // Orders
        if (ds.Tables[6] != null)
        {
            rgOrders.DataSource = ds.Tables[6];
            rgOrders.DataBind();
        }

        // Medications
        if (ds.Tables[7] != null)
        {
            rgMedications.DataSource = ds.Tables[7];
            rgMedications.DataBind();
        }

        // Appointments
        if (ds.Tables[8] != null)
        {
            //mapPatientAppointments(ds.Tables[8]);
            rgAppointments.DataSource = ds.Tables[8];
            rgAppointments.DataBind();
        }

        //// Oncology
        //if (ds.Tables[4] != null)
        //{
        //    mapOncologyDataTable(ds.Tables[4]);
        //}

        lblPatientName.Text = patientInformation.PatientLastName + ", " + patientInformation.PatientFirstName;
        lblPatientSSN.Text = "XXX-XX-" + patientInformation.PatientSSN.Substring(patientInformation.PatientSSN.Length - 4);
        lblPatientAge.Text = patientInformation.Age;
        lblPatientGender.Text = patientInformation.Gender;
        lblPatientRace.Text = patientInformation.Race;
        lblIsVeteran.Text = patientInformation.IsVeteran;

        lblHomePhone.Text = !String.IsNullOrEmpty(patientInformation.PhoneResidence) ? patientInformation.PhoneResidence : "--";
        lblWorkPhone.Text = !String.IsNullOrEmpty(patientInformation.PhoneWork) ? patientInformation.PhoneWork : "--";
        lblCellPhone.Text = !String.IsNullOrEmpty(patientInformation.PhoneCellular) ? patientInformation.PhoneCellular : "--";

        generateAddressInformationBlock(); // get address block

        lblTeamName.Text = (String.IsNullOrEmpty(patientInformation.PCMMTeam) ? "None Available" : patientInformation.PCMMTeam);
        lblProviderName.Text = (String.IsNullOrEmpty(patientInformation.PCMMStaffName) ? "None Available" : patientInformation.PCMMStaffName);

        //lblDateOfNextAppointment.Text = (String.IsNullOrEmpty(patientInformation.DateOfNextAppointment) ? "None Available" : patientInformation.DateOfNextAppointment);
        //lblLocationOfNextAppointment.Text = (String.IsNullOrEmpty(patientInformation.LocationNameOfNextAppointment) ? "None Available" : patientInformation.LocationNameOfNextAppointment);

        lblMMGCompliance.Text = (String.IsNullOrEmpty(patientInformation.MammogramScreeningComplianceText) ? "None Available" : patientInformation.MammogramScreeningComplianceText);
        lblDateLastMMG.Text = (String.IsNullOrEmpty(patientInformation.DateOFMostRecentMammogram) ? "None Available" : patientInformation.DateOFMostRecentMammogram);
        lblLastMMGScore.Text = (String.IsNullOrEmpty(patientInformation.BIRADScore) || patientInformation.BIRADScore == "-1" ? "No Score - Source: " + patientInformation.MostRecentMammogramSource : patientInformation.BIRADScore);
        lblDateNextMMGDue.Text = (String.IsNullOrEmpty(patientInformation.DateNextMammogramDue) ? "None Available" : patientInformation.DateNextMammogramDue);

        lblHadBilateralMast.Text = (String.IsNullOrEmpty(patientInformation.HadBilateralMastectomy) ? "None Available" : ((patientInformation.HadBilateralMastectomy == "True") ? "Yes" : "No"));
        lblHasATerminalHealthFactor.Text = (String.IsNullOrEmpty(patientInformation.HasATerminalHealthFactor) ? "None Available" : ((patientInformation.HasATerminalHealthFactor == "True") ? "Yes" : "No"));
        lblHasMammogramExclusionHealthFactor.Text = (String.IsNullOrEmpty(patientInformation.HasMammogramExclusionHealthFactor) ? "None Available" : ((patientInformation.HasMammogramExclusionHealthFactor == "True") ? "Yes " + patientInformation.DateOfMostRecentMammogramExclusionHealthFactor : "No"));
        lblMammogramScreenFreqHealthFactor.Text = (String.IsNullOrEmpty(patientInformation.MammogramMostRecentScreenFreq) ? "None Available" : patientInformation.MammogramMostRecentScreenFreq);
        lblDateOfMammogramScreenFreqHealthFactor.Text = (String.IsNullOrEmpty(patientInformation.DateOfMostRecentMammogramScreenFreq) ? "None Available" : patientInformation.DateOfMostRecentMammogramScreenFreq);

        // Treatment Lists
        //rgChemotherapy.DataSource = chemoTherapyList;
        //rgChemotherapy.DataBind();

        //rgRadiationTherapy.DataSource = radTherapyList;
        //rgRadiationTherapy.DataBind();

        //rgHormoneTherapy.DataSource = hormonalTherapyList;
        //rgHormoneTherapy.DataBind();
    }

    private void mapPatientDataTable(DataTable dt)
    {
        DataRow dr = dt.Rows[0];

        if (dr != null)
        {
            patientInformation.PatientSID = dr["PatientSID"].ToString();
            patientInformation.PatientICN = dr["PatientICN"].ToString();
            patientInformation.PatientIEN = dr["PatientIEN"].ToString();
            patientInformation.PatientSSN = dr["PatientSSN"].ToString();
            patientInformation.PatientLastName = dr["PatientLastName"].ToString();
            patientInformation.PatientFirstName = dr["PatientFirstName"].ToString();
            patientInformation.Sta3n = dr["Sta3n"].ToString();
            patientInformation.DateOfBirth = dr["DateOfBirth"].ToString();
            patientInformation.Age = dr["Age"].ToString();
            patientInformation.Gender = dr["Gender"].ToString();
            patientInformation.Race = dr["Race"].ToString();

            patientInformation.StreetAddress1 = dr["StreetAddress1"].ToString();
            patientInformation.StreetAddress2 = dr["StreetAddress2"].ToString();
            patientInformation.StreetAddress3 = dr["StreetAddress3"].ToString();
            patientInformation.City = dr["City"].ToString();
            patientInformation.State = dr["State"].ToString();

            patientInformation.Zip = dr["Zip"].ToString();
            patientInformation.PhoneResidence = dr["PhoneResidence"].ToString();
            patientInformation.PhoneWork = dr["PhoneWork"].ToString();
            patientInformation.PhoneCellular = dr["PhoneCellular"].ToString();
            patientInformation.ISOEFOIF = dr["ISOEFOIF"].ToString();
            patientInformation.PCMMTeam = dr["PCMMTeam"].ToString();
            patientInformation.PCMMStaffName = dr["PCMMStaffName"].ToString();
            //patientInformation.DateOfNextAppointment = dr["DateOfNextAppointment"].ToString();
            //patientInformation.LocationNameOfNextAppointment = dr["LocationNameOfNextAppointment"].ToString();

            patientInformation.HadBilateralMastectomy = dr["HadBilateralMastectomy"].ToString();
            patientInformation.HasATerminalHealthFactor = dr["HasATerminalHealthFactor"].ToString();
            patientInformation.HasMammogramExclusionHealthFactor = dr["HasMammogramExclusionHealthFactor"].ToString();
            patientInformation.DateOfMostRecentMammogramExclusionHealthFactor = dr["DateOfMostRecentMammogramExclusionHealthFactor"].ToString();
            
            patientInformation.MammogramMostRecentScreenFreq = dr["MammogramMostRecentScreenFreq"].ToString();
            patientInformation.DateOfMostRecentMammogramScreenFreq = dr["DateOfMostRecentMammogramScreenFreq"].ToString();
            
            patientInformation.DateOFMostRecentMammogram = dr["DateOFMostRecentMammogram"].ToString();
            patientInformation.MostRecentMammogramSource = dr["MostRecentMammogramSource"].ToString();
            patientInformation.BIRADScore = dr["BIRADScore"].ToString();
            patientInformation.DateNextMammogramDue = dr["DateNextMammogramDue"].ToString();
            patientInformation.MammogramScreeningComplianceText = dr["MammogramScreeningComplianceText"].ToString();

            patientInformation.IsVeteran = dr["IsVeteran"].ToString();
        }
    }

    //private void mapPatientOrdersTable(DataTable dt)
    //{
    //    StringBuilder sb = new StringBuilder();

    //    if (dt.Rows.Count > 0)
    //    {
    //        foreach (DataRow dr in dt.Rows)
    //        {
    //            sb.Append("Desired Date:<br />" + dr["DesiredDate"].ToString());
    //            sb.Append("Ordering Provider:<br />" + dr["Requesting_Provider"].ToString());
    //            sb.Append(dr["OrderableItemCodeSource"].ToString() + " Code:<br />" + dr["OrderableItemCode"].ToString());
    //            sb.Append("Procedure Description:<br />" + dr["CPTDescription"].ToString());
    //            sb.Append("Status:<br />" + dr["OrderStatus"].ToString() + "<br />");
    //            sb.Append("<hr />");
    //        }
    //    }

    //    lblUpcomingOrders.Text = sb.ToString();
    //}

    private void mapOncologyDataTable(DataTable dt)
    {
        foreach (DataRow dr in dt.Rows)
        {
            #region Chemotherapy
            PatientDetails_Chemotherapy chemT = new PatientDetails_Chemotherapy();

            chemT.chemotherapy_date = dr["chemotherapy_date"] as DateTime?;
            chemT.chemotherapeuticagent1 = dr["chemotherapeuticagent1"].ToString();
            chemT.chemotherapeuticagent2 = dr["chemotherapeuticagent2"].ToString();
            chemT.chemotherapeuticagent3 = dr["chemotherapeuticagent3"].ToString();
            chemT.chemotherapeuticagent4 = dr["chemotherapeuticagent4"].ToString();
            chemT.chemotherapeuticagent5 = dr["chemotherapeuticagent5"].ToString();
            chemT.chemotherapy = dr["chemotherapy"].ToString();
            chemT.chemotherapy_fac = dr["chemotherapy_fac"].ToString();
            chemT.chemotherapyhospitalx = dr["chemotherapyhospitalx"].ToString();

            if (chemT.chemotherapy_date != null)
            {
                // only add if not null and greater than 1900 - gets rid of date min.
                if (((DateTime)chemT.chemotherapy_date).Year > 1900)
                {
                    chemoTherapyList.Add(chemT);
                }
            }

            #endregion

            #region Radiation Therapy

            PatientDetails_RadiationTherapy radT = new PatientDetails_RadiationTherapy();
            radT.date_radiation_started = dr["date_radiation_started"] as DateTime?;
            radT.intentofradiation = dr["intentofradiation"].ToString();
            radT.locationofradiationtx = dr["locationofradiationtx"].ToString();
            radT.radiation = dr["radiation"].ToString();
            //radT.radiation_facility = dr["radiation_facility"].ToString();
            //radT.radiation_facility_date = dr["radiation_facility_date"] as DateTime?;
            radT.radiation_therapy_to_cns = dr["radiation_therapy_to_cns"].ToString();
            //radT.radiation_therapy_to_cns_date = dr["radiation_therapy_to_cns_date"] as DateTime?;
            radT.radiationcompletionstatus = dr["radiationcompletionstatus"].ToString();
            radT.radiationsurgery_sequence = dr["radiationsurgery_sequence"].ToString();

            radTherapyList.Add(radT);

            #endregion

            #region Hormonal Therapy

            PatientDetails_HormoneTherapy hormoneT = new PatientDetails_HormoneTherapy();
            hormoneT.hormone_therapy = dr["hormone_therapy"].ToString();
            hormoneT.hormone_therapy_date = dr["hormone_therapy_date"] as DateTime?;
            hormoneT.hormonetherapyhospitalx = dr["hormonetherapyhospitalx"].ToString();
            hormoneT.rxtexthormonewp = dr["rxtexthormonewp"].ToString();

            if (hormoneT.hormone_therapy_date != null)
            {
                hormonalTherapyList.Add(hormoneT);
            }

            #endregion
        }
    }

    /// <summary>
    /// Determines which address variables are present and builds a formatted block.
    /// </summary>
    private void generateAddressInformationBlock()
    {
        StringBuilder sb = new StringBuilder();

        if (!String.IsNullOrEmpty(patientInformation.StreetAddress1))
        {
            sb.Append(patientInformation.StreetAddress1);
        }

        if (!String.IsNullOrEmpty(patientInformation.StreetAddress2))
        {
            if (!String.IsNullOrEmpty(sb.ToString()))
            {
                sb.Append("<br />");
            }

            sb.Append(patientInformation.StreetAddress2);
        }

        if (!String.IsNullOrEmpty(patientInformation.StreetAddress3))
        {
            if (!String.IsNullOrEmpty(sb.ToString()))
            {
                sb.Append("<br />");
            }

            sb.Append(patientInformation.StreetAddress3);
        }

        if (!String.IsNullOrEmpty(sb.ToString()))
        {
            sb.Append("<br />");
        }

        sb.Append(patientInformation.City + ", " + patientInformation.State + " " + patientInformation.Zip);
                
        lblPatientAddressInformation.Text = sb.ToString();
    }

    /// <summary>
    /// Radiology Grid Data Bound
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgHistory_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
        try
        {
            if (e.Item is GridDataItem)
            {
                //add tool tip to edit row image for 508
                GridDataItem item = (GridDataItem)e.Item;
                //add access keys to row items for 508
                item.Attributes.Add("scope", "row");

                string[] fieldNames = { "reporttextwp", "impressiontextwp", "NotificationStatement", "FollowUpActionStatement" };
                string reportName = "BB_";

                if (isExport)
                {
                    string columnName;
                    TableCell ctc;

                    // Inspect link fields and format accordingly for the output report

                    foreach (string columnRoot in fieldNames)
                    {
                        item = (GridDataItem)e.Item;
                        columnName = reportName + columnRoot + "_Report";
                        ctc = item[columnName];
                        Helpers.configureLinkForExport(ctc, columnRoot, reportName, "");
                    }

                    // Format cells
                    foreach (TableCell cell in e.Item.Cells)
                    {
                        cell.Style["mso-number-format"] = @"\@";
                    }
                }
                else // Not exporting, just displaying grid
                {
                    Label lbl;
                    string sData;
                    RadToolTip rtt;

                    // Inspect each link field and create links or appropriate message

                    foreach (string columnRoot in fieldNames)
                    {

                        lbl = (Label)item.FindControl("lbl" + reportName + columnRoot);
                        sData = ((System.Data.DataRowView)e.Item.DataItem)[columnRoot].ToString();
                        rtt = (RadToolTip)item.FindControl("rtt" + reportName + columnRoot);

                        Helpers.setupLink(lbl, sData, rtt, columnRoot, reportName);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "Patient Details";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    protected void rbExportToExcel_Click(object sender, EventArgs e)
    {
        //
        Response.Redirect("Export/ExportPatientDetails.aspx?id=" + patientSID + "&mode=excel");
    }

    protected void rbExportToWord_Click(object sender, EventArgs e)
    {
        Response.Redirect("Export/ExportPatientDetails.aspx?id=" + patientSID + "&mode=word");
    }
}

/// <summary>
/// Inner Class for Holding Patient data
/// </summary>
public class PatientDetails_Patient
{
    #region Member Variables

    public string PatientSID { get; set; }
    public string PatientICN { get; set; }
    public string PatientIEN { get; set; }
    public string PatientSSN { get; set; }
    public string PatientLastName { get; set; }
    public string PatientFirstName { get; set; }
    public string Sta3n { get; set; }
    public string DateOfBirth { get; set; }
    public string Age { get; set; }
    public string Gender { get; set; }
    public string Race { get; set; }
    public string StreetAddress1 { get; set; }
    public string StreetAddress2 { get; set; }
    public string StreetAddress3 { get; set; }
    public string City { get; set; }
    public string State { get; set; }
    public string Zip { get; set; }
    public string PhoneResidence { get; set; }
    public string PhoneWork { get; set; }
    public string PhoneCellular { get; set; }
    public string ISOEFOIF { get; set; }
    public string PCMMTeam { get; set; }
    public string PCMMStaffName { get; set; }
    public string DateOfNextAppointment { get; set; }
    public string LocationNameOfNextAppointment { get; set; }
    public string MammogramScreeningComplianceText { get; set; }
    public string HadBilateralMastectomy { get; set; }
    public string HasATerminalHealthFactor { get; set; }
    public string HasMammogramExclusionHealthFactor { get; set; }
    public string DateOfMostRecentMammogramExclusionHealthFactor { get; set; }
    public string DateOFMostRecentMammogram { get; set; }
    public string MostRecentMammogramSource { get; set; }
    public string BIRADScore { get; set; }
    public string DateNextMammogramDue { get; set; }
    public string MammogramMostRecentScreenFreq { get; set; }
    public string DateOfMostRecentMammogramScreenFreq { get; set; }
    public string IsVeteran { get; set; }

    #endregion

    /// <summary>
    /// Default Constructor
    /// </summary>
    public PatientDetails_Patient()
    {
    }
}

public class PatientDetails_Chemotherapy
{
    public DateTime? chemotherapy_date { get; set; }
    public string chemotherapyhospitalx { get; set; }
    public string chemotherapy { get; set; }
    public string chemotherapy_fac { get; set; }
    public string chemotherapeuticagent1 { get; set; }
    public string chemotherapeuticagent2 { get; set; }
    public string chemotherapeuticagent3 { get; set; }
    public string chemotherapeuticagent4 { get; set; }
    public string chemotherapeuticagent5 { get; set; }

    /// <summary>
    /// Default Constructor
    /// </summary>
    public PatientDetails_Chemotherapy() { }
}

public class PatientDetails_RadiationTherapy
{
    public DateTime? date_radiation_started { get; set; }
    public string radiation { get; set; }
    public string radiationsurgery_sequence { get; set; }
    //public string radiation_facility { get; set; }
    //public DateTime? radiation_facility_date { get; set; }
    //public DateTime? radiation_therapy_to_cns_date { get; set; }
    public string radiation_therapy_to_cns { get; set; }
    public string locationofradiationtx { get; set; } // Location of Rad Treatment
    public string intentofradiation { get; set; }
    public string radiationcompletionstatus { get; set; }

    /// <summary>
    /// Default Constructor
    /// </summary>
    public PatientDetails_RadiationTherapy() { }
}

public class PatientDetails_HormoneTherapy
{
    public DateTime? hormone_therapy_date { get; set; }
    public string hormonetherapyhospitalx { get; set; }
    public string hormone_therapy { get; set; }
    public string rxtexthormonewp { get; set; }

    /// <summary>
    /// Default Constructor
    /// </summary>
    public PatientDetails_HormoneTherapy()
    {
    }
}

public class PatientDetails_Medications
{
    /// <summary>
    /// Default Constructor
    /// </summary>
    public PatientDetails_Medications() { }
}