﻿using System;
using System.Data;
using System.Text;
using System.Web;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Reports_PatientSearchResults : BasePage
{
    #region Member Variables

    private string _userId;
    private Reports r;
    private PatientData p;
    private string _searchTerms;
    private string _facilitySta3n;

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;
        LabelBreadCrumb.Text = "<a href=\"Dashboard.aspx\" title=\"Navigate to Reports Dashboard\" style=\"color:#FFFFFF;\">Reports</a> > Search Results";

        try
        {
            //set the user ID variable
            _userId = _user.UserId.ToString();
            p = new PatientData();

            if (!IsPostBack)
            {
                rgSearchResults.DataSource = "";
                rgSearchResults.DataBind();
            }

            fillSearchResults();
            rgSearchResults.DataBind();
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(_userId, ex);
        }
    }

    /// <summary>
    /// fillSearchResults: handles calling the stored procedure and filling the data grid.
    /// </summary>
    private void fillSearchResults()
    {
        if (!String.IsNullOrEmpty(Request.QueryString.Get("searchTerms")))
        {
            _searchTerms = Server.UrlDecode(Request.QueryString.Get("searchTerms"));
            _facilitySta3n = Request.QueryString.Get("sta3n");

            r = new Reports(_userId);

            StringBuilder sb = new StringBuilder();

            if (r.GetFacilitiesByUserId())
            {
                foreach (DataRow dr in r.dtFac.Rows)
                {
                    // get value
                    if (dr["sta3n"] != null)
                    {
                        if (!String.IsNullOrEmpty(dr["sta3n"].ToString()))
                        {
                            sb.Append(dr["sta3n"].ToString() + ",");
                        }
                    }
                }

                string facStr = sb.ToString();

                if (!String.IsNullOrEmpty(facStr))
                {
                    if (facStr.EndsWith(","))
                    {
                        // remove last ,
                        facStr = facStr.Substring(0, facStr.Length - 1);
                    }
                }

                // add in check to only search Facilities that a user has access to ... 
                DataTable searchResults = p.GetPatientSearchResults(_searchTerms, facStr, (String.IsNullOrEmpty(_facilitySta3n) ? null : _facilitySta3n));
                rgSearchResults.DataSource = searchResults;
            }
            else
            {
                // should never get here ... no facilities

                //rgSearchResults

                rgSearchResults.DataSource = "";
            }
        }
    }

    #region GUI Overides
    //these need to be added to MenuBuilder.cs BuildDynamicNavigationItems method
    public override string TopMenuName()
    {
        return "Reporting";
    }

    public override string MenuItemName()
    {
        return "BCR Patient Search Results";
    }

    public override string AppObjectCode()
    {
        return "BCR_STANDARD_REPORTS";
    }
    #endregion

    #region Grid Events

    /// <summary>
    /// Called when grid needs data source.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgSearchResults_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        fillSearchResults();
    }

    /// <summary>
    /// Redirect to Patient Details upon selecting a patient.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgSearchResults_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int patientSID = (int)(e.Item as GridDataItem).GetDataKeyValue("PatientSID");
            Response.Redirect(string.Format("PatientDetails.aspx?id={0}", patientSID));
        }
    }

    #endregion
}